@@ -15,11 +15,9 @@
 #ifndef GRPC_SRC_CORE_LIB_TRANSPORT_CALL_SPINE_H
 #define GRPC_SRC_CORE_LIB_TRANSPORT_CALL_SPINE_H
 
-#include "absl/log/check.h"
-
 #include <grpc/support/port_platform.h>
 
-#include "src/core/lib/gprpp/dual_ref_counted.h"
+#include "absl/log/check.h"
 #include "src/core/lib/promise/detail/status.h"
 #include "src/core/lib/promise/if.h"
 #include "src/core/lib/promise/latch.h"
@@ -33,6 +31,7 @@
 #include "src/core/lib/transport/call_filters.h"
 #include "src/core/lib/transport/message.h"
 #include "src/core/lib/transport/metadata.h"
+#include "src/core/util/dual_ref_counted.h"
 
 namespace grpc_core {
 
@@ -54,17 +53,23 @@ class CallSpine final : public Party {
 
   CallFilters& call_filters() { return call_filters_; }
 
-  // Add a callback to be called when server trailing metadata is received.
-  void OnDone(absl::AnyInvocable<void(bool)> fn) {
+  // Add a callback to be called when server trailing metadata is received and
+  // return true.
+  // If CallOnDone has already been invoked, does nothing and returns false.
+  GRPC_MUST_USE_RESULT bool OnDone(absl::AnyInvocable<void(bool)> fn) {
+    if (call_filters().WasServerTrailingMetadataPulled()) {
+      return false;
+    }
     if (on_done_ == nullptr) {
       on_done_ = std::move(fn);
-      return;
+      return true;
     }
     on_done_ = [first = std::move(fn),
                 next = std::move(on_done_)](bool cancelled) mutable {
       first(cancelled);
       next(cancelled);
     };
+    return true;
   }
   void CallOnDone(bool cancelled) {
     if (on_done_ != nullptr) std::exchange(on_done_, nullptr)(cancelled);
@@ -100,6 +105,8 @@ class CallSpine final : public Party {
   }
 
   void PushServerTrailingMetadata(ServerMetadataHandle md) {
+    GRPC_TRACE_LOG(call_state, INFO)
+        << "[call_state] PushServerTrailingMetadata: " << md->DebugString();
     call_filters().PushServerTrailingMetadata(std::move(md));
   }
 
@@ -128,15 +135,20 @@ class CallSpine final : public Party {
     using P = promise_detail::PromiseLike<Promise>;
     using ResultType = typename P::Result;
     return Map(std::move(promise), [this](ResultType r) {
-      if (!IsStatusOk(r)) {
-        auto md = StatusCast<ServerMetadataHandle>(r);
-        md->Set(GrpcCallWasCancelled(), true);
-        PushServerTrailingMetadata(std::move(md));
-      }
+      CancelIfFailed(r);
       return r;
     });
   }
 
+  template <typename StatusType>
+  void CancelIfFailed(const StatusType& r) {
+    if (!IsStatusOk(r)) {
+      Cancel();
+    }
+  }
+
+  void Cancel() { call_filters().Cancel(); }
+
   // Spawn a promise that returns Empty{} and save some boilerplate handling
   // that detail.
   template <typename PromiseFactory>
@@ -186,6 +198,75 @@ class CallSpine final : public Party {
     });
   }
 
+  // Spawned operations: these are callable from /outside/ the call; they spawn
+  // an operation into the call and execute that operation.
+
+  void SpawnPushServerInitialMetadata(ServerMetadataHandle md) {
+    SpawnInfallible(
+        "push-server-initial-metadata",
+        [md = std::move(md), self = RefAsSubclass<CallSpine>()]() mutable {
+          self->CancelIfFailed(self->PushServerInitialMetadata(std::move(md)));
+        });
+  }
+
+  auto SpawnPushServerToClientMessage(MessageHandle msg) {
+    return SpawnWaitable(
+        "push-message",
+        [msg = std::move(msg), self = RefAsSubclass<CallSpine>()]() mutable {
+          return self->CancelIfFails(
+              self->PushServerToClientMessage(std::move(msg)));
+        });
+  }
+
+  auto SpawnPushClientToServerMessage(MessageHandle msg) {
+    return SpawnWaitable(
+        "push-message",
+        [msg = std::move(msg), self = RefAsSubclass<CallSpine>()]() mutable {
+          return self->CancelIfFails(
+              self->PushClientToServerMessage(std::move(msg)));
+        });
+  }
+
+  void SpawnFinishSends() {
+    SpawnInfallible("finish-sends", [self = RefAsSubclass<CallSpine>()]() {
+      self->FinishSends();
+      return Empty{};
+    });
+  }
+
+  void SpawnPushServerTrailingMetadata(ServerMetadataHandle md) {
+    SpawnInfallible(
+        "push-server-trailing-metadata",
+        [md = std::move(md), self = RefAsSubclass<CallSpine>()]() mutable {
+          self->PushServerTrailingMetadata(std::move(md));
+          return Empty{};
+        });
+  }
+
+  void SpawnCancel() {
+    SpawnInfallible("cancel", [self = RefAsSubclass<CallSpine>()]() {
+      self->call_filters().Cancel();
+    });
+  }
+
+  void AddChildCall(RefCountedPtr<CallSpine> child_call) {
+    child_calls_.push_back(std::move(child_call));
+    if (child_calls_.size() == 1) {
+      SpawnInfallible(
+          "check_cancellation", [self = RefAsSubclass<CallSpine>()]() mutable {
+            auto was_completed =
+                self->call_filters().ServerTrailingMetadataWasPushed();
+            return Map(std::move(was_completed),
+                       [self = std::move(self)](Empty) {
+                         for (auto& child : self->child_calls_) {
+                           child->SpawnCancel();
+                         }
+                         return Empty{};
+                       });
+          });
+    }
+  }
+
  private:
   friend class Arena;
   CallSpine(ClientMetadataHandle client_initial_metadata,
@@ -196,10 +277,16 @@ class CallSpine final : public Party {
   // Call filters/pipes part of the spine
   CallFilters call_filters_;
   absl::AnyInvocable<void(bool)> on_done_{nullptr};
+  // Call spines that should be cancelled if this spine is cancelled
+  absl::InlinedVector<RefCountedPtr<CallSpine>, 3> child_calls_;
 };
 
+class CallHandler;
+
 class CallInitiator {
  public:
+  using NextMessage = ServerToClientNextMessage;
+
   CallInitiator() = default;
   explicit CallInitiator(RefCountedPtr<CallSpine> spine)
       : spine_(std::move(spine)) {}
@@ -217,23 +304,40 @@ class CallInitiator {
     return spine_->PushClientToServerMessage(std::move(message));
   }
 
+  auto SpawnPushMessage(MessageHandle message) {
+    return spine_->SpawnPushClientToServerMessage(std::move(message));
+  }
+
   void FinishSends() { spine_->FinishSends(); }
 
+  void SpawnFinishSends() { spine_->SpawnFinishSends(); }
+
   auto PullMessage() { return spine_->PullServerToClientMessage(); }
 
   auto PullServerTrailingMetadata() {
     return spine_->PullServerTrailingMetadata();
   }
 
-  void Cancel(absl::Status error = absl::CancelledError()) {
+  void Cancel(absl::Status error) {
     CHECK(!error.ok());
     auto status = ServerMetadataFromStatus(error);
     status->Set(GrpcCallWasCancelled(), true);
     spine_->PushServerTrailingMetadata(std::move(status));
   }
 
-  void OnDone(absl::AnyInvocable<void(bool)> fn) {
-    spine_->OnDone(std::move(fn));
+  void SpawnCancel(absl::Status error) {
+    CHECK(!error.ok());
+    auto status = ServerMetadataFromStatus(error);
+    status->Set(GrpcCallWasCancelled(), true);
+    spine_->SpawnPushServerTrailingMetadata(std::move(status));
+  }
+
+  void Cancel() { spine_->Cancel(); }
+
+  void SpawnCancel() { spine_->SpawnCancel(); }
+
+  GRPC_MUST_USE_RESULT bool OnDone(absl::AnyInvocable<void(bool)> fn) {
+    return spine_->OnDone(std::move(fn));
   }
 
   template <typename PromiseFactory>
@@ -257,15 +361,22 @@ class CallInitiator {
     return spine_->SpawnWaitable(name, std::move(promise_factory));
   }
 
+  bool WasCancelledPushed() const {
+    return spine_->call_filters().WasCancelledPushed();
+  }
+
   Arena* arena() { return spine_->arena(); }
   Party* party() { return spine_.get(); }
 
  private:
+  friend class CallHandler;
   RefCountedPtr<CallSpine> spine_;
 };
 
 class CallHandler {
  public:
+  using NextMessage = ClientToServerNextMessage;
+
   explicit CallHandler(RefCountedPtr<CallSpine> spine)
       : spine_(std::move(spine)) {}
 
@@ -277,12 +388,20 @@ class CallHandler {
     return spine_->PushServerInitialMetadata(std::move(md));
   }
 
+  void SpawnPushServerInitialMetadata(ServerMetadataHandle md) {
+    return spine_->SpawnPushServerInitialMetadata(std::move(md));
+  }
+
   void PushServerTrailingMetadata(ServerMetadataHandle status) {
     spine_->PushServerTrailingMetadata(std::move(status));
   }
 
-  void OnDone(absl::AnyInvocable<void(bool)> fn) {
-    spine_->OnDone(std::move(fn));
+  void SpawnPushServerTrailingMetadata(ServerMetadataHandle status) {
+    spine_->SpawnPushServerTrailingMetadata(std::move(status));
+  }
+
+  GRPC_MUST_USE_RESULT bool OnDone(absl::AnyInvocable<void(bool)> fn) {
+    return spine_->OnDone(std::move(fn));
   }
 
   template <typename Promise>
@@ -294,10 +413,18 @@ class CallHandler {
     return spine_->PushServerToClientMessage(std::move(message));
   }
 
+  auto SpawnPushMessage(MessageHandle message) {
+    return spine_->SpawnPushServerToClientMessage(std::move(message));
+  }
+
   auto PullMessage() { return spine_->PullClientToServerMessage(); }
 
   auto WasCancelled() { return spine_->WasCancelled(); }
 
+  bool WasCancelledPushed() const {
+    return spine_->call_filters().WasCancelledPushed();
+  }
+
   template <typename PromiseFactory>
   void SpawnGuarded(absl::string_view name, PromiseFactory promise_factory,
                     DebugLocation whence = {}) {
@@ -320,6 +447,11 @@ class CallHandler {
     return spine_->SpawnWaitable(name, std::move(promise_factory));
   }
 
+  void AddChildCall(const CallInitiator& initiator) {
+    CHECK(initiator.spine_ != nullptr);
+    spine_->AddChildCall(initiator.spine_);
+  }
+
   Arena* arena() { return spine_->arena(); }
   Party* party() { return spine_.get(); }
 
@@ -336,8 +468,8 @@ class UnstartedCallHandler {
     spine_->PushServerTrailingMetadata(std::move(status));
   }
 
-  void OnDone(absl::AnyInvocable<void(bool)> fn) {
-    spine_->OnDone(std::move(fn));
+  GRPC_MUST_USE_RESULT bool OnDone(absl::AnyInvocable<void(bool)> fn) {
+    return spine_->OnDone(std::move(fn));
   }
 
   template <typename Promise>
@@ -395,7 +527,7 @@ CallInitiatorAndHandler MakeCallPair(
     ClientMetadataHandle client_initial_metadata, RefCountedPtr<Arena> arena);
 
 template <typename CallHalf>
-auto OutgoingMessages(CallHalf h) {
+auto MessagesFrom(CallHalf h) {
   struct Wrapper {
     CallHalf h;
     auto Next() { return h.PullMessage(); }
@@ -403,6 +535,15 @@ auto OutgoingMessages(CallHalf h) {
   return Wrapper{std::move(h)};
 }
 
+template <typename CallHalf>
+auto MessagesFrom(CallHalf* h) {
+  struct Wrapper {
+    CallHalf* h;
+    auto Next() { return h->PullMessage(); }
+  };
+  return Wrapper{h};
+}
+
 // Forward a call from `call_handler` to `call_initiator` (with initial metadata
 // `client_initial_metadata`)
 // `on_server_trailing_metadata_from_initiator` is a callback that will be
