@@ -14,6 +14,8 @@
 // limitations under the License.
 //
 
+#include <grpc/impl/connectivity_state.h>
+#include <grpc/support/port_platform.h>
 #include <inttypes.h>
 #include <stdlib.h>
 
@@ -25,6 +27,7 @@
 #include <vector>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/meta/type_traits.h"
 #include "absl/random/random.h"
 #include "absl/status/status.h"
@@ -32,24 +35,19 @@
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
-
-#include <grpc/impl/connectivity_state.h>
-#include <grpc/support/log.h>
-#include <grpc/support/port_platform.h>
-
+#include "src/core/config/core_configuration.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gprpp/debug_location.h"
-#include "src/core/lib/gprpp/orphanable.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
-#include "src/core/lib/gprpp/work_serializer.h"
 #include "src/core/lib/transport/connectivity_state.h"
 #include "src/core/load_balancing/endpoint_list.h"
 #include "src/core/load_balancing/lb_policy.h"
 #include "src/core/load_balancing/lb_policy_factory.h"
 #include "src/core/resolver/endpoint_addresses.h"
+#include "src/core/util/debug_location.h"
 #include "src/core/util/json/json.h"
+#include "src/core/util/orphanable.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/work_serializer.h"
 
 namespace grpc_core {
 
@@ -71,9 +69,9 @@ class RoundRobin final : public LoadBalancingPolicy {
    public:
     RoundRobinEndpointList(RefCountedPtr<RoundRobin> round_robin,
                            EndpointAddressesIterator* endpoints,
-                           const ChannelArgs& args,
+                           const ChannelArgs& args, std::string resolution_note,
                            std::vector<std::string>* errors)
-        : EndpointList(std::move(round_robin),
+        : EndpointList(std::move(round_robin), std::move(resolution_note),
                        GRPC_TRACE_FLAG_ENABLED(round_robin)
                            ? "RoundRobinEndpointList"
                            : nullptr) {
@@ -184,23 +182,19 @@ RoundRobin::Picker::Picker(
   // the picker, see https://github.com/grpc/grpc-go/issues/2580.
   size_t index = absl::Uniform<size_t>(parent->bit_gen_, 0, pickers_.size());
   last_picked_index_.store(index, std::memory_order_relaxed);
-  if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-    gpr_log(GPR_INFO,
-            "[RR %p picker %p] created picker from endpoint_list=%p "
-            "with %" PRIuPTR " READY children; last_picked_index_=%" PRIuPTR,
-            parent_, this, parent_->endpoint_list_.get(), pickers_.size(),
-            index);
-  }
+  GRPC_TRACE_LOG(round_robin, INFO)
+      << "[RR " << parent_ << " picker " << this
+      << "] created picker from endpoint_list=" << parent_->endpoint_list_.get()
+      << " with " << pickers_.size()
+      << " READY children; last_picked_index_=" << index;
 }
 
 RoundRobin::PickResult RoundRobin::Picker::Pick(PickArgs args) {
   size_t index = last_picked_index_.fetch_add(1, std::memory_order_relaxed) %
                  pickers_.size();
-  if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-    gpr_log(GPR_INFO,
-            "[RR %p picker %p] using picker index %" PRIuPTR ", picker=%p",
-            parent_, this, index, pickers_[index].get());
-  }
+  GRPC_TRACE_LOG(round_robin, INFO)
+      << "[RR " << parent_ << " picker " << this << "] using picker index "
+      << index << ", picker=" << pickers_[index].get();
   return pickers_[index]->Pick(args);
 }
 
@@ -209,23 +203,18 @@ RoundRobin::PickResult RoundRobin::Picker::Pick(PickArgs args) {
 //
 
 RoundRobin::RoundRobin(Args args) : LoadBalancingPolicy(std::move(args)) {
-  if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-    gpr_log(GPR_INFO, "[RR %p] Created", this);
-  }
+  GRPC_TRACE_LOG(round_robin, INFO) << "[RR " << this << "] Created";
 }
 
 RoundRobin::~RoundRobin() {
-  if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-    gpr_log(GPR_INFO, "[RR %p] Destroying Round Robin policy", this);
-  }
+  GRPC_TRACE_LOG(round_robin, INFO)
+      << "[RR " << this << "] Destroying Round Robin policy";
   CHECK(endpoint_list_ == nullptr);
   CHECK(latest_pending_endpoint_list_ == nullptr);
 }
 
 void RoundRobin::ShutdownLocked() {
-  if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-    gpr_log(GPR_INFO, "[RR %p] Shutting down", this);
-  }
+  GRPC_TRACE_LOG(round_robin, INFO) << "[RR " << this << "] Shutting down";
   shutdown_ = true;
   endpoint_list_.reset();
   latest_pending_endpoint_list_.reset();
@@ -241,15 +230,12 @@ void RoundRobin::ResetBackoffLocked() {
 absl::Status RoundRobin::UpdateLocked(UpdateArgs args) {
   EndpointAddressesIterator* addresses = nullptr;
   if (args.addresses.ok()) {
-    if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      gpr_log(GPR_INFO, "[RR %p] received update", this);
-    }
+    GRPC_TRACE_LOG(round_robin, INFO) << "[RR " << this << "] received update";
     addresses = args.addresses->get();
   } else {
-    if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      gpr_log(GPR_INFO, "[RR %p] received update with address error: %s", this,
-              args.addresses.status().ToString().c_str());
-    }
+    GRPC_TRACE_LOG(round_robin, INFO)
+        << "[RR " << this
+        << "] received update with address error: " << args.addresses.status();
     // If we already have a child list, then keep using the existing
     // list, but still report back that the update was not accepted.
     if (endpoint_list_ != nullptr) return args.addresses.status();
@@ -257,28 +243,25 @@ absl::Status RoundRobin::UpdateLocked(UpdateArgs args) {
   // Create new child list, replacing the previous pending list, if any.
   if (GRPC_TRACE_FLAG_ENABLED(round_robin) &&
       latest_pending_endpoint_list_ != nullptr) {
-    gpr_log(GPR_INFO, "[RR %p] replacing previous pending child list %p", this,
-            latest_pending_endpoint_list_.get());
+    LOG(INFO) << "[RR " << this << "] replacing previous pending child list "
+              << latest_pending_endpoint_list_.get();
   }
   std::vector<std::string> errors;
   latest_pending_endpoint_list_ = MakeOrphanable<RoundRobinEndpointList>(
       RefAsSubclass<RoundRobin>(DEBUG_LOCATION, "RoundRobinEndpointList"),
-      addresses, args.args, &errors);
+      addresses, args.args, std::move(args.resolution_note), &errors);
   // If the new list is empty, immediately promote it to
   // endpoint_list_ and report TRANSIENT_FAILURE.
   if (latest_pending_endpoint_list_->size() == 0) {
     if (GRPC_TRACE_FLAG_ENABLED(round_robin) && endpoint_list_ != nullptr) {
-      gpr_log(GPR_INFO, "[RR %p] replacing previous child list %p", this,
-              endpoint_list_.get());
+      LOG(INFO) << "[RR " << this << "] replacing previous child list "
+                << endpoint_list_.get();
     }
     endpoint_list_ = std::move(latest_pending_endpoint_list_);
-    absl::Status status =
-        args.addresses.ok() ? absl::UnavailableError(absl::StrCat(
-                                  "empty address list: ", args.resolution_note))
-                            : args.addresses.status();
-    channel_control_helper()->UpdateState(
-        GRPC_CHANNEL_TRANSIENT_FAILURE, status,
-        MakeRefCounted<TransientFailurePicker>(status));
+    absl::Status status = args.addresses.ok()
+                              ? absl::UnavailableError("empty address list")
+                              : args.addresses.status();
+    endpoint_list_->ReportTransientFailure(status);
     return status;
   }
   // Otherwise, if this is the initial update, immediately promote it to
@@ -302,20 +285,17 @@ void RoundRobin::RoundRobinEndpointList::RoundRobinEndpoint::OnStateUpdate(
     grpc_connectivity_state new_state, const absl::Status& status) {
   auto* rr_endpoint_list = endpoint_list<RoundRobinEndpointList>();
   auto* round_robin = policy<RoundRobin>();
-  if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-    gpr_log(
-        GPR_INFO,
-        "[RR %p] connectivity changed for child %p, endpoint_list %p "
-        "(index %" PRIuPTR " of %" PRIuPTR "): prev_state=%s new_state=%s (%s)",
-        round_robin, this, rr_endpoint_list, Index(), rr_endpoint_list->size(),
-        (old_state.has_value() ? ConnectivityStateName(*old_state) : "N/A"),
-        ConnectivityStateName(new_state), status.ToString().c_str());
-  }
+  GRPC_TRACE_LOG(round_robin, INFO)
+      << "[RR " << round_robin << "] connectivity changed for child " << this
+      << ", endpoint_list " << rr_endpoint_list << " (index " << Index()
+      << " of " << rr_endpoint_list->size() << "): prev_state="
+      << (old_state.has_value() ? ConnectivityStateName(*old_state) : "N/A")
+      << " new_state=" << ConnectivityStateName(new_state) << " (" << status
+      << ")";
   if (new_state == GRPC_CHANNEL_IDLE) {
-    if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      gpr_log(GPR_INFO, "[RR %p] child %p reported IDLE; requesting connection",
-              round_robin, this);
-    }
+    GRPC_TRACE_LOG(round_robin, INFO)
+        << "[RR " << round_robin << "] child " << this
+        << " reported IDLE; requesting connection";
     ExitIdleLocked();
   }
   // If state changed, update state counters.
@@ -376,14 +356,10 @@ void RoundRobin::RoundRobinEndpointList::
        (num_ready_ > 0 && AllEndpointsSeenInitialState()) ||
        num_transient_failure_ == size())) {
     if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      const std::string old_counters_string =
-          round_robin->endpoint_list_ != nullptr
-              ? round_robin->endpoint_list_->CountersString()
-              : "";
-      gpr_log(GPR_INFO,
-              "[RR %p] swapping out child list %p (%s) in favor of %p (%s)",
-              round_robin, round_robin->endpoint_list_.get(),
-              old_counters_string.c_str(), this, CountersString().c_str());
+      LOG(INFO) << "[RR " << round_robin << "] swapping out child list "
+                << round_robin->endpoint_list_.get() << " ("
+                << round_robin->endpoint_list_->CountersString()
+                << ") in favor of " << this << " (" << CountersString() << ")";
     }
     round_robin->endpoint_list_ =
         std::move(round_robin->latest_pending_endpoint_list_);
@@ -395,10 +371,9 @@ void RoundRobin::RoundRobinEndpointList::
   // 2) ANY child is CONNECTING => policy is CONNECTING.
   // 3) ALL children are TRANSIENT_FAILURE => policy is TRANSIENT_FAILURE.
   if (num_ready_ > 0) {
-    if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      gpr_log(GPR_INFO, "[RR %p] reporting READY with child list %p",
-              round_robin, this);
-    }
+    GRPC_TRACE_LOG(round_robin, INFO)
+        << "[RR " << round_robin << "] reporting READY with child list "
+        << this;
     std::vector<RefCountedPtr<LoadBalancingPolicy::SubchannelPicker>> pickers;
     for (const auto& endpoint : endpoints()) {
       auto state = endpoint->connectivity_state();
@@ -411,27 +386,23 @@ void RoundRobin::RoundRobinEndpointList::
         GRPC_CHANNEL_READY, absl::OkStatus(),
         MakeRefCounted<Picker>(round_robin, std::move(pickers)));
   } else if (num_connecting_ > 0) {
-    if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      gpr_log(GPR_INFO, "[RR %p] reporting CONNECTING with child list %p",
-              round_robin, this);
-    }
+    GRPC_TRACE_LOG(round_robin, INFO)
+        << "[RR " << round_robin << "] reporting CONNECTING with child list "
+        << this;
     round_robin->channel_control_helper()->UpdateState(
-        GRPC_CHANNEL_CONNECTING, absl::Status(),
+        GRPC_CHANNEL_CONNECTING, absl::OkStatus(),
         MakeRefCounted<QueuePicker>(nullptr));
   } else if (num_transient_failure_ == size()) {
-    if (GRPC_TRACE_FLAG_ENABLED(round_robin)) {
-      gpr_log(GPR_INFO,
-              "[RR %p] reporting TRANSIENT_FAILURE with child list %p: %s",
-              round_robin, this, status_for_tf.ToString().c_str());
-    }
+    GRPC_TRACE_LOG(round_robin, INFO)
+        << "[RR " << round_robin
+        << "] reporting TRANSIENT_FAILURE with child list " << this << ": "
+        << status_for_tf;
     if (!status_for_tf.ok()) {
       last_failure_ = absl::UnavailableError(
           absl::StrCat("connections to all backends failing; last error: ",
                        status_for_tf.message()));
     }
-    round_robin->channel_control_helper()->UpdateState(
-        GRPC_CHANNEL_TRANSIENT_FAILURE, last_failure_,
-        MakeRefCounted<TransientFailurePicker>(last_failure_));
+    ReportTransientFailure(last_failure_);
   }
 }
 
