@@ -16,6 +16,7 @@
 
 #include "src/core/xds/grpc/xds_endpoint_parser.h"
 
+#include <grpc/support/port_platform.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -39,21 +40,25 @@
 #include "envoy/config/endpoint/v3/endpoint_components.upb.h"
 #include "envoy/type/v3/percent.upb.h"
 #include "google/protobuf/wrappers.upb.h"
-#include "upb/text/encode.h"
-
-#include <grpc/support/port_platform.h>
-
 #include "src/core/lib/address_utils/parse_address.h"
 #include "src/core/lib/address_utils/sockaddr_utils.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/debug/trace.h"
-#include "src/core/lib/gprpp/env.h"
-#include "src/core/lib/gprpp/validation_errors.h"
 #include "src/core/lib/iomgr/resolved_address.h"
+#include "src/core/load_balancing/ring_hash/ring_hash.h"
+#include "src/core/util/down_cast.h"
+#include "src/core/util/env.h"
+#include "src/core/util/json/json_args.h"
+#include "src/core/util/json/json_object_loader.h"
 #include "src/core/util/string.h"
 #include "src/core/util/upb_utils.h"
+#include "src/core/util/validation_errors.h"
+#include "src/core/xds/grpc/xds_cluster_parser.h"
+#include "src/core/xds/grpc/xds_common_types_parser.h"
 #include "src/core/xds/grpc/xds_health_status.h"
+#include "src/core/xds/grpc/xds_metadata_parser.h"
 #include "src/core/xds/xds_client/xds_resource_type.h"
+#include "upb/text/encode.h"
 
 // IWYU pragma: no_include "absl/meta/type_traits.h"
 
@@ -61,10 +66,20 @@ namespace grpc_core {
 
 namespace {
 
-// TODO(roth): Remove this once dualstack support is stable.
+// TODO(roth): Remove this after 1.67 is released.
 bool XdsDualstackEndpointsEnabled() {
   auto value = GetEnv("GRPC_EXPERIMENTAL_XDS_DUALSTACK_ENDPOINTS");
-  if (!value.has_value()) return false;
+  if (!value.has_value()) return true;
+  bool parsed_value;
+  bool parse_succeeded = gpr_parse_bool_value(value->c_str(), &parsed_value);
+  return parse_succeeded && parsed_value;
+}
+
+// TODO(roth): Flip the default to false once this proves stable, then
+// remove it entirely at some point in the future.
+bool XdsEndpointHashKeyBackwardCompatEnabled() {
+  auto value = GetEnv("GRPC_XDS_ENDPOINT_HASH_KEY_BACKWARD_COMPAT");
+  if (!value.has_value()) return true;
   bool parsed_value;
   bool parse_succeeded = gpr_parse_bool_value(value->c_str(), &parsed_value);
   return parse_succeeded && parsed_value;
@@ -85,57 +100,57 @@ void MaybeLogClusterLoadAssignment(
   }
 }
 
-absl::optional<grpc_resolved_address> ParseCoreAddress(
-    const envoy_config_core_v3_Address* address, ValidationErrors* errors) {
-  if (address == nullptr) {
-    errors->AddError("field not present");
-    return absl::nullopt;
-  }
-  ValidationErrors::ScopedField field(errors, ".socket_address");
-  const envoy_config_core_v3_SocketAddress* socket_address =
-      envoy_config_core_v3_Address_socket_address(address);
-  if (socket_address == nullptr) {
-    errors->AddError("field not present");
-    return absl::nullopt;
-  }
-  std::string address_str = UpbStringToStdString(
-      envoy_config_core_v3_SocketAddress_address(socket_address));
-  uint32_t port;
-  {
-    ValidationErrors::ScopedField field(errors, ".port_value");
-    port = envoy_config_core_v3_SocketAddress_port_value(socket_address);
-    if (GPR_UNLIKELY(port >> 16) != 0) {
-      errors->AddError("invalid port");
-      return absl::nullopt;
-    }
-  }
-  auto addr = StringToSockaddr(address_str, port);
-  if (!addr.ok()) {
-    errors->AddError(addr.status().message());
-    return absl::nullopt;
-  }
-  return *addr;
+std::string GetProxyAddressFromMetadata(const XdsMetadataMap& metadata_map) {
+  auto* proxy_address_entry = metadata_map.FindType<XdsAddressMetadataValue>(
+      "envoy.http11_proxy_transport_socket.proxy_address");
+  if (proxy_address_entry == nullptr) return "";
+  return proxy_address_entry->address();
+}
+
+std::string GetHashKeyFromMetadata(const XdsMetadataMap& metadata_map) {
+  auto* hash_key_entry =
+      metadata_map.FindType<XdsStructMetadataValue>("envoy.lb");
+  if (hash_key_entry == nullptr) return "";
+  ValidationErrors unused_errors;
+  return LoadJsonObjectField<std::string>(hash_key_entry->json().object(),
+                                          JsonArgs(), "hash_key",
+                                          &unused_errors)
+      .value_or("");
 }
 
 absl::optional<EndpointAddresses> EndpointAddressesParse(
+    const XdsResourceType::DecodeContext& context,
     const envoy_config_endpoint_v3_LbEndpoint* lb_endpoint,
-    ValidationErrors* errors) {
+    absl::string_view locality_proxy_address, ValidationErrors* errors) {
   // health_status
   const int32_t health_status =
       envoy_config_endpoint_v3_LbEndpoint_health_status(lb_endpoint);
   auto status = XdsHealthStatus::FromUpb(health_status);
   if (!status.has_value()) return absl::nullopt;
   // load_balancing_weight
-  uint32_t weight = 1;
+  uint32_t weight;
   {
     ValidationErrors::ScopedField field(errors, ".load_balancing_weight");
-    const google_protobuf_UInt32Value* load_balancing_weight =
-        envoy_config_endpoint_v3_LbEndpoint_load_balancing_weight(lb_endpoint);
-    if (load_balancing_weight != nullptr) {
-      weight = google_protobuf_UInt32Value_value(load_balancing_weight);
-      if (weight == 0) {
-        errors->AddError("must be greater than 0");
-      }
+    weight = ParseUInt32Value(
+                 envoy_config_endpoint_v3_LbEndpoint_load_balancing_weight(
+                     lb_endpoint))
+                 .value_or(1);
+    if (weight == 0) {
+      errors->AddError("must be greater than 0");
+    }
+  }
+  // metadata
+  std::string proxy_address;
+  std::string hash_key;
+  if (XdsHttpConnectEnabled() || !XdsEndpointHashKeyBackwardCompatEnabled()) {
+    XdsMetadataMap metadata_map = ParseXdsMetadataMap(
+        context, envoy_config_endpoint_v3_LbEndpoint_metadata(lb_endpoint),
+        errors);
+    if (XdsHttpConnectEnabled()) {
+      proxy_address = GetProxyAddressFromMetadata(metadata_map);
+    }
+    if (!XdsEndpointHashKeyBackwardCompatEnabled()) {
+      hash_key = GetHashKeyFromMetadata(metadata_map);
     }
   }
   // endpoint
@@ -151,7 +166,7 @@ absl::optional<EndpointAddresses> EndpointAddressesParse(
     }
     {
       ValidationErrors::ScopedField field(errors, ".address");
-      auto address = ParseCoreAddress(
+      auto address = ParseXdsAddress(
           envoy_config_endpoint_v3_Endpoint_address(endpoint), errors);
       if (address.has_value()) addresses.push_back(*address);
     }
@@ -163,7 +178,7 @@ absl::optional<EndpointAddresses> EndpointAddressesParse(
       for (size_t i = 0; i < size; ++i) {
         ValidationErrors::ScopedField field(
             errors, absl::StrCat(".additional_addresses[", i, "].address"));
-        auto address = ParseCoreAddress(
+        auto address = ParseXdsAddress(
             envoy_config_endpoint_v3_Endpoint_AdditionalAddress_address(
                 additional_addresses[i]),
             errors);
@@ -181,6 +196,14 @@ absl::optional<EndpointAddresses> EndpointAddressesParse(
   if (!hostname.empty()) {
     args = args.Set(GRPC_ARG_ADDRESS_NAME, hostname);
   }
+  if (!proxy_address.empty()) {
+    args = args.Set(GRPC_ARG_XDS_HTTP_PROXY, proxy_address);
+  } else if (!locality_proxy_address.empty()) {
+    args = args.Set(GRPC_ARG_XDS_HTTP_PROXY, locality_proxy_address);
+  }
+  if (!hash_key.empty()) {
+    args = args.Set(GRPC_ARG_RING_HASH_ENDPOINT_HASH_KEY, hash_key);
+  }
   return EndpointAddresses(addresses, args);
 }
 
@@ -200,6 +223,7 @@ using ResolvedAddressSet =
     std::set<grpc_resolved_address, ResolvedAddressLessThan>;
 
 absl::optional<ParsedLocality> LocalityParse(
+    const XdsResourceType::DecodeContext& context,
     const envoy_config_endpoint_v3_LocalityLbEndpoints* locality_lb_endpoints,
     ResolvedAddressSet* address_set, ValidationErrors* errors) {
   const size_t original_error_size = errors->size();
@@ -207,11 +231,11 @@ absl::optional<ParsedLocality> LocalityParse(
   // load_balancing_weight
   // If LB weight is not specified or 0, it means this locality is assigned
   // no load.
-  const google_protobuf_UInt32Value* lb_weight =
-      envoy_config_endpoint_v3_LocalityLbEndpoints_load_balancing_weight(
-          locality_lb_endpoints);
   parsed_locality.locality.lb_weight =
-      lb_weight != nullptr ? google_protobuf_UInt32Value_value(lb_weight) : 0;
+      ParseUInt32Value(
+          envoy_config_endpoint_v3_LocalityLbEndpoints_load_balancing_weight(
+              locality_lb_endpoints))
+          .value_or(0);
   if (parsed_locality.locality.lb_weight == 0) return absl::nullopt;
   // locality
   const envoy_config_core_v3_Locality* locality =
@@ -233,6 +257,16 @@ absl::optional<ParsedLocality> LocalityParse(
       UpbStringToStdString(envoy_config_core_v3_Locality_sub_zone(locality));
   parsed_locality.locality.name = MakeRefCounted<XdsLocalityName>(
       std::move(region), std::move(zone), std::move(sub_zone));
+  // metadata
+  std::string proxy_address;
+  if (XdsHttpConnectEnabled()) {
+    XdsMetadataMap metadata_map = ParseXdsMetadataMap(
+        context,
+        envoy_config_endpoint_v3_LocalityLbEndpoints_metadata(
+            locality_lb_endpoints),
+        errors);
+    proxy_address = GetProxyAddressFromMetadata(metadata_map);
+  }
   // lb_endpoints
   size_t size;
   const envoy_config_endpoint_v3_LbEndpoint* const* lb_endpoints =
@@ -241,7 +275,8 @@ absl::optional<ParsedLocality> LocalityParse(
   for (size_t i = 0; i < size; ++i) {
     ValidationErrors::ScopedField field(errors,
                                         absl::StrCat(".lb_endpoints[", i, "]"));
-    auto endpoint = EndpointAddressesParse(lb_endpoints[i], errors);
+    auto endpoint =
+        EndpointAddressesParse(context, lb_endpoints[i], proxy_address, errors);
     if (endpoint.has_value()) {
       for (const auto& address : endpoint->addresses()) {
         bool inserted = address_set->insert(address).second;
@@ -313,7 +348,7 @@ void DropParseAndAppend(
 }
 
 absl::StatusOr<std::shared_ptr<const XdsEndpointResource>> EdsResourceParse(
-    const XdsResourceType::DecodeContext& /*context*/,
+    const XdsResourceType::DecodeContext& context,
     const envoy_config_endpoint_v3_ClusterLoadAssignment*
         cluster_load_assignment) {
   ValidationErrors errors;
@@ -328,7 +363,8 @@ absl::StatusOr<std::shared_ptr<const XdsEndpointResource>> EdsResourceParse(
             cluster_load_assignment, &locality_size);
     for (size_t i = 0; i < locality_size; ++i) {
       ValidationErrors::ScopedField field(&errors, absl::StrCat("[", i, "]"));
-      auto parsed_locality = LocalityParse(endpoints[i], &address_set, &errors);
+      auto parsed_locality =
+          LocalityParse(context, endpoints[i], &address_set, &errors);
       if (parsed_locality.has_value()) {
         CHECK_NE(parsed_locality->locality.lb_weight, 0u);
         // Make sure prorities is big enough. Note that they might not
