@@ -16,19 +16,17 @@
 //
 //
 
+#include <grpc/grpc_security_constants.h>
+#include <grpcpp/security/alts_context.h>
 #include <stddef.h>
 
 #include <map>
 #include <string>
 
-#include "upb/base/string_view.h"
-#include "upb/collections/map.h"
-
-#include <grpc/grpc_security_constants.h>
-#include <grpcpp/security/alts_context.h>
-
 #include "src/proto/grpc/gcp/altscontext.upb.h"
 #include "src/proto/grpc/gcp/transport_security_common.upb.h"
+#include "upb/base/string_view.h"
+#include "upb/message/map.h"
 
 namespace grpc {
 namespace experimental {
