@@ -18,9 +18,13 @@
 
 #include "src/cpp/ext/filters/census/server_call_tracer.h"
 
+#include <grpc/grpc.h>
+#include <grpc/support/port_platform.h>
+#include <grpcpp/opencensus.h>
 #include <stdint.h>
 #include <string.h>
 
+#include <atomic>
 #include <memory>
 #include <string>
 #include <utility>
@@ -39,11 +43,6 @@
 #include "opencensus/trace/span_context.h"
 #include "opencensus/trace/span_id.h"
 #include "opencensus/trace/trace_id.h"
-
-#include <grpc/grpc.h>
-#include <grpc/support/port_platform.h>
-#include <grpcpp/opencensus.h>
-
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/promise/context.h"
 #include "src/core/lib/resource_quota/arena.h"
@@ -154,6 +153,11 @@ class OpenCensusServerCallTracer : public grpc_core::ServerCallTracer {
 
   void RecordEnd(const grpc_call_final_info* final_info) override;
 
+  void RecordIncomingBytes(
+      const TransportByteSize& transport_byte_size) override;
+  void RecordOutgoingBytes(
+      const TransportByteSize& transport_byte_size) override;
+
   void RecordAnnotation(absl::string_view annotation) override {
     if (!context_.Span().IsRecording()) {
       return;
@@ -190,9 +194,15 @@ class OpenCensusServerCallTracer : public grpc_core::ServerCallTracer {
   absl::Duration elapsed_time_;
   uint64_t recv_message_count_;
   uint64_t sent_message_count_;
-  // Buffer needed for grpc_slice to reference it when adding metatdata to
+  // Buffer needed for grpc_slice to reference it when adding metadata to
   // response.
   char stats_buf_[kMaxServerStatsLen];
+  // TODO(roth, ctiller): Won't need atomic here once chttp2 is migrated
+  // to promises, after which we can ensure that the transport invokes
+  // the RecordIncomingBytes() and RecordOutgoingBytes() methods inside
+  // the call's party.
+  std::atomic<uint64_t> incoming_bytes_{0};
+  std::atomic<uint64_t> outgoing_bytes_{0};
 };
 
 void OpenCensusServerCallTracer::RecordReceivedInitialMetadata(
@@ -236,8 +246,18 @@ void OpenCensusServerCallTracer::RecordSendTrailingMetadata(
 void OpenCensusServerCallTracer::RecordEnd(
     const grpc_call_final_info* final_info) {
   if (OpenCensusStatsEnabled()) {
-    const uint64_t request_size = GetOutgoingDataSize(final_info);
-    const uint64_t response_size = GetIncomingDataSize(final_info);
+    uint64_t outgoing_bytes;
+    uint64_t incoming_bytes;
+    if (grpc_core::IsCallTracerInTransportEnabled()) {
+      outgoing_bytes = outgoing_bytes_.load();
+      incoming_bytes = incoming_bytes_.load();
+    } else {
+      // Note: We are incorrectly swapping the two values here, which is
+      // a pre-existing bug.  This code will go away as part of the
+      // experiment rollout.
+      outgoing_bytes = GetIncomingDataSize(final_info);
+      incoming_bytes = GetOutgoingDataSize(final_info);
+    }
     double elapsed_time_ms = absl::ToDoubleMilliseconds(elapsed_time_);
     std::vector<std::pair<opencensus::tags::TagKey, std::string>> tags =
         context_.tags().tags();
@@ -246,8 +266,8 @@ void OpenCensusServerCallTracer::RecordEnd(
         ServerStatusTagKey(),
         std::string(StatusCodeToString(final_info->final_status)));
     ::opencensus::stats::Record(
-        {{RpcServerSentBytesPerRpc(), static_cast<double>(response_size)},
-         {RpcServerReceivedBytesPerRpc(), static_cast<double>(request_size)},
+        {{RpcServerSentBytesPerRpc(), static_cast<double>(outgoing_bytes)},
+         {RpcServerReceivedBytesPerRpc(), static_cast<double>(incoming_bytes)},
          {RpcServerServerLatency(), elapsed_time_ms},
          {RpcServerSentMessagesPerRpc(), sent_message_count_},
          {RpcServerReceivedMessagesPerRpc(), recv_message_count_}},
@@ -258,6 +278,16 @@ void OpenCensusServerCallTracer::RecordEnd(
   }
 }
 
+void OpenCensusServerCallTracer::RecordIncomingBytes(
+    const TransportByteSize& transport_byte_size) {
+  incoming_bytes_.fetch_add(transport_byte_size.data_bytes);
+}
+
+void OpenCensusServerCallTracer::RecordOutgoingBytes(
+    const TransportByteSize& transport_byte_size) {
+  outgoing_bytes_.fetch_add(transport_byte_size.data_bytes);
+}
+
 //
 // OpenCensusServerCallTracerFactory
 //
