@@ -14,6 +14,7 @@
 
 require 'etc'
 require 'mkmf'
+require_relative '../../lib/grpc/version.rb'
 
 windows = RUBY_PLATFORM =~ /mingw|mswin/
 windows_ucrt = RUBY_PLATFORM =~ /(mingw|mswin).*ucrt/
@@ -28,7 +29,28 @@ grpc_root = File.expand_path(File.join(File.dirname(__FILE__), '../../../..'))
 
 grpc_config = ENV['GRPC_CONFIG'] || 'opt'
 
-ENV['MACOSX_DEPLOYMENT_TARGET'] = '10.10'
+ENV['MACOSX_DEPLOYMENT_TARGET'] = '10.14'
+
+def debug_symbols_output_dir
+  d = ENV['GRPC_RUBY_DEBUG_SYMBOLS_OUTPUT_DIR']
+  return nil if d.nil? or d.size == 0
+  d
+end
+
+def maybe_remove_strip_all_linker_flag(flags)
+  if debug_symbols_output_dir
+    # Hack to prevent automatic stripping during shared library linking.
+    # rake-compiler-dock sets the -s LDFLAG when building rubies for
+    # cross compilation, and this -s flag propagates into RbConfig. Stripping
+    # during the link is problematic because it prevents us from saving
+    # debug symbols. We want to first link our shared library, then save
+    # debug symbols, and only after that strip.
+    flags = flags.split(' ')
+    flags = flags.reject {|flag| flag == '-s'}
+    flags = flags.join(' ')
+  end
+  flags
+end
 
 def env_unset?(name)
   ENV[name].nil? || ENV[name].size == 0
@@ -38,21 +60,27 @@ def inherit_env_or_rbconfig(name)
   ENV[name] = inherit_rbconfig(name) if env_unset?(name)
 end
 
-def inherit_rbconfig(name)
-  ENV[name] = RbConfig::CONFIG[name] || ''
+def inherit_rbconfig(name, linker_flag: false)
+  value = RbConfig::CONFIG[name] || ''
+  if linker_flag
+    value = maybe_remove_strip_all_linker_flag(value)
+  end
+  p "extconf.rb setting ENV[#{name}] = #{value}"
+  ENV[name] = value
 end
 
 def env_append(name, string)
   ENV[name] += ' ' + string
 end
 
+# build grpc C-core
 inherit_env_or_rbconfig 'AR'
 inherit_env_or_rbconfig 'CC'
 inherit_env_or_rbconfig 'CXX'
 inherit_env_or_rbconfig 'RANLIB'
 inherit_env_or_rbconfig 'STRIP'
 inherit_rbconfig 'CPPFLAGS'
-inherit_rbconfig 'LDFLAGS'
+inherit_rbconfig('LDFLAGS', linker_flag: true)
 
 ENV['LD'] = ENV['CC'] if env_unset?('LD')
 ENV['LDXX'] = ENV['CXX'] if env_unset?('LDXX')
@@ -72,8 +100,6 @@ ENV['EMBED_OPENSSL'] = (RUBY_ENGINE != 'truffleruby').to_s
 # Don't embed on TruffleRuby (the system zlib is already linked for the zlib C extension, slow build times)
 ENV['EMBED_ZLIB'] = (RUBY_ENGINE != 'truffleruby').to_s
 
-ENV['EMBED_CARES'] = 'true'
-
 ENV['ARCH_FLAGS'] = RbConfig::CONFIG['ARCH_FLAG']
 if apple_toolchain && !cross_compiling
   if RUBY_PLATFORM =~ /arm64/
@@ -83,7 +109,6 @@ if apple_toolchain && !cross_compiling
   end
 end
 
-env_append 'CPPFLAGS', '-DGPR_BACKWARDS_COMPATIBILITY_MODE'
 env_append 'CPPFLAGS', '-DGRPC_XDS_USER_AGENT_NAME_SUFFIX="\"RUBY\""'
 
 require_relative '../../lib/grpc/version'
@@ -111,25 +136,20 @@ unless windows
   puts "Building grpc native library: #{cmd}"
   system(cmd)
   exit 1 unless $? == 0
-
-  if grpc_config == 'opt'
-    rm_obj_cmd = "rm -rf #{File.join(output_dir, 'objs')}"
-    puts "Removing grpc object files: #{rm_obj_cmd}"
-    system(rm_obj_cmd)
-    exit 1 unless $? == 0
-    strip_cmd = "#{strip_tool} #{grpc_lib_dir}/*.a"
-    puts "Stripping grpc native library: #{strip_cmd}"
-    system(strip_cmd)
-    exit 1 unless $? == 0
-  end
 end
 
+# C-core built, generate Makefile for ruby extension
+$LDFLAGS = maybe_remove_strip_all_linker_flag($LDFLAGS)
+$DLDFLAGS = maybe_remove_strip_all_linker_flag($DLDFLAGS)
+
 $CFLAGS << ' -DGRPC_RUBY_WINDOWS_UCRT' if windows_ucrt
 $CFLAGS << ' -I' + File.join(grpc_root, 'include')
+$CFLAGS << ' -g'
 
 def have_ruby_abi_version()
   return true if RUBY_ENGINE == 'truffleruby'
   # ruby_abi_version is only available in development versions: https://github.com/ruby/ruby/pull/6231
+  # See also discussion for Ruby 3.4 in https://github.com/grpc/grpc/pull/38338 and https://github.com/grpc/grpc/pull/38487
   return false if RUBY_PATCHLEVEL >= 0
 
   m = /(\d+)\.(\d+)/.match(RUBY_VERSION)
@@ -168,10 +188,9 @@ if grpc_config == 'gcov'
 end
 
 if grpc_config == 'dbg'
-  $CFLAGS << ' -O0 -ggdb3'
+  $CFLAGS << ' -O0'
 end
 
-$LDFLAGS << ' -Wl,-wrap,memcpy' if linux
 # Do not statically link standard libraries on TruffleRuby as this does not work when compiling to bitcode
 if linux && RUBY_ENGINE != 'truffleruby'
   $LDFLAGS << ' -static-libgcc -static-libstdc++'
@@ -184,9 +203,49 @@ $CFLAGS << ' -Wextra '
 $CFLAGS << ' -pedantic '
 
 output = File.join('grpc', 'grpc_c')
+puts "extconf.rb $LDFLAGS: #{$LDFLAGS}"
+puts "extconf.rb $DLDFLAGS: #{$DLDFLAGS}"
+puts "extconf.rb $CFLAGS: #{$CFLAGS}"
 puts 'Generating Makefile for ' + output
 create_makefile(output)
 
+ruby_major_minor = /(\d+\.\d+)/.match(RUBY_VERSION).to_s
+debug_symbols = "grpc-#{GRPC::VERSION}-#{RUBY_PLATFORM}-ruby-#{ruby_major_minor}.dbg"
+
+File.open('Makefile.new', 'w') do |o|
+  o.puts 'hijack_remove_unused_artifacts: all remove_unused_artifacts'
+  o.puts
+  o.write(File.read('Makefile'))
+  o.puts
+  o.puts 'remove_unused_artifacts: $(DLLIB)'
+  # Now that the extension library has been linked, we can remove unused artifacts
+  # that take up a lot of disk space.
+  rm_obj_cmd = "rm -rf #{File.join(output_dir, 'objs')}"
+  o.puts "\t$(ECHO) Removing unused object artifacts: #{rm_obj_cmd}"
+  o.puts "\t$(Q) #{rm_obj_cmd}"
+  rm_grpc_core_libs = "rm -f #{grpc_lib_dir}/*.a"
+  o.puts "\t$(ECHO) Removing unused grpc core libraries: #{rm_grpc_core_libs}"
+  o.puts "\t$(Q) #{rm_grpc_core_libs}"
+end
+File.rename('Makefile.new', 'Makefile')
+
+if grpc_config == 'opt'
+  File.open('Makefile.new', 'w') do |o|
+    o.puts 'hijack: all strip'
+    o.puts
+    o.write(File.read('Makefile'))
+    o.puts
+    o.puts 'strip: $(DLLIB)'
+    if debug_symbols_output_dir
+      o.puts "\t$(ECHO) Saving debug symbols in #{debug_symbols_output_dir}/#{debug_symbols}"
+      o.puts "\t$(Q) objcopy --only-keep-debug $(DLLIB) #{debug_symbols_output_dir}/#{debug_symbols}"
+    end
+    o.puts "\t$(ECHO) Stripping $(DLLIB)"
+    o.puts "\t$(Q) #{strip_tool} $(DLLIB)"
+  end
+  File.rename('Makefile.new', 'Makefile')
+end
+
 if ENV['GRPC_RUBY_TEST_ONLY_WORKAROUND_MAKE_INSTALL_BUG']
   # Note: this env var setting is intended to work around a problem observed
   # with the ginstall command on grpc's macos automated test infrastructure,
