@@ -15,21 +15,21 @@
 #ifndef GRPC_TEST_CORE_CALL_YODEL_YODEL_TEST_H
 #define GRPC_TEST_CORE_CALL_YODEL_YODEL_TEST_H
 
+#include <grpc/event_engine/event_engine.h>
+
 #include "absl/functional/any_invocable.h"
 #include "absl/log/log.h"
 #include "absl/random/bit_gen_ref.h"
 #include "absl/strings/string_view.h"
 #include "gtest/gtest.h"
-
-#include <grpc/event_engine/event_engine.h>
-
-#include "src/core/lib/gprpp/debug_location.h"
+#include "src/core/lib/event_engine/event_engine_context.h"
 #include "src/core/lib/promise/cancel_callback.h"
 #include "src/core/lib/promise/detail/promise_factory.h"
 #include "src/core/lib/promise/promise.h"
 #include "src/core/lib/transport/call_arena_allocator.h"
 #include "src/core/lib/transport/call_spine.h"
 #include "src/core/lib/transport/metadata.h"
+#include "src/core/util/debug_location.h"
 #include "test/core/event_engine/fuzzing_event_engine/fuzzing_event_engine.h"
 #include "test/core/test_util/test_config.h"
 
@@ -362,10 +362,46 @@ class YodelTest : public ::testing::Test {
         .Start(std::move(actions)...);
   }
 
+  class NoContext {
+   public:
+    explicit NoContext(grpc_event_engine::experimental::EventEngine* ee) {
+      auto arena = SimpleArenaAllocator()->MakeArena();
+      arena->SetContext(ee);
+      party_ = Party::Make(std::move(arena));
+    }
+
+    template <typename PromiseFactory>
+    void SpawnInfallible(absl::string_view name,
+                         PromiseFactory promise_factory) {
+      party_->Spawn(
+          name,
+          [party = party_,
+           promise_factory = std::move(promise_factory)]() mutable {
+            promise_detail::OncePromiseFactory<void, PromiseFactory> factory(
+                std::move(promise_factory));
+            return [party, underlying = factory.Make()]() mutable {
+              return underlying();
+            };
+          },
+          [](Empty) {});
+    }
+
+   private:
+    RefCountedPtr<Party> party_;
+  };
+
+  template <typename... Actions>
+  void SpawnTestSeqWithoutContext(
+      yodel_detail::NameAndLocation name_and_location, Actions... actions) {
+    SpawnTestSeq(NoContext{event_engine().get()}, name_and_location,
+                 std::move(actions)...);
+  }
+
   auto MakeCall(ClientMetadataHandle client_initial_metadata) {
-    return MakeCallPair(std::move(client_initial_metadata),
-                        state_->event_engine.get(),
-                        state_->call_arena_allocator->MakeArena());
+    auto arena = state_->call_arena_allocator->MakeArena();
+    arena->SetContext<grpc_event_engine::experimental::EventEngine>(
+        state_->event_engine.get());
+    return MakeCallPair(std::move(client_initial_metadata), std::move(arena));
   }
 
   void WaitForAllPendingWork();
@@ -389,6 +425,10 @@ class YodelTest : public ::testing::Test {
     return state_->event_engine;
   }
 
+  void SetMaxRandomMessageSize(size_t max_random_message_size) {
+    max_random_message_size_ = max_random_message_size;
+  }
+
  private:
   class WatchDog;
   struct State {
@@ -415,6 +455,7 @@ class YodelTest : public ::testing::Test {
   fuzzing_event_engine::Actions actions_;
   std::unique_ptr<State> state_;
   std::queue<std::shared_ptr<yodel_detail::ActionState>> pending_actions_;
+  size_t max_random_message_size_ = 1024 * 1024;
 };
 
 }  // namespace grpc_core
