@@ -18,22 +18,22 @@
 
 #include "src/core/lib/channel/channel_args.h"
 
-#include <string.h>
-
-#include "gtest/gtest.h"
-
+#include <grpc/credentials.h>
 #include <grpc/grpc.h>
 #include <grpc/grpc_security.h>
 #include <grpc/impl/channel_arg_names.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
+#include <string.h>
 
-#include "src/core/lib/gpr/useful.h"
-#include "src/core/lib/gprpp/notification.h"
-#include "src/core/lib/gprpp/ref_counted.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "gtest/gtest.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
-#include "test/core/util/test_config.h"
+#include "src/core/util/notification.h"
+#include "src/core/util/ref_counted.h"
+#include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/useful.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc_core {
 
@@ -253,15 +253,14 @@ TEST(GrpcChannelArgsTest, Create) {
                                              const_cast<char*>("str value"));
   ch_args = grpc_channel_args_copy_and_add(nullptr, to_add, 2);
 
-  GPR_ASSERT(ch_args->num_args == 2);
-  GPR_ASSERT(strcmp(ch_args->args[0].key, to_add[0].key) == 0);
-  GPR_ASSERT(ch_args->args[0].type == to_add[0].type);
-  GPR_ASSERT(ch_args->args[0].value.integer == to_add[0].value.integer);
+  CHECK_EQ(ch_args->num_args, 2);
+  CHECK_EQ(strcmp(ch_args->args[0].key, to_add[0].key), 0);
+  CHECK(ch_args->args[0].type == to_add[0].type);
+  CHECK(ch_args->args[0].value.integer == to_add[0].value.integer);
 
-  GPR_ASSERT(strcmp(ch_args->args[1].key, to_add[1].key) == 0);
-  GPR_ASSERT(ch_args->args[1].type == to_add[1].type);
-  GPR_ASSERT(strcmp(ch_args->args[1].value.string, to_add[1].value.string) ==
-             0);
+  CHECK_EQ(strcmp(ch_args->args[1].key, to_add[1].key), 0);
+  CHECK(ch_args->args[1].type == to_add[1].type);
+  CHECK(strcmp(ch_args->args[1].value.string, to_add[1].value.string) == 0);
 
   grpc_channel_args_destroy(ch_args);
 }
@@ -271,7 +270,7 @@ struct fake_class {
 };
 
 static void* fake_pointer_arg_copy(void* arg) {
-  gpr_log(GPR_DEBUG, "fake_pointer_arg_copy");
+  VLOG(2) << "fake_pointer_arg_copy";
   fake_class* fc = static_cast<fake_class*>(arg);
   fake_class* new_fc = static_cast<fake_class*>(gpr_malloc(sizeof(fake_class)));
   new_fc->foo = fc->foo;
@@ -279,7 +278,7 @@ static void* fake_pointer_arg_copy(void* arg) {
 }
 
 static void fake_pointer_arg_destroy(void* arg) {
-  gpr_log(GPR_DEBUG, "fake_pointer_arg_destroy");
+  VLOG(2) << "fake_pointer_arg_destroy";
   fake_class* fc = static_cast<fake_class*>(arg);
   gpr_free(fc);
 }
@@ -317,19 +316,18 @@ TEST(GrpcChannelArgsTest, ChannelCreateWithArgs) {
 grpc_channel_args* mutate_channel_args(const char* target,
                                        grpc_channel_args* old_args,
                                        grpc_channel_stack_type /*type*/) {
-  GPR_ASSERT(old_args != nullptr);
-  GPR_ASSERT(grpc_channel_args_find(old_args, "arg_int")->value.integer == 0);
-  GPR_ASSERT(strcmp(grpc_channel_args_find(old_args, "arg_str")->value.string,
-                    "arg_str_val") == 0);
-  GPR_ASSERT(
-      grpc_channel_args_find(old_args, "arg_pointer")->value.pointer.vtable ==
-      &fake_pointer_arg_vtable);
+  CHECK_NE(old_args, nullptr);
+  CHECK_EQ(grpc_channel_args_find(old_args, "arg_int")->value.integer, 0);
+  CHECK(strcmp(grpc_channel_args_find(old_args, "arg_str")->value.string,
+               "arg_str_val") == 0);
+  CHECK(grpc_channel_args_find(old_args, "arg_pointer")->value.pointer.vtable ==
+        &fake_pointer_arg_vtable);
 
   if (strcmp(target, "no_op_mutator") == 0) {
     return old_args;
   }
 
-  GPR_ASSERT(strcmp(target, "minimal_stack_mutator") == 0);
+  CHECK_EQ(strcmp(target, "minimal_stack_mutator"), 0);
   const char* args_to_remove[] = {"arg_int", "arg_str", "arg_pointer"};
 
   grpc_arg no_deadline_filter_arg = grpc_channel_arg_integer_create(
