@@ -14,6 +14,7 @@
 
 #include "test/core/end2end/fuzzers/network_input.h"
 
+#include <grpc/slice.h>
 #include <stddef.h>
 #include <stdint.h>
 
@@ -26,15 +27,12 @@
 #include "absl/log/log.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/span.h"
-
-#include <grpc/slice.h>
-
+#include "src/core/config/core_configuration.h"
 #include "src/core/ext/transport/chaotic_good/frame_header.h"
 #include "src/core/ext/transport/chttp2/transport/frame.h"
 #include "src/core/ext/transport/chttp2/transport/varint.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_args_preconditioning.h"
-#include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/event_engine/channel_args_endpoint_config.h"
 #include "src/core/lib/event_engine/tcp_socket_utils.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
@@ -200,73 +198,94 @@ SliceBuffer ChaoticGoodFrame(const fuzzer_input::ChaoticGoodFrame& frame) {
   chaotic_good::FrameHeader h;
   SliceBuffer suffix;
   h.stream_id = frame.stream_id();
-  switch (frame.type()) {
-    case fuzzer_input::ChaoticGoodFrame::SETTINGS:
-      h.type = chaotic_good::FrameType::kSettings;
+  switch (frame.frame_type_case()) {
+    case fuzzer_input::ChaoticGoodFrame::kKnownType:
+      switch (frame.known_type()) {
+        case fuzzer_input::ChaoticGoodFrame::SETTINGS:
+          h.type = chaotic_good::FrameType::kSettings;
+          break;
+        case fuzzer_input::ChaoticGoodFrame::CLIENT_INITIAL_METADATA:
+          h.type = chaotic_good::FrameType::kClientInitialMetadata;
+          break;
+        case fuzzer_input::ChaoticGoodFrame::MESSAGE:
+          h.type = chaotic_good::FrameType::kMessage;
+          break;
+        case fuzzer_input::ChaoticGoodFrame::CLIENT_END_OF_STREAM:
+          h.type = chaotic_good::FrameType::kClientEndOfStream;
+          break;
+        case fuzzer_input::ChaoticGoodFrame::SERVER_INITIAL_METADATA:
+          h.type = chaotic_good::FrameType::kServerInitialMetadata;
+          break;
+        case fuzzer_input::ChaoticGoodFrame::SERVER_TRAILING_METADATA:
+          h.type = chaotic_good::FrameType::kServerTrailingMetadata;
+          break;
+        case fuzzer_input::ChaoticGoodFrame::CANCEL:
+          h.type = chaotic_good::FrameType::kCancel;
+          break;
+        default:
+          break;
+      }
       break;
-    case fuzzer_input::ChaoticGoodFrame::FRAGMENT:
-      h.type = chaotic_good::FrameType::kFragment;
+    case fuzzer_input::ChaoticGoodFrame::kUnknownType:
+      h.type = static_cast<chaotic_good::FrameType>(frame.unknown_type());
       break;
-    case fuzzer_input::ChaoticGoodFrame::CANCEL:
-      h.type = chaotic_good::FrameType::kCancel;
-      break;
-    default:
+    case fuzzer_input::ChaoticGoodFrame::FRAME_TYPE_NOT_SET:
+      h.type = chaotic_good::FrameType::kMessage;
       break;
   }
-  switch (frame.headers_case()) {
-    case fuzzer_input::ChaoticGoodFrame::kHeadersNone:
-    case fuzzer_input::ChaoticGoodFrame::HEADERS_NOT_SET:
+  h.stream_id = frame.stream_id();
+  h.payload_connection_id = 0;
+  h.payload_length = 0;
+  auto proto_payload = [&](auto payload) {
+    std::string temp = payload.SerializeAsString();
+    h.payload_length = temp.length();
+    suffix.Append(Slice::FromCopiedString(temp));
+  };
+  switch (frame.payload_case()) {
+    case fuzzer_input::ChaoticGoodFrame::kPayloadNone:
+    case fuzzer_input::ChaoticGoodFrame::PAYLOAD_NOT_SET:
       break;
-    case fuzzer_input::ChaoticGoodFrame::kHeadersRawBytes:
-      if (frame.headers_raw_bytes().empty()) break;
-      h.header_length = frame.headers_raw_bytes().size();
-      h.flags.Set(0, true);
-      suffix.Append(Slice::FromCopiedString(frame.headers_raw_bytes()));
+    case fuzzer_input::ChaoticGoodFrame::kPayloadRawBytes:
+      if (frame.payload_raw_bytes().empty()) break;
+      h.payload_length = frame.payload_raw_bytes().length();
+      suffix.Append(Slice::FromCopiedString(frame.payload_raw_bytes()));
       break;
-    case fuzzer_input::ChaoticGoodFrame::kHeadersSimpleHeader: {
-      SliceBuffer append =
-          SliceBufferFromSimpleHeaders(frame.headers_simple_header());
-      if (append.Length() == 0) break;
-      h.header_length = append.Length();
-      h.flags.Set(0, true);
-      suffix.Append(append.JoinIntoSlice());
-    } break;
-  }
-  switch (frame.data_case()) {
-    case fuzzer_input::ChaoticGoodFrame::kDataNone:
-    case fuzzer_input::ChaoticGoodFrame::DATA_NOT_SET:
+    case fuzzer_input::ChaoticGoodFrame::kPayloadEmptyOfLength:
+      h.payload_length = frame.payload_empty_of_length();
+      suffix.Append(Slice::FromCopiedString(
+          std::string(frame.payload_empty_of_length(), 'a')));
       break;
-    case fuzzer_input::ChaoticGoodFrame::kDataSized:
-      h.flags.Set(1, true);
-      h.message_length = frame.data_sized().length();
-      h.message_padding = frame.data_sized().padding();
+    case fuzzer_input::ChaoticGoodFrame::kPayloadOtherConnectionId:
+      h.payload_connection_id =
+          frame.payload_other_connection_id().connection_id();
+      h.payload_length = frame.payload_other_connection_id().length();
+      break;
+    case fuzzer_input::ChaoticGoodFrame::kSettings:
+      proto_payload(frame.settings());
+      break;
+    case fuzzer_input::ChaoticGoodFrame::kClientMetadata:
+      proto_payload(frame.client_metadata());
+      break;
+    case fuzzer_input::ChaoticGoodFrame::kServerMetadata:
+      proto_payload(frame.server_metadata());
       break;
   }
-  switch (frame.trailers_case()) {
-    case fuzzer_input::ChaoticGoodFrame::kTrailersNone:
-    case fuzzer_input::ChaoticGoodFrame::TRAILERS_NOT_SET:
-      break;
-    case fuzzer_input::ChaoticGoodFrame::kTrailersRawBytes:
-      h.trailer_length = frame.trailers_raw_bytes().size();
-      h.flags.Set(2, true);
-      suffix.Append(Slice::FromCopiedString(frame.trailers_raw_bytes()));
-      break;
-    case fuzzer_input::ChaoticGoodFrame::kTrailersSimpleHeader: {
-      SliceBuffer append =
-          SliceBufferFromSimpleHeaders(frame.trailers_simple_header());
-      h.trailer_length = append.Length();
-      h.flags.Set(2, true);
-      suffix.Append(append.JoinIntoSlice());
-    } break;
-  }
-  uint8_t bytes[24];
+  uint8_t bytes[chaotic_good::FrameHeader::kFrameHeaderSize];
   h.Serialize(bytes);
   SliceBuffer out;
-  out.Append(Slice::FromCopiedBuffer(bytes, 24));
+  out.Append(Slice::FromCopiedBuffer(
+      bytes, chaotic_good::FrameHeader::kFrameHeaderSize));
   out.Append(suffix);
   return out;
 }
 
+void store32_little_endian(uint32_t value, unsigned char* buf) {
+  buf[3] = static_cast<unsigned char>((value >> 24) & 0xFF);
+  buf[2] = static_cast<unsigned char>((value >> 16) & 0xFF);
+  buf[1] = static_cast<unsigned char>((value >> 8) & 0xFF);
+  buf[0] = static_cast<unsigned char>((value) & 0xFF);
+}
+
 grpc_slice SliceFromSegment(const fuzzer_input::InputSegment& segment) {
   switch (segment.payload_case()) {
     case fuzzer_input::InputSegment::kRawBytes:
@@ -321,6 +340,9 @@ grpc_slice SliceFromSegment(const fuzzer_input::InputSegment& segment) {
           segment.window_update().stream_id(),
           segment.window_update().increment(),
       });
+    case fuzzer_input::InputSegment::kSecurityFrame:
+      return SliceFromH2Frame(Http2SecurityFrame{
+          SliceBufferFromBytes(segment.security_frame().payload())});
     case fuzzer_input::InputSegment::kClientPrefix:
       return grpc_slice_from_static_string("PRI * HTTP/2.0\r\n\r\nSM\r\n\r\n");
     case fuzzer_input::InputSegment::kRepeatedZeros: {
@@ -333,6 +355,38 @@ grpc_slice SliceFromSegment(const fuzzer_input::InputSegment& segment) {
           .JoinIntoSlice()
           .TakeCSlice();
     } break;
+    case fuzzer_input::InputSegment::kFakeTransportFrame: {
+      auto generate = [](absl::string_view payload) {
+        uint32_t length = payload.length();
+        std::vector<unsigned char> bytes;
+        bytes.resize(4);
+        store32_little_endian(length + 4, bytes.data());
+        for (auto c : payload) {
+          bytes.push_back(static_cast<unsigned char>(c));
+        }
+        return grpc_slice_from_copied_buffer(
+            reinterpret_cast<const char*>(bytes.data()), bytes.size());
+      };
+      switch (segment.fake_transport_frame().payload_case()) {
+        case fuzzer_input::FakeTransportFrame::kRawBytes:
+          return generate(segment.fake_transport_frame().raw_bytes());
+        case fuzzer_input::FakeTransportFrame::kMessageString:
+          switch (segment.fake_transport_frame().message_string()) {
+            default:
+              return generate("UNKNOWN");
+            case fuzzer_input::FakeTransportFrame::CLIENT_INIT:
+              return generate("CLIENT_INIT");
+            case fuzzer_input::FakeTransportFrame::SERVER_INIT:
+              return generate("SERVER_INIT");
+            case fuzzer_input::FakeTransportFrame::CLIENT_FINISHED:
+              return generate("CLIENT_FINISHED");
+            case fuzzer_input::FakeTransportFrame::SERVER_FINISHED:
+              return generate("SERVER_FINISHED");
+          }
+        case fuzzer_input::FakeTransportFrame::PAYLOAD_NOT_SET:
+          return generate("");
+      }
+    }
     case fuzzer_input::InputSegment::PAYLOAD_NOT_SET:
       break;
   }
@@ -384,23 +438,27 @@ std::vector<QueuedRead> MakeSchedule(
 
 Duration ScheduleReads(
     const fuzzer_input::NetworkInput& network_input,
-    grpc_endpoint* mock_endpoint,
+    std::shared_ptr<grpc_event_engine::experimental::MockEndpointController>
+        mock_endpoint_controller,
     grpc_event_engine::experimental::FuzzingEventEngine* event_engine) {
   int delay = 0;
   for (const auto& q : MakeSchedule(network_input)) {
     event_engine->RunAfterExactly(
         std::chrono::milliseconds(q.delay_ms),
-        [mock_endpoint, slices = q.slices.JoinIntoSlice()]() mutable {
+        [mock_endpoint_controller,
+         slices = q.slices.JoinIntoSlice()]() mutable {
           ExecCtx exec_ctx;
-          grpc_mock_endpoint_put_read(mock_endpoint, slices.TakeCSlice());
+          mock_endpoint_controller->TriggerReadEvent(
+              std::move(grpc_event_engine::experimental::internal::SliceCast<
+                        grpc_event_engine::experimental::Slice>(slices)));
         });
     delay = std::max(delay, q.delay_ms);
   }
-  event_engine->RunAfterExactly(
-      std::chrono::milliseconds(delay + 1), [mock_endpoint] {
-        ExecCtx exec_ctx;
-        grpc_mock_endpoint_finish_put_reads(mock_endpoint);
-      });
+  event_engine->RunAfterExactly(std::chrono::milliseconds(delay + 1),
+                                [mock_endpoint_controller] {
+                                  ExecCtx exec_ctx;
+                                  mock_endpoint_controller->NoMoreReads();
+                                });
   return Duration::Milliseconds(delay + 2);
 }
 
@@ -541,4 +599,15 @@ Duration ScheduleConnection(
   return delay;
 }
 
+void ScheduleWrites(
+    const fuzzer_input::NetworkInput& network_input,
+    std::unique_ptr<grpc_event_engine::experimental::EventEngine::Endpoint>
+        endpoint,
+    grpc_event_engine::experimental::FuzzingEventEngine* event_engine) {
+  auto schedule = MakeSchedule(network_input);
+  auto ep = std::shared_ptr<EventEngine::Endpoint>(std::move(endpoint));
+  ReadForever(ep);
+  ScheduleWritesForReads(ep, event_engine, std::move(schedule));
+}
+
 }  // namespace grpc_core
