@@ -16,6 +16,16 @@
 //
 //
 
+#include <grpc/credentials.h>
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/grpc.h>
+#include <grpc/grpc_security.h>
+#include <grpc/impl/channel_arg_names.h>
+#include <grpc/impl/propagation_bits.h>
+#include <grpc/slice.h>
+#include <grpc/status.h>
+#include <grpc/support/sync.h>
+#include <grpc/support/time.h>
 #include <stdint.h>
 #include <string.h>
 
@@ -26,26 +36,15 @@
 #include <utility>
 #include <vector>
 
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
-
-#include <grpc/event_engine/event_engine.h>
-#include <grpc/grpc.h>
-#include <grpc/grpc_security.h>
-#include <grpc/impl/propagation_bits.h>
-#include <grpc/slice.h>
-#include <grpc/status.h>
-#include <grpc/support/log.h>
-#include <grpc/support/sync.h>
-#include <grpc/support/time.h>
-
-#include "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/event_engine/default_event_engine.h"
-#include "src/core/lib/gprpp/debug_location.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/lib/experiments/experiments.h"
 #include "src/core/lib/iomgr/closure.h"
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
@@ -55,10 +54,13 @@
 #include "src/core/lib/iomgr/resolved_address.h"
 #include "src/core/lib/iomgr/sockaddr.h"
 #include "src/core/lib/iomgr/socket_utils.h"
-#include "src/core/lib/resolver/server_address.h"
+#include "src/core/resolver/dns/c_ares/grpc_ares_wrapper.h"
+#include "src/core/resolver/endpoint_addresses.h"
+#include "src/core/util/debug_location.h"
+#include "src/core/util/time.h"
 #include "test/core/end2end/cq_verifier.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
 
 static gpr_mu g_mu;
 static int g_resolve_port = -1;
@@ -66,7 +68,7 @@ static int g_resolve_port = -1;
 static grpc_ares_request* (*iomgr_dns_lookup_ares)(
     const char* dns_server, const char* addr, const char* default_port,
     grpc_pollset_set* interested_parties, grpc_closure* on_done,
-    std::unique_ptr<grpc_core::ServerAddressList>* addresses,
+    std::unique_ptr<grpc_core::EndpointAddressesList>* addresses,
     int query_timeout_ms);
 
 static void (*iomgr_cancel_ares_request)(grpc_ares_request* request);
@@ -169,7 +171,7 @@ class TestDNSResolver : public grpc_core::DNSResolver {
 static grpc_ares_request* my_dns_lookup_ares(
     const char* dns_server, const char* addr, const char* default_port,
     grpc_pollset_set* interested_parties, grpc_closure* on_done,
-    std::unique_ptr<grpc_core::ServerAddressList>* addresses,
+    std::unique_ptr<grpc_core::EndpointAddressesList>* addresses,
     int query_timeout_ms) {
   if (0 != strcmp(addr, "test")) {
     // A records should suffice
@@ -184,7 +186,7 @@ static grpc_ares_request* my_dns_lookup_ares(
     gpr_mu_unlock(&g_mu);
     error = GRPC_ERROR_CREATE("Forced Failure");
   } else {
-    *addresses = std::make_unique<grpc_core::ServerAddressList>();
+    *addresses = std::make_unique<grpc_core::EndpointAddressesList>();
     grpc_resolved_address address;
     memset(&address, 0, sizeof(address));
     auto* sa = reinterpret_cast<grpc_sockaddr_in*>(&address.addr);
@@ -206,6 +208,12 @@ static void my_cancel_ares_request(grpc_ares_request* request) {
 }
 
 int main(int argc, char** argv) {
+  // TODO(yijiem): rewrite this test with a custom EventEngine DNS Resolver
+  if (grpc_core::IsEventEngineDnsEnabled()) {
+    LOG(ERROR) << "Skipping iomgr-specific DNS test because EventEngine DNS is "
+                  "enabled";
+    return 0;
+  }
   grpc_completion_queue* cq;
   grpc_op ops[6];
   grpc_op* op;
@@ -257,7 +265,7 @@ int main(int argc, char** argv) {
           .Set(GRPC_ARG_MAX_RECONNECT_BACKOFF_MS, 1000)
           .Set(GRPC_ARG_MIN_RECONNECT_BACKOFF_MS, 5000)
           // When this test brings down server1 and then brings up server2,
-          // the targetted server port number changes, and the client channel
+          // the targeted server port number changes, and the client channel
           // needs to re-resolve to pick this up. This test requires that
           // happen within 10 seconds, but gRPC's DNS resolvers rate limit
           // resolution attempts to at most once every 30 seconds by default.
@@ -283,9 +291,9 @@ int main(int argc, char** argv) {
   op->flags = GRPC_INITIAL_METADATA_WAIT_FOR_READY;
   op->reserved = nullptr;
   op++;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_call_start_batch(call1, ops, (size_t)(op - ops),
-                                   grpc_core::CqVerifier::tag(0x101), nullptr));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_call_start_batch(call1, ops, (size_t)(op - ops),
+                                 grpc_core::CqVerifier::tag(0x101), nullptr));
   // and receive status to probe termination
   memset(ops, 0, sizeof(ops));
   op = ops;
@@ -296,9 +304,9 @@ int main(int argc, char** argv) {
   op->flags = 0;
   op->reserved = nullptr;
   op++;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_call_start_batch(call1, ops, (size_t)(op - ops),
-                                   grpc_core::CqVerifier::tag(0x102), nullptr));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_call_start_batch(call1, ops, (size_t)(op - ops),
+                                 grpc_core::CqVerifier::tag(0x102), nullptr));
 
   // bring a server up on the first port
   grpc_server* server1 = grpc_server_create(nullptr, nullptr);
@@ -312,10 +320,10 @@ int main(int argc, char** argv) {
 
   // request a call to the server
   grpc_call* server_call1;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_server_request_call(server1, &server_call1, &request_details1,
-                                      &request_metadata1, cq, cq,
-                                      grpc_core::CqVerifier::tag(0x301)));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_server_request_call(server1, &server_call1, &request_details1,
+                                    &request_metadata1, cq, cq,
+                                    grpc_core::CqVerifier::tag(0x301)));
 
   set_resolve_port(port1);
 
@@ -324,8 +332,7 @@ int main(int argc, char** argv) {
   cqv.Expect(grpc_core::CqVerifier::tag(0x301), true);
   cqv.Verify();
 
-  GPR_ASSERT(GRPC_CHANNEL_READY ==
-             grpc_channel_check_connectivity_state(chan, 0));
+  CHECK(GRPC_CHANNEL_READY == grpc_channel_check_connectivity_state(chan, 0));
   grpc_channel_watch_connectivity_state(chan, GRPC_CHANNEL_READY,
                                         gpr_inf_future(GPR_CLOCK_REALTIME), cq,
                                         grpc_core::CqVerifier::tag(0x9999));
@@ -337,9 +344,9 @@ int main(int argc, char** argv) {
   op->data.recv_close_on_server.cancelled = &was_cancelled1;
   op->flags = 0;
   op++;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_call_start_batch(server_call1, ops, (size_t)(op - ops),
-                                   grpc_core::CqVerifier::tag(0x302), nullptr));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_call_start_batch(server_call1, ops, (size_t)(op - ops),
+                                 grpc_core::CqVerifier::tag(0x302), nullptr));
 
   // shutdown first server:
   // we should see a connectivity change and then nothing
@@ -363,9 +370,9 @@ int main(int argc, char** argv) {
   op->flags = GRPC_INITIAL_METADATA_WAIT_FOR_READY;
   op->reserved = nullptr;
   op++;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_call_start_batch(call2, ops, (size_t)(op - ops),
-                                   grpc_core::CqVerifier::tag(0x201), nullptr));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_call_start_batch(call2, ops, (size_t)(op - ops),
+                                 grpc_core::CqVerifier::tag(0x201), nullptr));
   // and receive status to probe termination
   memset(ops, 0, sizeof(ops));
   op = ops;
@@ -376,9 +383,9 @@ int main(int argc, char** argv) {
   op->flags = 0;
   op->reserved = nullptr;
   op++;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_call_start_batch(call2, ops, (size_t)(op - ops),
-                                   grpc_core::CqVerifier::tag(0x202), nullptr));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_call_start_batch(call2, ops, (size_t)(op - ops),
+                                 grpc_core::CqVerifier::tag(0x202), nullptr));
 
   // and bring up second server
   set_resolve_port(port2);
@@ -393,10 +400,10 @@ int main(int argc, char** argv) {
 
   // request a call to the server
   grpc_call* server_call2;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_server_request_call(server2, &server_call2, &request_details2,
-                                      &request_metadata2, cq, cq,
-                                      grpc_core::CqVerifier::tag(0x401)));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_server_request_call(server2, &server_call2, &request_details2,
+                                    &request_metadata2, cq, cq,
+                                    grpc_core::CqVerifier::tag(0x401)));
 
   // second call should now start
   cqv.Expect(grpc_core::CqVerifier::tag(0x201), true);
@@ -410,9 +417,9 @@ int main(int argc, char** argv) {
   op->data.recv_close_on_server.cancelled = &was_cancelled2;
   op->flags = 0;
   op++;
-  GPR_ASSERT(GRPC_CALL_OK ==
-             grpc_call_start_batch(server_call2, ops, (size_t)(op - ops),
-                                   grpc_core::CqVerifier::tag(0x402), nullptr));
+  CHECK_EQ(GRPC_CALL_OK,
+           grpc_call_start_batch(server_call2, ops, (size_t)(op - ops),
+                                 grpc_core::CqVerifier::tag(0x402), nullptr));
 
   // shutdown second server: we should see nothing
   grpc_server_shutdown_and_notify(server2, cq,
