@@ -16,33 +16,32 @@
 //
 //
 
-#include <stdint.h>
+#include <grpc/credentials.h>
+#include <grpc/grpc.h>
+#include <grpc/grpc_security.h>
+#include <grpc/grpc_security_constants.h>
+#include <grpc/impl/channel_arg_names.h>
+#include <grpc/impl/propagation_bits.h>
+#include <grpc/slice.h>
+#include <grpc/status.h>
+#include <grpc/support/time.h>
+#include <gtest/gtest.h>
 #include <string.h>
 
 #include <string>
 
-#include <gtest/gtest.h>
-
-#include <grpc/grpc.h>
-#include <grpc/grpc_security.h>
-#include <grpc/grpc_security_constants.h>
-#include <grpc/impl/propagation_bits.h>
-#include <grpc/slice.h>
-#include <grpc/status.h>
-#include <grpc/support/log.h>
-#include <grpc/support/time.h>
-
+#include "absl/log/check.h"
+#include "absl/types/optional.h"
+#include "src/core/config/config_vars.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gpr/useful.h"
-#include "src/core/lib/gprpp/global_config_generic.h"
-#include "src/core/lib/gprpp/host_port.h"
 #include "src/core/lib/iomgr/error.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
-#include "src/core/lib/iomgr/load_file.h"
-#include "src/core/lib/security/security_connector/ssl_utils_config.h"
+#include "src/core/util/host_port.h"
+#include "src/core/util/useful.h"
 #include "test/core/end2end/cq_verifier.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
+#include "test/core/test_util/tls_utils.h"
 
 #define CA_CERT_PATH "src/core/tsi/test_creds/ca.pem"
 #define CLIENT_CERT_PATH "src/core/tsi/test_creds/client.pem"
@@ -54,60 +53,37 @@ namespace grpc {
 namespace testing {
 namespace {
 
-void* tag(intptr_t t) { return reinterpret_cast<void*>(t); }
-
 gpr_timespec five_seconds_time() { return grpc_timeout_seconds_to_deadline(5); }
 
 grpc_server* server_create(grpc_completion_queue* cq, const char* server_addr) {
-  grpc_slice ca_slice, cert_slice, key_slice;
-  GPR_ASSERT(GRPC_LOG_IF_ERROR("load_file",
-                               grpc_load_file(CA_CERT_PATH, 1, &ca_slice)));
-  GPR_ASSERT(GRPC_LOG_IF_ERROR(
-      "load_file", grpc_load_file(SERVER_CERT_PATH, 1, &cert_slice)));
-  GPR_ASSERT(GRPC_LOG_IF_ERROR("load_file",
-                               grpc_load_file(SERVER_KEY_PATH, 1, &key_slice)));
-  const char* ca_cert =
-      reinterpret_cast<const char*> GRPC_SLICE_START_PTR(ca_slice);
-  const char* server_cert =
-      reinterpret_cast<const char*> GRPC_SLICE_START_PTR(cert_slice);
-  const char* server_key =
-      reinterpret_cast<const char*> GRPC_SLICE_START_PTR(key_slice);
-  grpc_ssl_pem_key_cert_pair pem_cert_key_pair = {server_key, server_cert};
+  std::string ca_cert = grpc_core::testing::GetFileContents(CA_CERT_PATH);
+  std::string server_cert =
+      grpc_core::testing::GetFileContents(SERVER_CERT_PATH);
+  std::string server_key = grpc_core::testing::GetFileContents(SERVER_KEY_PATH);
+  grpc_ssl_pem_key_cert_pair pem_cert_key_pair = {server_key.c_str(),
+                                                  server_cert.c_str()};
   grpc_server_credentials* server_creds = grpc_ssl_server_credentials_create_ex(
-      ca_cert, &pem_cert_key_pair, 1,
+      ca_cert.c_str(), &pem_cert_key_pair, 1,
       GRPC_SSL_REQUEST_CLIENT_CERTIFICATE_AND_VERIFY, nullptr);
 
   grpc_server* server = grpc_server_create(nullptr, nullptr);
   grpc_server_register_completion_queue(server, cq, nullptr);
-  GPR_ASSERT(grpc_server_add_http2_port(server, server_addr, server_creds));
+  CHECK(grpc_server_add_http2_port(server, server_addr, server_creds));
   grpc_server_credentials_release(server_creds);
   grpc_server_start(server);
-
-  grpc_slice_unref(cert_slice);
-  grpc_slice_unref(key_slice);
-  grpc_slice_unref(ca_slice);
   return server;
 }
 
 grpc_channel* client_create(const char* server_addr,
                             grpc_ssl_session_cache* cache) {
-  grpc_slice ca_slice, cert_slice, key_slice;
-  GPR_ASSERT(GRPC_LOG_IF_ERROR("load_file",
-                               grpc_load_file(CA_CERT_PATH, 1, &ca_slice)));
-  GPR_ASSERT(GRPC_LOG_IF_ERROR(
-      "load_file", grpc_load_file(CLIENT_CERT_PATH, 1, &cert_slice)));
-  GPR_ASSERT(GRPC_LOG_IF_ERROR("load_file",
-                               grpc_load_file(CLIENT_KEY_PATH, 1, &key_slice)));
-  const char* ca_cert =
-      reinterpret_cast<const char*> GRPC_SLICE_START_PTR(ca_slice);
-  const char* client_cert =
-      reinterpret_cast<const char*> GRPC_SLICE_START_PTR(cert_slice);
-  const char* client_key =
-      reinterpret_cast<const char*> GRPC_SLICE_START_PTR(key_slice);
-  grpc_ssl_pem_key_cert_pair signed_client_key_cert_pair = {client_key,
-                                                            client_cert};
+  std::string ca_cert = grpc_core::testing::GetFileContents(CA_CERT_PATH);
+  std::string client_cert =
+      grpc_core::testing::GetFileContents(CLIENT_CERT_PATH);
+  std::string client_key = grpc_core::testing::GetFileContents(CLIENT_KEY_PATH);
+  grpc_ssl_pem_key_cert_pair signed_client_key_cert_pair = {
+      client_key.c_str(), client_cert.c_str()};
   grpc_channel_credentials* client_creds = grpc_ssl_credentials_create(
-      ca_cert, &signed_client_key_cert_pair, nullptr, nullptr);
+      ca_cert.c_str(), &signed_client_key_cert_pair, nullptr, nullptr);
 
   grpc_arg args[] = {
       grpc_channel_arg_string_create(
@@ -121,7 +97,7 @@ grpc_channel* client_create(const char* server_addr,
 
   grpc_channel* client =
       grpc_channel_create(server_addr, client_creds, client_args);
-  GPR_ASSERT(client != nullptr);
+  CHECK_NE(client, nullptr);
   grpc_channel_credentials_release(client_creds);
 
   {
@@ -129,9 +105,6 @@ grpc_channel* client_create(const char* server_addr,
     grpc_channel_args_destroy(client_args);
   }
 
-  grpc_slice_unref(cert_slice);
-  grpc_slice_unref(key_slice);
-  grpc_slice_unref(ca_slice);
   return client;
 }
 
@@ -156,7 +129,7 @@ void do_round_trip(grpc_completion_queue* cq, grpc_server* server,
   grpc_call* c = grpc_channel_create_call(
       client, nullptr, GRPC_PROPAGATE_DEFAULTS, cq,
       grpc_slice_from_static_string("/foo"), nullptr, deadline, nullptr);
-  GPR_ASSERT(c);
+  CHECK(c);
 
   grpc_metadata_array_init(&initial_metadata_recv);
   grpc_metadata_array_init(&trailing_metadata_recv);
@@ -186,26 +159,27 @@ void do_round_trip(grpc_completion_queue* cq, grpc_server* server,
   op->flags = 0;
   op->reserved = nullptr;
   op++;
-  error = grpc_call_start_batch(c, ops, static_cast<size_t>(op - ops), tag(1),
-                                nullptr);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  error = grpc_call_start_batch(c, ops, static_cast<size_t>(op - ops),
+                                grpc_core::CqVerifier::tag(1), nullptr);
+  CHECK_EQ(error, GRPC_CALL_OK);
 
   grpc_call* s;
   error = grpc_server_request_call(server, &s, &call_details,
-                                   &request_metadata_recv, cq, cq, tag(101));
-  GPR_ASSERT(GRPC_CALL_OK == error);
-  cqv.Expect(tag(101), true);
+                                   &request_metadata_recv, cq, cq,
+                                   grpc_core::CqVerifier::tag(101));
+  CHECK_EQ(error, GRPC_CALL_OK);
+  cqv.Expect(grpc_core::CqVerifier::tag(101), true);
   cqv.Verify();
 
   grpc_auth_context* auth = grpc_call_auth_context(s);
   grpc_auth_property_iterator it = grpc_auth_context_find_properties_by_name(
       auth, GRPC_SSL_SESSION_REUSED_PROPERTY);
   const grpc_auth_property* property = grpc_auth_property_iterator_next(&it);
-  GPR_ASSERT(property != nullptr);
+  CHECK_NE(property, nullptr);
   if (expect_session_reuse) {
-    GPR_ASSERT(strcmp(property->value, "true") == 0);
+    CHECK_EQ(strcmp(property->value, "true"), 0);
   } else {
-    GPR_ASSERT(strcmp(property->value, "false") == 0);
+    CHECK_EQ(strcmp(property->value, "false"), 0);
   }
   grpc_auth_context_release(auth);
 
@@ -227,12 +201,12 @@ void do_round_trip(grpc_completion_queue* cq, grpc_server* server,
   op->flags = 0;
   op->reserved = nullptr;
   op++;
-  error = grpc_call_start_batch(s, ops, static_cast<size_t>(op - ops), tag(103),
-                                nullptr);
-  GPR_ASSERT(GRPC_CALL_OK == error);
+  error = grpc_call_start_batch(s, ops, static_cast<size_t>(op - ops),
+                                grpc_core::CqVerifier::tag(103), nullptr);
+  CHECK_EQ(error, GRPC_CALL_OK);
 
-  cqv.Expect(tag(103), true);
-  cqv.Expect(tag(1), true);
+  cqv.Expect(grpc_core::CqVerifier::tag(103), true);
+  cqv.Expect(grpc_core::CqVerifier::tag(1), true);
   cqv.Verify();
 
   grpc_metadata_array_destroy(&initial_metadata_recv);
@@ -269,16 +243,17 @@ TEST(H2SessionReuseTest, SingleReuse) {
 
   grpc_ssl_session_cache_destroy(cache);
 
-  GPR_ASSERT(grpc_completion_queue_next(
-                 cq, grpc_timeout_milliseconds_to_deadline(100), nullptr)
-                 .type == GRPC_QUEUE_TIMEOUT);
+  CHECK(grpc_completion_queue_next(
+            cq, grpc_timeout_milliseconds_to_deadline(100), nullptr)
+            .type == GRPC_QUEUE_TIMEOUT);
 
-  grpc_server_shutdown_and_notify(server, cq, tag(1000));
+  grpc_server_shutdown_and_notify(server, cq, grpc_core::CqVerifier::tag(1000));
   grpc_event ev;
   do {
     ev = grpc_completion_queue_next(cq, grpc_timeout_seconds_to_deadline(5),
                                     nullptr);
-  } while (ev.type != GRPC_OP_COMPLETE || ev.tag != tag(1000));
+  } while (ev.type != GRPC_OP_COMPLETE ||
+           ev.tag != grpc_core::CqVerifier::tag(1000));
   grpc_server_destroy(server);
 
   grpc_completion_queue_shutdown(cq);
@@ -292,7 +267,9 @@ TEST(H2SessionReuseTest, SingleReuse) {
 
 int main(int argc, char** argv) {
   grpc::testing::TestEnvironment env(&argc, argv);
-  GPR_GLOBAL_CONFIG_SET(grpc_default_ssl_roots_file_path, CA_CERT_PATH);
+  grpc_core::ConfigVars::Overrides overrides;
+  overrides.default_ssl_roots_file_path = CA_CERT_PATH;
+  grpc_core::ConfigVars::SetOverrides(overrides);
 
   grpc_init();
   ::testing::InitGoogleTest(&argc, argv);
