@@ -16,6 +16,8 @@
 //
 //
 
+#include <grpc/impl/channel_arg_names.h>
+#include <grpc/status.h>
 #include <stdio.h>
 
 #include <utility>
@@ -24,12 +26,8 @@
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "gtest/gtest.h"
-
-#include <grpc/grpc.h>
-#include <grpc/status.h>
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/no_destruct.h"
+#include "src/core/util/no_destruct.h"
 #include "test/core/end2end/end2end_tests.h"
 
 namespace grpc_core {
@@ -97,8 +95,8 @@ void SimpleRequestBody(CoreEnd2endTest& test, size_t index) {
   auto method =
       absl::StrCat("/", hobbit.first, ".", hobbit.second, "/", dragon);
   auto c = test.NewClientCall(method).Create();
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
+  IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
   c.NewBatch(1)
       .SendInitialMetadata({
           {"hobbit-first-name", (*hobbits)[index % hobbits->size()].first},
@@ -111,7 +109,7 @@ void SimpleRequestBody(CoreEnd2endTest& test, size_t index) {
   auto s = test.RequestCall(101);
   test.Expect(101, true);
   test.Step();
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
+  IncomingCloseOnServer client_close;
   s.NewBatch(102)
       .SendInitialMetadata({})
       .SendStatusFromServer(GRPC_STATUS_UNIMPLEMENTED, dragon, {})
