@@ -16,17 +16,18 @@
 //
 //
 
+#include <grpc/impl/channel_arg_names.h>
+#include <grpc/status.h>
 #include <limits.h>
 
+#include <memory>
+
+#include "gmock/gmock.h"
 #include "gtest/gtest.h"
-
-#include <grpc/grpc.h>
-#include <grpc/status.h>
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/util/time.h"
 #include "test/core/end2end/end2end_tests.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 
 #define MAX_CONNECTION_AGE_MS 500
 #define MAX_CONNECTION_AGE_GRACE_MS 2000
@@ -64,8 +65,8 @@ CORE_END2END_TEST(Http2Test, MaxAgeForciblyClose) {
           static_cast<int>(MAX_CONNECTION_AGE_MS *
                            MAX_CONNECTION_AGE_JITTER_MULTIPLIER) +
           MAX_CONNECTION_AGE_GRACE_MS + IMMEDIATE_SHUTDOWN_GRACE_TIME_MS));
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
+  IncomingStatusOnClient server_status;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendCloseFromClient()
@@ -86,7 +87,7 @@ CORE_END2END_TEST(Http2Test, MaxAgeForciblyClose) {
     Expect(1, true);
     Step();
     EXPECT_LT(Timestamp::Now(), expect_shutdown_time);
-    CoreEnd2endTest::IncomingCloseOnServer client_close;
+    IncomingCloseOnServer client_close;
     s.NewBatch(102)
         .SendInitialMetadata({})
         .SendStatusFromServer(GRPC_STATUS_UNIMPLEMENTED, "xyz", {})
@@ -107,6 +108,8 @@ CORE_END2END_TEST(Http2Test, MaxAgeForciblyClose) {
   // The connection should be closed immediately after the max age grace period,
   // the in-progress RPC should fail.
   EXPECT_EQ(server_status.status(), GRPC_STATUS_UNAVAILABLE);
+  EXPECT_THAT(server_status.message(),
+              ::testing::MatchesRegex("max connection age"));
 }
 
 CORE_END2END_TEST(Http2Test, MaxAgeGracefullyClose) {
@@ -121,8 +124,8 @@ CORE_END2END_TEST(Http2Test, MaxAgeGracefullyClose) {
   auto c = NewClientCall("/foo")
                .Timeout(Duration::Seconds(CALL_DEADLINE_S))
                .Create();
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
+  IncomingStatusOnClient server_status;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendCloseFromClient()
@@ -141,7 +144,7 @@ CORE_END2END_TEST(Http2Test, MaxAgeGracefullyClose) {
     // The connection is shutting down gracefully. In-progress rpc should not be
     // closed, hence the completion queue should see nothing here.
     Step(Duration::Seconds(CQ_MAX_CONNECTION_AGE_GRACE_WAIT_TIME_S));
-    CoreEnd2endTest::IncomingCloseOnServer client_close;
+    IncomingCloseOnServer client_close;
     s.NewBatch(102)
         .SendInitialMetadata({})
         .SendStatusFromServer(GRPC_STATUS_UNIMPLEMENTED, "xyz", {})
@@ -160,10 +163,12 @@ CORE_END2END_TEST(Http2Test, MaxAgeGracefullyClose) {
     Expect(101, false);
   }
   Step();
-  // The connection is closed gracefully with goaway, the rpc should still be
-  // completed.
-  EXPECT_EQ(server_status.status(), GRPC_STATUS_UNIMPLEMENTED);
-  EXPECT_EQ(server_status.message(), "xyz");
+  if (got_server) {
+    // The connection is closed gracefully with goaway, the rpc should still be
+    // completed.
+    EXPECT_EQ(server_status.status(), GRPC_STATUS_UNIMPLEMENTED);
+    EXPECT_EQ(server_status.message(), "xyz");
+  }
 }
 
 }  // namespace
