@@ -14,21 +14,19 @@
 // limitations under the License.
 //
 
-#include <initializer_list>
+#include <grpc/impl/channel_arg_names.h>
+#include <grpc/status.h>
+
 #include <memory>
 
 #include "absl/strings/str_format.h"
 #include "absl/types/optional.h"
 #include "gtest/gtest.h"
-
-#include <grpc/impl/channel_arg_names.h>
-#include <grpc/status.h>
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/util/time.h"
 #include "test/core/end2end/end2end_tests.h"
 #include "test/core/end2end/tests/cancel_test_helpers.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc_core {
 namespace {
@@ -48,7 +46,7 @@ void TestRetryCancelDuringDelay(
           "    \"retryPolicy\": {\n"
           "      \"maxAttempts\": 3,\n"
           "      \"initialBackoff\": \"%ds\",\n"
-          "      \"maxBackoff\": \"120s\",\n"
+          "      \"maxBackoff\": \"1000s\",\n"
           "      \"backoffMultiplier\": 1.6,\n"
           "      \"retryableStatusCodes\": [ \"ABORTED\" ]\n"
           "    }\n"
@@ -57,13 +55,13 @@ void TestRetryCancelDuringDelay(
           35 * grpc_test_slowdown_factor())));
   auto expect_finish_before = test.TimestampAfterDuration(Duration::Minutes(2));
   auto c = test.NewClientCall("/service/method")
-               .Timeout(Duration::Seconds(30))
+               .Timeout(Duration::Seconds(20))
                .Create();
   EXPECT_NE(c.GetPeer(), absl::nullopt);
   // Client starts a batch with all 6 ops.
-  CoreEnd2endTest::IncomingMetadata server_initial_metadata;
-  CoreEnd2endTest::IncomingMessage server_message;
-  CoreEnd2endTest::IncomingStatusOnClient server_status;
+  IncomingMetadata server_initial_metadata;
+  IncomingMessage server_message;
+  IncomingStatusOnClient server_status;
   c.NewBatch(1)
       .SendInitialMetadata({})
       .SendMessage("foo")
@@ -76,7 +74,7 @@ void TestRetryCancelDuringDelay(
   test.Step();
   EXPECT_NE(s.GetPeer(), absl::nullopt);
   EXPECT_NE(c.GetPeer(), absl::nullopt);
-  CoreEnd2endTest::IncomingCloseOnServer client_close;
+  IncomingCloseOnServer client_close;
   s.NewBatch(102)
       .SendInitialMetadata({})
       .SendStatusFromServer(GRPC_STATUS_ABORTED, "xyz", {})
@@ -104,10 +102,12 @@ void TestRetryCancelDuringDelay(
 }
 
 CORE_END2END_TEST(RetryTest, CancelDuringDelay) {
+  SKIP_IF_V3();  // Not working yet
   TestRetryCancelDuringDelay(*this, std::make_unique<CancelCancellationMode>());
 }
 
 CORE_END2END_TEST(RetryTest, DeadlineDuringDelay) {
+  SKIP_IF_V3();  // Not working yet
   TestRetryCancelDuringDelay(*this,
                              std::make_unique<DeadlineCancellationMode>());
 }
