@@ -16,14 +16,15 @@
 //
 //
 
-#include "absl/types/optional.h"
-#include "gtest/gtest.h"
-
 #include <grpc/impl/channel_arg_names.h>
 #include <grpc/status.h>
 
+#include <memory>
+
+#include "absl/types/optional.h"
+#include "gtest/gtest.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/util/time.h"
 #include "test/core/end2end/end2end_tests.h"
 
 namespace grpc_core {
@@ -36,6 +37,7 @@ namespace {
 // - client sends a 3-byte message
 // - first attempt gets ABORTED but is not retried
 CORE_END2END_TEST(RetryTest, RetryExceedsBufferSizeInInitialBatch) {
+  SKIP_IF_V3();  // Not working yet
   InitServer(ChannelArgs());
   InitClient(
       ChannelArgs()
