@@ -16,16 +16,15 @@
 //
 //
 
+#include <grpc/impl/channel_arg_names.h>
+#include <grpc/status.h>
+
 #include <string>
 
 #include "absl/types/optional.h"
 #include "gtest/gtest.h"
-
-#include <grpc/grpc.h>
-#include <grpc/status.h>
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/util/time.h"
 #include "test/core/end2end/end2end_tests.h"
 
 namespace grpc_core {
@@ -38,6 +37,7 @@ namespace {
 // - client sends a 100 KiB message
 // - first attempt gets ABORTED but is not retried
 CORE_END2END_TEST(RetryTest, RetryExceedsBufferSizeInSubsequentBatch) {
+  SKIP_IF_V3();
   InitServer(ChannelArgs());
   InitClient(
       ChannelArgs()
