@@ -14,14 +14,15 @@
 // limitations under the License.
 //
 
-#include "absl/types/optional.h"
-#include "gtest/gtest.h"
-
-#include <grpc/grpc.h>
+#include <grpc/impl/channel_arg_names.h>
 #include <grpc/status.h>
 
+#include <memory>
+
+#include "absl/types/optional.h"
+#include "gtest/gtest.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/util/time.h"
 #include "test/core/end2end/end2end_tests.h"
 
 namespace grpc_core {
@@ -31,6 +32,7 @@ namespace {
 // they complete.  This ensures that we don't drop callbacks or cause a
 // memory leak.
 CORE_END2END_TEST(RetryTest, UnrefBeforeRecv) {
+  if (!IsRetryInCallv3Enabled()) SKIP_IF_V3();
   InitServer(ChannelArgs());
   InitClient(ChannelArgs().Set(
       GRPC_ARG_SERVICE_CONFIG,
@@ -80,7 +82,14 @@ CORE_END2END_TEST(RetryTest, UnrefBeforeRecv) {
       .SendStatusFromServer(GRPC_STATUS_FAILED_PRECONDITION, "xyz", {})
       .RecvCloseOnServer(client_close);
   // Server ops complete and client recv ops complete.
-  Expect(2, false);  // Failure!
+  if (GetParam()->feature_mask & FEATURE_MASK_IS_CALL_V3) {
+    // Call-v3 behavior change: the cancellation used to signal different
+    // behavior, but we're effectively just returning a trailers-only response -
+    // and a trailers only response succeeds here, so we're normalizing that.
+    Expect(2, true);
+  } else {
+    Expect(2, false);  // Failure!
+  }
   Expect(102, true);
   Step();
 
