@@ -11,6 +11,10 @@
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
+#include <grpc/event_engine/endpoint_config.h>
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/event_engine/memory_allocator.h>
+#include <grpc/grpc.h>
 #include <grpc/support/port_platform.h>
 
 #include <algorithm>
@@ -24,14 +28,8 @@
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
 #include "gtest/gtest.h"
-
-#include <grpc/event_engine/endpoint_config.h>
-#include <grpc/event_engine/event_engine.h>
-#include <grpc/event_engine/memory_allocator.h>
-#include <grpc/grpc.h>
-
 #include "src/core/lib/event_engine/default_event_engine.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/test_config.h"
 
 namespace {
 
@@ -70,7 +68,7 @@ class DefaultEngineTest : public testing::Test {
         const DNSResolver::ResolverOptions& /* options */) override {
       return nullptr;
     };
-    void Run(Closure* /* closure */) override{};
+    void Run(Closure* /* closure */) override {};
     void Run(absl::AnyInvocable<void()> /* closure */) override{};
     TaskHandle RunAfter(Duration /* when */, Closure* /* closure */) override {
       return {-1, -1};
