@@ -15,6 +15,11 @@
 #ifndef GRPC_TEST_CORE_EVENT_ENGINE_FUZZING_EVENT_ENGINE_FUZZING_EVENT_ENGINE_H
 #define GRPC_TEST_CORE_EVENT_ENGINE_FUZZING_EVENT_ENGINE_FUZZING_EVENT_ENGINE_H
 
+#include <grpc/event_engine/endpoint_config.h>
+#include <grpc/event_engine/event_engine.h>
+#include <grpc/event_engine/memory_allocator.h>
+#include <grpc/event_engine/slice_buffer.h>
+#include <grpc/support/time.h>
 #include <stddef.h>
 
 #include <atomic>
@@ -34,16 +39,10 @@
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/types/optional.h"
-
-#include <grpc/event_engine/endpoint_config.h>
-#include <grpc/event_engine/event_engine.h>
-#include <grpc/event_engine/memory_allocator.h>
-#include <grpc/event_engine/slice_buffer.h>
-#include <grpc/support/time.h>
-
 #include "src/core/lib/event_engine/time_util.h"
-#include "src/core/lib/gprpp/no_destruct.h"
-#include "src/core/lib/gprpp/sync.h"
+#include "src/core/lib/experiments/experiments.h"
+#include "src/core/util/no_destruct.h"
+#include "src/core/util/sync.h"
 #include "test/core/event_engine/fuzzing_event_engine/fuzzing_event_engine.pb.h"
 #include "test/core/test_util/port.h"
 
@@ -72,6 +71,8 @@ class FuzzingEventEngine : public EventEngine {
       ABSL_LOCKS_EXCLUDED(mu_);
   // Repeatedly call Tick() until there is no more work to do.
   void TickUntilIdle() ABSL_LOCKS_EXCLUDED(mu_);
+  // Returns true if idle.
+  bool IsIdle() ABSL_LOCKS_EXCLUDED(mu_);
   // Tick until some time
   void TickUntil(Time t) ABSL_LOCKS_EXCLUDED(mu_);
   // Tick for some duration
@@ -296,6 +297,26 @@ class FuzzingEventEngine : public EventEngine {
   int AllocatePort() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
   // Is the given port in use by any listener?
   bool IsPortUsed(int port) ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
+
+  bool IsIdleLocked() ABSL_EXCLUSIVE_LOCKS_REQUIRED(mu_);
+
+  // Called whenever the time is incremented, and used by
+  // ThreadedFuzzingEventEngine to insert a sleep so that real time passes at
+  // approximately the same rate as FuzzingEventEngine time.
+  // TODO(ctiller): This is very approximate and unsound and we should probably
+  // evaluate whether we want to continue supporting ThreadedFuzzingEventEngine
+  // at all.
+  virtual void OnClockIncremented(Duration) {}
+
+  // We need everything EventEngine to do reasonable timer steps -- without it
+  // we need to do a bunch of evil to make sure both timer systems are ticking
+  // each step.
+  static bool IsSaneTimerEnvironment() {
+    return grpc_core::IsEventEngineClientEnabled() &&
+           grpc_core::IsEventEngineListenerEnabled() &&
+           grpc_core::IsEventEngineDnsEnabled();
+  }
+
   // For the next connection being built, query the list of fuzzer selected
   // write size limits.
   std::queue<size_t> WriteSizesForConnection()
@@ -310,12 +331,13 @@ class FuzzingEventEngine : public EventEngine {
   static grpc_core::NoDestruct<grpc_core::Mutex> now_mu_
       ABSL_ACQUIRED_AFTER(mu_);
 
-  Duration exponential_gate_time_increment_ ABSL_GUARDED_BY(mu_) =
-      std::chrono::milliseconds(1);
   const Duration max_delay_[2];
-  intptr_t next_task_id_ ABSL_GUARDED_BY(mu_);
-  intptr_t current_tick_ ABSL_GUARDED_BY(now_mu_);
-  Time now_ ABSL_GUARDED_BY(now_mu_);
+  intptr_t next_task_id_ ABSL_GUARDED_BY(mu_) = 1;
+  // Start at 5 seconds after the epoch.
+  // This needs to be more than 1, and otherwise is kind of arbitrary.
+  // The grpc_core::Timer code special cases the zero second time period after
+  // epoch to allow for some fancy atomic stuff.
+  Time now_ ABSL_GUARDED_BY(now_mu_) = Time() + std::chrono::seconds(5);
   std::queue<Duration> task_delays_ ABSL_GUARDED_BY(mu_);
   std::map<intptr_t, std::shared_ptr<Task>> tasks_by_id_ ABSL_GUARDED_BY(mu_);
   std::multimap<Time, std::shared_ptr<Task>> tasks_by_time_
@@ -335,6 +357,12 @@ class FuzzingEventEngine : public EventEngine {
   std::atomic<size_t> outstanding_writes_{0};
   std::atomic<size_t> outstanding_reads_{0};
 
+  // TODO(ctiller): these can be removed when IsSaneTimerEnvironment() is
+  // guaranteed to be true.
+  Duration exponential_gate_time_increment_ ABSL_GUARDED_BY(mu_) =
+      std::chrono::milliseconds(1);
+  intptr_t current_tick_ ABSL_GUARDED_BY(now_mu_) = 0;
+
   grpc_core::Mutex run_after_duration_callback_mu_;
   absl::AnyInvocable<void(Duration)> run_after_duration_callback_
       ABSL_GUARDED_BY(run_after_duration_callback_mu_);
@@ -350,11 +378,7 @@ class ThreadedFuzzingEventEngine : public FuzzingEventEngine {
                            fuzzing_event_engine::Actions()),
         main_([this, max_time]() {
           while (!done_.load()) {
-            if (max_time > Duration::zero()) {
-              absl::SleepFor(absl::Milliseconds(
-                  grpc_event_engine::experimental::Milliseconds(max_time)));
-            }
-            Tick();
+            Tick(max_time);
           }
         }) {}
 
@@ -364,6 +388,11 @@ class ThreadedFuzzingEventEngine : public FuzzingEventEngine {
   }
 
  private:
+  void OnClockIncremented(Duration duration) override {
+    absl::SleepFor(absl::Milliseconds(
+        1 + grpc_event_engine::experimental::Milliseconds(duration)));
+  }
+
   std::atomic<bool> done_{false};
   std::thread main_;
 };
