@@ -13,6 +13,9 @@
 // limitations under the License.
 #include "src/core/lib/event_engine/thread_pool/thread_pool.h"
 
+#include <grpc/grpc.h>
+#include <grpc/support/thd_id.h>
+
 #include <atomic>
 #include <chrono>
 #include <cmath>
@@ -26,15 +29,11 @@
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
 #include "gtest/gtest.h"
-
-#include <grpc/grpc.h>
-#include <grpc/support/thd_id.h>
-
 #include "src/core/lib/event_engine/thread_pool/thread_count.h"
 #include "src/core/lib/event_engine/thread_pool/work_stealing_thread_pool.h"
-#include "src/core/lib/gprpp/notification.h"
-#include "src/core/lib/gprpp/thd.h"
-#include "src/core/lib/gprpp/time.h"
+#include "src/core/util/notification.h"
+#include "src/core/util/thd.h"
+#include "src/core/util/time.h"
 #include "test/core/test_util/test_config.h"
 
 namespace grpc_event_engine {
@@ -96,7 +95,7 @@ TYPED_TEST(ThreadPoolTest, ForkStressTest) {
   // This test exercises a subset of the fork logic, the pieces we can control
   // without an actual OS fork.
   constexpr int expected_runcount = 1000;
-  constexpr absl::Duration fork_freqency{absl::Milliseconds(50)};
+  constexpr absl::Duration fork_frequency{absl::Milliseconds(50)};
   constexpr int num_closures_between_forks{100};
   TypeParam pool(8);
   std::atomic<int> runcount{0};
@@ -122,7 +121,7 @@ TYPED_TEST(ThreadPoolTest, ForkStressTest) {
   // simulate multiple forks at a fixed frequency
   int curr_runcount = 0;
   while (curr_runcount < expected_runcount) {
-    absl::SleepFor(fork_freqency);
+    absl::SleepFor(fork_frequency);
     curr_runcount = runcount.load(std::memory_order_relaxed);
     int curr_forkcount = fork_count.load(std::memory_order_relaxed);
     if (curr_forkcount * num_closures_between_forks > curr_runcount) {
