@@ -14,7 +14,9 @@
 
 #include "src/core/lib/promise/pipe.h"
 
-#include <initializer_list>
+#include <grpc/event_engine/memory_allocator.h>
+#include <grpc/grpc.h>
+
 #include <memory>
 #include <tuple>
 #include <utility>
@@ -23,18 +25,14 @@
 #include "absl/status/status.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
-
-#include <grpc/event_engine/memory_allocator.h>
-#include <grpc/grpc.h>
-
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/lib/promise/activity.h"
 #include "src/core/lib/promise/join.h"
 #include "src/core/lib/promise/map.h"
 #include "src/core/lib/promise/seq.h"
 #include "src/core/lib/resource_quota/memory_quota.h"
 #include "src/core/lib/resource_quota/resource_quota.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/ref_counted_ptr.h"
 #include "test/core/promise/test_wakeup_schedulers.h"
 
 using testing::MockFunction;
@@ -42,13 +40,7 @@ using testing::StrictMock;
 
 namespace grpc_core {
 
-class PipeTest : public ::testing::Test {
- protected:
-  MemoryAllocator memory_allocator_ = MemoryAllocator(
-      ResourceQuota::Default()->memory_quota()->CreateMemoryAllocator("test"));
-};
-
-TEST_F(PipeTest, CanSendAndReceive) {
+TEST(PipeTest, CanSendAndReceive) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -69,10 +61,10 @@ TEST_F(PipeTest, CanSendAndReceive) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanInterceptAndMapAtSender) {
+TEST(PipeTest, CanInterceptAndMapAtSender) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -94,10 +86,10 @@ TEST_F(PipeTest, CanInterceptAndMapAtSender) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanInterceptAndMapAtReceiver) {
+TEST(PipeTest, CanInterceptAndMapAtReceiver) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -119,10 +111,10 @@ TEST_F(PipeTest, CanInterceptAndMapAtReceiver) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, InterceptionOrderingIsCorrect) {
+TEST(PipeTest, InterceptionOrderingIsCorrect) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -155,10 +147,10 @@ TEST_F(PipeTest, InterceptionOrderingIsCorrect) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanReceiveAndSend) {
+TEST(PipeTest, CanReceiveAndSend) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -179,10 +171,10 @@ TEST_F(PipeTest, CanReceiveAndSend) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanSeeClosedOnSend) {
+TEST(PipeTest, CanSeeClosedOnSend) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -209,10 +201,10 @@ TEST_F(PipeTest, CanSeeClosedOnSend) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanSeeClosedOnReceive) {
+TEST(PipeTest, CanSeeClosedOnReceive) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -241,10 +233,10 @@ TEST_F(PipeTest, CanSeeClosedOnReceive) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanCloseSend) {
+TEST(PipeTest, CanCloseSend) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -271,10 +263,10 @@ TEST_F(PipeTest, CanCloseSend) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanCloseWithErrorSend) {
+TEST(PipeTest, CanCloseWithErrorSend) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -301,10 +293,10 @@ TEST_F(PipeTest, CanCloseWithErrorSend) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanCloseWithErrorRecv) {
+TEST(PipeTest, CanCloseWithErrorRecv) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -331,10 +323,10 @@ TEST_F(PipeTest, CanCloseWithErrorRecv) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanCloseSendWithInterceptor) {
+TEST(PipeTest, CanCloseSendWithInterceptor) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -362,10 +354,10 @@ TEST_F(PipeTest, CanCloseSendWithInterceptor) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanCancelSendWithInterceptor) {
+TEST(PipeTest, CanCancelSendWithInterceptor) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -390,10 +382,10 @@ TEST_F(PipeTest, CanCancelSendWithInterceptor) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
-TEST_F(PipeTest, CanFlowControlThroughManyStages) {
+TEST(PipeTest, CanFlowControlThroughManyStages) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   auto done = std::make_shared<bool>(false);
@@ -438,11 +430,11 @@ TEST_F(PipeTest, CanFlowControlThroughManyStages) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
   ASSERT_TRUE(*done);
 }
 
-TEST_F(PipeTest, AwaitClosedWorks) {
+TEST(PipeTest, AwaitClosedWorks) {
   StrictMock<MockFunction<void(absl::Status)>> on_done;
   EXPECT_CALL(on_done, Call(absl::OkStatus()));
   MakeActivity(
@@ -469,7 +461,7 @@ TEST_F(PipeTest, AwaitClosedWorks) {
       },
       NoWakeupScheduler(),
       [&on_done](absl::Status status) { on_done.Call(std::move(status)); },
-      MakeScopedArena(1024, &memory_allocator_));
+      SimpleArenaAllocator()->MakeArena());
 }
 
 class FakeActivity final : public Activity {
@@ -484,7 +476,7 @@ class FakeActivity final : public Activity {
   }
 };
 
-TEST_F(PipeTest, PollAckWaitsForReadyClosed) {
+TEST(PipeTest, PollAckWaitsForReadyClosed) {
   FakeActivity().Run([]() {
     pipe_detail::Center<int> c;
     int i = 1;
