@@ -14,6 +14,8 @@
 
 #include "src/core/lib/promise/sleep.h"
 
+#include <grpc/grpc.h>
+
 #include <chrono>
 #include <cstddef>
 #include <memory>
@@ -23,15 +25,14 @@
 #include "absl/log/log.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
-
-#include <grpc/grpc.h>
-
 #include "src/core/lib/event_engine/default_event_engine.h"
-#include "src/core/lib/gprpp/notification.h"
-#include "src/core/lib/gprpp/orphanable.h"
+#include "src/core/lib/event_engine/event_engine_context.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/promise/exec_ctx_wakeup_scheduler.h"
 #include "src/core/lib/promise/race.h"
+#include "src/core/lib/resource_quota/arena.h"
+#include "src/core/util/notification.h"
+#include "src/core/util/orphanable.h"
 #include "test/core/event_engine/mock_event_engine.h"
 #include "test/core/promise/test_wakeup_schedulers.h"
 
@@ -49,6 +50,12 @@ using testing::StrictMock;
 namespace grpc_core {
 namespace {
 
+RefCountedPtr<Arena> ArenaWithEventEngine(EventEngine* ee) {
+  auto arena = SimpleArenaAllocator()->MakeArena();
+  arena->SetContext<grpc_event_engine::experimental::EventEngine>(ee);
+  return arena;
+}
+
 TEST(Sleep, Zzzz) {
   ExecCtx exec_ctx;
   Notification done;
@@ -61,7 +68,7 @@ TEST(Sleep, Zzzz) {
         EXPECT_EQ(r, absl::OkStatus());
         done.Notify();
       },
-      engine.get());
+      ArenaWithEventEngine(engine.get()));
   done.WaitForNotification();
   exec_ctx.InvalidateNow();
   EXPECT_GE(Timestamp::Now(), done_time);
@@ -84,7 +91,7 @@ TEST(Sleep, OverlyEagerEventEngine) {
         EXPECT_EQ(r, absl::OkStatus());
         done = true;
       },
-      static_cast<EventEngine*>(&mock_event_engine));
+      ArenaWithEventEngine(static_cast<EventEngine*>(&mock_event_engine)));
   Mock::VerifyAndClearExpectations(&mock_event_engine);
   EXPECT_NE(wakeup, nullptr);
   EXPECT_FALSE(done);
@@ -106,7 +113,7 @@ TEST(Sleep, AlreadyDone) {
         EXPECT_EQ(r, absl::OkStatus());
         done.Notify();
       },
-      engine.get());
+      ArenaWithEventEngine(engine.get()));
   done.WaitForNotification();
 }
 
@@ -123,7 +130,7 @@ TEST(Sleep, Cancel) {
         EXPECT_EQ(r, absl::CancelledError());
         done.Notify();
       },
-      engine.get());
+      ArenaWithEventEngine(engine.get()));
   done.WaitForNotification();
   exec_ctx.InvalidateNow();
   EXPECT_LT(Timestamp::Now(), done_time);
@@ -143,7 +150,7 @@ TEST(Sleep, MoveSemantics) {
         EXPECT_EQ(r, absl::OkStatus());
         done.Notify();
       },
-      engine.get());
+      ArenaWithEventEngine(engine.get()));
   done.WaitForNotification();
   exec_ctx.InvalidateNow();
   EXPECT_GE(Timestamp::Now(), done_time);
@@ -163,7 +170,7 @@ TEST(Sleep, StressTest) {
         Sleep(Timestamp::Now() + Duration::Seconds(1)),
         ExecCtxWakeupScheduler(),
         [notification](absl::Status /*r*/) { notification->Notify(); },
-        engine.get());
+        ArenaWithEventEngine(engine.get()));
     notifications.push_back(std::move(notification));
     activities.push_back(std::move(activity));
   }
@@ -172,7 +179,7 @@ TEST(Sleep, StressTest) {
   for (size_t i = 0; i < kNumActivities / 2; i++) {
     notifications[i]->WaitForNotification();
     activities[i].reset();
-    activities[i + kNumActivities / 2].reset();
+    activities[i + (kNumActivities / 2)].reset();
     exec_ctx.Flush();
   }
 }
