@@ -18,27 +18,26 @@
 
 #include "src/core/lib/security/security_connector/tls/tls_security_connector.h"
 
+#include <gmock/gmock.h>
+#include <grpc/credentials.h>
+#include <grpc/support/alloc.h>
+#include <grpc/support/string_util.h>
+#include <gtest/gtest.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
-#include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
-#include <grpc/support/string_util.h>
-
+#include "absl/log/check.h"
+#include "src/core/config/config_vars.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/config/config_vars.h"
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/unique_type_name.h"
 #include "src/core/lib/security/context/security_context.h"
 #include "src/core/lib/security/credentials/tls/grpc_tls_certificate_provider.h"
 #include "src/core/lib/security/credentials/tls/grpc_tls_credentials_options.h"
 #include "src/core/lib/security/credentials/tls/tls_credentials.h"
 #include "src/core/tsi/transport_security.h"
-#include "test/core/util/test_config.h"
-#include "test/core/util/tls_utils.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/unique_type_name.h"
+#include "test/core/test_util/test_config.h"
+#include "test/core/test_util/tls_utils.h"
 
 #define CA_CERT_PATH "src/core/tsi/test_creds/ca.pem"
 #define CLIENT_CERT_PATH "src/core/tsi/test_creds/multi-domain.pem"
@@ -81,7 +80,7 @@ class TlsSecurityConnectorTest : public ::testing::Test {
 
   static std::string GetErrorMsg(grpc_error_handle error) {
     std::string error_str;
-    GPR_ASSERT(
+    CHECK(
         grpc_error_get_str(error, StatusStrProperty::kDescription, &error_str));
     return error_str;
   }
@@ -367,13 +366,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -403,13 +402,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
@@ -556,13 +555,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -592,13 +591,13 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
@@ -630,29 +629,29 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a full TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(7, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
-             TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_SECURITY_LEVEL_PEER_PROPERTY,
-                 tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
-                 &peer.properties[3]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
-                 &peer.properties[4]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[5]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
-                 &peer.properties[6]) == TSI_OK);
+  CHECK(tsi_construct_peer(7, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
+        TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_SECURITY_LEVEL_PEER_PROPERTY,
+            tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
+            &peer.properties[3]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
+            &peer.properties[4]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[5]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
+            &peer.properties[6]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -678,29 +677,29 @@ TEST_F(TlsSecurityConnectorTest,
   EXPECT_NE(tls_connector->ClientHandshakerFactoryForTesting(), nullptr);
   // Construct a full TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(7, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.com",
-                 &peer.properties[1]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
-             TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_SECURITY_LEVEL_PEER_PROPERTY,
-                 tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
-                 &peer.properties[3]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
-                 &peer.properties[4]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "*.com",
-                 &peer.properties[5]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
-                 &peer.properties[6]) == TSI_OK);
+  CHECK(tsi_construct_peer(7, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.com",
+            &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_PROPERTY, "pem_cert", &peer.properties[2]) ==
+        TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_SECURITY_LEVEL_PEER_PROPERTY,
+            tsi_security_level_to_string(TSI_PRIVACY_AND_INTEGRITY),
+            &peer.properties[3]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_PEM_CERT_CHAIN_PROPERTY, "pem_cert_chain",
+            &peer.properties[4]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "*.com",
+            &peer.properties[5]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_ALTERNATIVE_NAME_PEER_PROPERTY, "foo.baz.com",
+            &peer.properties[6]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: Hostname "
@@ -987,13 +986,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -1018,13 +1017,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
@@ -1053,13 +1052,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   ExecCtx exec_ctx;
   grpc_closure* on_peer_checked = GRPC_CLOSURE_CREATE(
@@ -1086,13 +1085,13 @@ TEST_F(TlsSecurityConnectorTest,
   auto connector = credentials->create_security_connector(ChannelArgs());
   // Construct a basic TSI Peer.
   tsi_peer peer;
-  GPR_ASSERT(tsi_construct_peer(2, &peer) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL,
-                                                "h2", strlen("h2"),
-                                                &peer.properties[0]) == TSI_OK);
-  GPR_ASSERT(tsi_construct_string_peer_property_from_cstring(
-                 TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
-                 &peer.properties[1]) == TSI_OK);
+  CHECK(tsi_construct_peer(2, &peer) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property(TSI_SSL_ALPN_SELECTED_PROTOCOL, "h2",
+                                           strlen("h2"),
+                                           &peer.properties[0]) == TSI_OK);
+  CHECK(tsi_construct_string_peer_property_from_cstring(
+            TSI_X509_SUBJECT_COMMON_NAME_PEER_PROPERTY, "foo.bar.com",
+            &peer.properties[1]) == TSI_OK);
   RefCountedPtr<grpc_auth_context> auth_context;
   const char* expected_error_msg =
       "Custom verification check failed with error: UNAUTHENTICATED: "
