@@ -14,6 +14,8 @@
 // limitations under the License.
 //
 
+#include <grpc/grpc.h>
+
 #include <map>
 #include <memory>
 #include <set>
@@ -27,47 +29,66 @@
 #include "absl/strings/string_view.h"
 #include "absl/time/time.h"
 #include "absl/types/optional.h"
-
-#include <grpc/grpc.h>
-
-#include "src/core/lib/event_engine/default_event_engine.h"
-#include "src/core/lib/gprpp/orphanable.h"
-#include "src/core/lib/gprpp/ref_counted_ptr.h"
+#include "envoy/service/discovery/v3/discovery.pb.h"
+#include "src/core/lib/iomgr/timer_manager.h"
+#include "src/core/util/orphanable.h"
+#include "src/core/util/ref_counted_ptr.h"
 #include "src/core/xds/grpc/xds_bootstrap_grpc.h"
 #include "src/core/xds/grpc/xds_cluster.h"
+#include "src/core/xds/grpc/xds_cluster_parser.h"
 #include "src/core/xds/grpc/xds_endpoint.h"
+#include "src/core/xds/grpc/xds_endpoint_parser.h"
 #include "src/core/xds/grpc/xds_listener.h"
+#include "src/core/xds/grpc/xds_listener_parser.h"
 #include "src/core/xds/grpc/xds_route_config.h"
+#include "src/core/xds/grpc/xds_route_config_parser.h"
 #include "src/core/xds/xds_client/xds_bootstrap.h"
 #include "src/core/xds/xds_client/xds_client.h"
 #include "src/libfuzzer/libfuzzer_macro.h"
-#include "src/proto/grpc/testing/xds/v3/discovery.pb.h"
+#include "test/core/event_engine/event_engine_test_utils.h"
+#include "test/core/event_engine/fuzzing_event_engine/fuzzing_event_engine.h"
 #include "test/core/xds/xds_client_fuzzer.pb.h"
 #include "test/core/xds/xds_client_test_peer.h"
 #include "test/core/xds/xds_transport_fake.h"
 
+using grpc_event_engine::experimental::FuzzingEventEngine;
+
 namespace grpc_core {
 
 class Fuzzer {
  public:
-  explicit Fuzzer(absl::string_view bootstrap_json) {
+  Fuzzer(absl::string_view bootstrap_json,
+         const fuzzing_event_engine::Actions& fuzzing_ee_actions) {
+    event_engine_ = std::make_shared<FuzzingEventEngine>(
+        FuzzingEventEngine::Options(), fuzzing_ee_actions);
+    grpc_timer_manager_set_start_threaded(false);
+    grpc_init();
     auto bootstrap = GrpcXdsBootstrap::Create(bootstrap_json);
     if (!bootstrap.ok()) {
       LOG(ERROR) << "error creating bootstrap: " << bootstrap.status();
       // Leave xds_client_ unset, so Act() will be a no-op.
       return;
     }
-    auto transport_factory = MakeOrphanable<FakeXdsTransportFactory>(
-        []() { Crash("Multiple concurrent reads"); });
-    transport_factory->SetAutoCompleteMessagesFromClient(false);
-    transport_factory->SetAbortOnUndrainedMessages(false);
-    transport_factory_ = transport_factory.get();
+    transport_factory_ = MakeRefCounted<FakeXdsTransportFactory>(
+        []() { Crash("Multiple concurrent reads"); }, event_engine_);
+    transport_factory_->SetAutoCompleteMessagesFromClient(false);
+    transport_factory_->SetAbortOnUndrainedMessages(false);
     xds_client_ = MakeRefCounted<XdsClient>(
-        std::move(*bootstrap), std::move(transport_factory),
-        grpc_event_engine::experimental::GetDefaultEventEngine(),
+        std::move(*bootstrap), transport_factory_, event_engine_,
         /*metrics_reporter=*/nullptr, "foo agent", "foo version");
   }
 
+  ~Fuzzer() {
+    transport_factory_.reset();
+    xds_client_.reset();
+    event_engine_->FuzzingDone();
+    event_engine_->TickUntilIdle();
+    event_engine_->UnsetGlobalHooks();
+    grpc_event_engine::experimental::WaitForSingleOwner(
+        std::move(event_engine_));
+    grpc_shutdown_blocking();
+  }
+
   void Act(const xds_client_fuzzer::Action& action) {
     if (xds_client_ == nullptr) return;
     switch (action.action_type_case()) {
@@ -168,27 +189,23 @@ class Fuzzer {
         : resource_name_(std::move(resource_name)) {}
 
     void OnResourceChanged(
-        std::shared_ptr<const typename ResourceType::ResourceType> resource,
+        absl::StatusOr<
+            std::shared_ptr<const typename ResourceType::ResourceType>>
+            resource,
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
       LOG(INFO) << "==> OnResourceChanged(" << ResourceType::Get()->type_url()
-                << " " << resource_name_ << "): " << resource->ToString();
+                << " " << resource_name_ << "): "
+                << (resource.ok() ? (*resource)->ToString()
+                                  : resource.status().ToString());
     }
 
-    void OnError(
+    void OnAmbientError(
         absl::Status status,
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
-      LOG(INFO) << "==> OnError(" << ResourceType::Get()->type_url() << " "
-                << resource_name_ << "): " << status;
-    }
-
-    void OnResourceDoesNotExist(
-        RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
-        override {
-      LOG(INFO) << "==> OnResourceDoesNotExist("
-                << ResourceType::Get()->type_url() << " " << resource_name_
-                << ")";
+      LOG(INFO) << "==> OnAmbientError(" << ResourceType::Get()->type_url()
+                << " " << resource_name_ << "): " << status;
     }
 
    private:
@@ -273,8 +290,7 @@ class Fuzzer {
     if (xds_server == nullptr) return nullptr;
     const char* method = StreamIdMethod(stream_id);
     if (method == nullptr) return nullptr;
-    return transport_factory_->WaitForStream(*xds_server, method,
-                                             absl::ZeroDuration());
+    return transport_factory_->WaitForStream(*xds_server, method);
   }
 
   static std::string StreamIdString(
@@ -290,7 +306,7 @@ class Fuzzer {
     auto stream = GetStream(stream_id);
     if (stream == nullptr) return;
     LOG(INFO) << "    stream=" << stream.get();
-    auto message = stream->WaitForMessageFromClient(absl::ZeroDuration());
+    auto message = stream->WaitForMessageFromClient();
     if (message.has_value()) {
       LOG(INFO) << "    completing send_message";
       stream->CompleteSendMessageFromClient(ok);
@@ -317,8 +333,9 @@ class Fuzzer {
     stream->MaybeSendStatusToClient(std::move(status));
   }
 
+  std::shared_ptr<FuzzingEventEngine> event_engine_;
   RefCountedPtr<XdsClient> xds_client_;
-  FakeXdsTransportFactory* transport_factory_;
+  RefCountedPtr<FakeXdsTransportFactory> transport_factory_;
 
   // Maps of currently active watchers for each resource type, keyed by
   // resource name.
@@ -332,11 +349,10 @@ class Fuzzer {
 
 bool squelch = true;
 
-DEFINE_PROTO_FUZZER(const xds_client_fuzzer::Message& message) {
-  grpc_init();
-  grpc_core::Fuzzer fuzzer(message.bootstrap());
+DEFINE_PROTO_FUZZER(const xds_client_fuzzer::Msg& message) {
+  grpc_core::Fuzzer fuzzer(message.bootstrap(),
+                           message.fuzzing_event_engine_actions());
   for (int i = 0; i < message.actions_size(); i++) {
     fuzzer.Act(message.actions(i));
   }
-  grpc_shutdown();
 }
