@@ -16,15 +16,17 @@
 //
 //
 
-#include <condition_variable>
-
-#include <gtest/gtest.h>
-
-#include "absl/strings/str_format.h"
+#ifndef GRPC_TEST_CPP_END2END_INTERCEPTORS_UTIL_H
+#define GRPC_TEST_CPP_END2END_INTERCEPTORS_UTIL_H
 
 #include <grpcpp/channel.h>
+#include <gtest/gtest.h>
 
-#include "src/core/lib/gprpp/crash.h"
+#include <condition_variable>
+
+#include "absl/log/check.h"
+#include "absl/strings/str_format.h"
+#include "src/core/util/crash.h"
 #include "src/proto/grpc/testing/echo.grpc.pb.h"
 #include "test/cpp/util/string_ref_helper.h"
 
@@ -290,7 +292,7 @@ class Verifier {
   // This version of Verify allows optionally ignoring the
   // outcome of the expectation
   void Verify(CompletionQueue* cq, bool ignore_ok) {
-    GPR_ASSERT(!expectations_.empty() || !maybe_expectations_.empty());
+    CHECK(!expectations_.empty() || !maybe_expectations_.empty());
     while (!expectations_.empty()) {
       Next(cq, ignore_ok);
     }
@@ -354,3 +356,5 @@ class Verifier {
 
 }  // namespace testing
 }  // namespace grpc
+
+#endif  // GRPC_TEST_CPP_END2END_INTERCEPTORS_UTIL_H
