@@ -15,11 +15,6 @@
 //
 
 #include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
-#include "absl/strings/str_cat.h"
-#include "absl/types/optional.h"
-
 #include <grpc/grpc.h>
 #include <grpcpp/channel.h>
 #include <grpcpp/client_context.h>
@@ -27,15 +22,26 @@
 #include <grpcpp/ext/call_metric_recorder.h>
 #include <grpcpp/ext/orca_service.h>
 #include <grpcpp/ext/server_metric_recorder.h>
+#include <grpcpp/generic/generic_stub.h>
 #include <grpcpp/server.h>
 #include <grpcpp/server_builder.h>
 #include <grpcpp/server_context.h>
+#include <grpcpp/support/byte_buffer.h>
+#include <grpcpp/support/status.h>
+#include <gtest/gtest.h>
 
-#include "src/core/lib/gprpp/time.h"
+#include <memory>
+
+#include "absl/log/log.h"
+#include "absl/strings/str_cat.h"
+#include "absl/time/time.h"
+#include "absl/types/optional.h"
+#include "src/core/util/notification.h"
+#include "src/core/util/time.h"
 #include "src/proto/grpc/testing/xds/v3/orca_service.grpc.pb.h"
 #include "src/proto/grpc/testing/xds/v3/orca_service.pb.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
 
 using xds::data::orca::v3::OrcaLoadReport;
 using xds::service::orca::v3::OpenRcaService;
@@ -77,6 +83,7 @@ class OrcaServiceEnd2endTest : public ::testing::Test {
             grpc_core::Duration::Milliseconds(750) *
             grpc_test_slowdown_factor();
         auto elapsed = now - *last_response_time_;
+        LOG(INFO) << "received ORCA response after " << elapsed;
         EXPECT_GE(elapsed, requested_interval_ - fudge_factor)
             << elapsed.ToString();
         EXPECT_LE(elapsed, requested_interval_ + fudge_factor)
@@ -93,6 +100,34 @@ class OrcaServiceEnd2endTest : public ::testing::Test {
     absl::optional<grpc_core::Timestamp> last_response_time_;
   };
 
+  class GenericOrcaClientReactor
+      : public grpc::ClientBidiReactor<grpc::ByteBuffer, grpc::ByteBuffer> {
+   public:
+    explicit GenericOrcaClientReactor(GenericStub* stub) : stub_(stub) {}
+
+    void Prepare() {
+      stub_->PrepareBidiStreamingCall(
+          &cli_ctx_, "/xds.service.orca.v3.OpenRcaService/StreamCoreMetrics",
+          StubOptions(), this);
+    }
+
+    grpc::Status Await() {
+      notification_.WaitForNotification();
+      return status_;
+    }
+
+    void OnDone(const grpc::Status& s) override {
+      status_ = s;
+      notification_.Notify();
+    }
+
+   private:
+    GenericStub* stub_;
+    grpc::ClientContext cli_ctx_;
+    grpc_core::Notification notification_;
+    grpc::Status status_;
+  };
+
   OrcaServiceEnd2endTest()
       : server_metric_recorder_(ServerMetricRecorder::Create()),
         orca_service_(server_metric_recorder_.get(),
@@ -101,21 +136,19 @@ class OrcaServiceEnd2endTest : public ::testing::Test {
     std::string server_address =
         absl::StrCat("localhost:", grpc_pick_unused_port_or_die());
     ServerBuilder builder;
-    builder.AddListeningPort(server_address, grpc::InsecureServerCredentials());
+    builder.AddListeningPort(server_address, InsecureServerCredentials());
     builder.RegisterService(&orca_service_);
     server_ = builder.BuildAndStart();
-    gpr_log(GPR_INFO, "server started on %s", server_address_.c_str());
-    auto channel = CreateChannel(server_address, InsecureChannelCredentials());
-    stub_ = OpenRcaService::NewStub(channel);
+    LOG(INFO) << "server started on " << server_address;
+    channel_ = CreateChannel(server_address, InsecureChannelCredentials());
   }
 
   ~OrcaServiceEnd2endTest() override { server_->Shutdown(); }
 
-  std::string server_address_;
   std::unique_ptr<ServerMetricRecorder> server_metric_recorder_;
   OrcaService orca_service_;
   std::unique_ptr<Server> server_;
-  std::unique_ptr<OpenRcaService::Stub> stub_;
+  std::shared_ptr<Channel> channel_;
 };
 
 TEST_F(OrcaServiceEnd2endTest, Basic) {
@@ -123,48 +156,55 @@ TEST_F(OrcaServiceEnd2endTest, Basic) {
   constexpr char kMetricName2[] = "bar";
   constexpr char kMetricName3[] = "baz";
   constexpr char kMetricName4[] = "quux";
+  auto stub = OpenRcaService::NewStub(channel_);
   // Start stream1 with 5s interval and stream2 with 2.5s interval.
   // Throughout the test, we should get two responses on stream2 for
   // every one response on stream1.
-  Stream stream1(stub_.get(), grpc_core::Duration::Milliseconds(5000));
-  Stream stream2(stub_.get(), grpc_core::Duration::Milliseconds(2500));
+  Stream stream1(stub.get(), grpc_core::Duration::Milliseconds(5000));
+  Stream stream2(stub.get(), grpc_core::Duration::Milliseconds(2500));
   auto ReadResponses = [&](std::function<void(const OrcaLoadReport&)> checker) {
-    gpr_log(GPR_INFO, "reading response from stream1");
+    LOG(INFO) << "reading response from stream1";
     OrcaLoadReport response = stream1.ReadResponse();
     checker(response);
-    gpr_log(GPR_INFO, "reading response from stream2");
+    LOG(INFO) << "reading response from stream2";
     response = stream2.ReadResponse();
     checker(response);
-    gpr_log(GPR_INFO, "reading response from stream2");
+    LOG(INFO) << "reading response from stream2";
     response = stream2.ReadResponse();
     checker(response);
   };
   // Initial response should not have any values populated.
   ReadResponses([](const OrcaLoadReport& response) {
+    EXPECT_EQ(response.application_utilization(), 0);
     EXPECT_EQ(response.cpu_utilization(), 0);
     EXPECT_EQ(response.mem_utilization(), 0);
     EXPECT_THAT(response.utilization(), ::testing::UnorderedElementsAre());
   });
-  // Now set CPU utilization on the server.
-  server_metric_recorder_->SetCpuUtilization(0.5);
+  // Now set app utilization on the server.
+  server_metric_recorder_->SetApplicationUtilization(0.5);
   ReadResponses([](const OrcaLoadReport& response) {
-    EXPECT_EQ(response.cpu_utilization(), 0.5);
+    EXPECT_EQ(response.application_utilization(), 0.5);
+    EXPECT_EQ(response.cpu_utilization(), 0);
     EXPECT_EQ(response.mem_utilization(), 0);
     EXPECT_THAT(response.utilization(), ::testing::UnorderedElementsAre());
   });
-  // Update CPU utilization and set memory utilization.
-  server_metric_recorder_->SetCpuUtilization(1.8);
+  // Update app utilization and set CPU and memory utilization.
+  server_metric_recorder_->SetApplicationUtilization(1.8);
+  server_metric_recorder_->SetCpuUtilization(0.3);
   server_metric_recorder_->SetMemoryUtilization(0.4);
   ReadResponses([](const OrcaLoadReport& response) {
-    EXPECT_EQ(response.cpu_utilization(), 1.8);
+    EXPECT_EQ(response.application_utilization(), 1.8);
+    EXPECT_EQ(response.cpu_utilization(), 0.3);
     EXPECT_EQ(response.mem_utilization(), 0.4);
     EXPECT_THAT(response.utilization(), ::testing::UnorderedElementsAre());
   });
-  // Unset CPU and memory utilization and set a named utilization.
+  // Unset app, CPU, and memory utilization and set a named utilization.
+  server_metric_recorder_->ClearApplicationUtilization();
   server_metric_recorder_->ClearCpuUtilization();
   server_metric_recorder_->ClearMemoryUtilization();
   server_metric_recorder_->SetNamedUtilization(kMetricName1, 0.3);
   ReadResponses([&](const OrcaLoadReport& response) {
+    EXPECT_EQ(response.application_utilization(), 0);
     EXPECT_EQ(response.cpu_utilization(), 0);
     EXPECT_EQ(response.mem_utilization(), 0);
     EXPECT_THAT(
@@ -176,6 +216,7 @@ TEST_F(OrcaServiceEnd2endTest, Basic) {
   server_metric_recorder_->SetNamedUtilization(kMetricName2, 0.2);
   server_metric_recorder_->SetNamedUtilization(kMetricName3, 0.1);
   ReadResponses([&](const OrcaLoadReport& response) {
+    EXPECT_EQ(response.application_utilization(), 0);
     EXPECT_EQ(response.cpu_utilization(), 0);
     EXPECT_EQ(response.mem_utilization(), 0);
     EXPECT_THAT(
@@ -187,6 +228,7 @@ TEST_F(OrcaServiceEnd2endTest, Basic) {
   server_metric_recorder_->SetAllNamedUtilization(
       {{kMetricName2, 0.5}, {kMetricName4, 0.9}});
   ReadResponses([&](const OrcaLoadReport& response) {
+    EXPECT_EQ(response.application_utilization(), 0);
     EXPECT_EQ(response.cpu_utilization(), 0);
     EXPECT_EQ(response.mem_utilization(), 0);
     EXPECT_THAT(
@@ -196,6 +238,15 @@ TEST_F(OrcaServiceEnd2endTest, Basic) {
   });
 }
 
+TEST_F(OrcaServiceEnd2endTest, ClientClosesBeforeSendingMessage) {
+  auto stub = std::make_unique<GenericStub>(channel_);
+  GenericOrcaClientReactor reactor(stub.get());
+  reactor.Prepare();
+  reactor.StartWritesDone();
+  reactor.StartCall();
+  EXPECT_EQ(reactor.Await().error_code(), grpc::StatusCode::INTERNAL);
+}
+
 }  // namespace
 }  // namespace testing
 }  // namespace grpc
