@@ -21,48 +21,52 @@
 //   fires; request is processed at that point
 // - find some deterministic way to exercise adaptive throttler code
 
-#include <deque>
-#include <map>
-#include <thread>
-
 #include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
-#include "absl/strings/str_format.h"
-#include "absl/strings/str_join.h"
-#include "absl/types/optional.h"
-
+#include <grpc/credentials.h>
 #include <grpcpp/channel.h>
 #include <grpcpp/create_channel.h>
 #include <grpcpp/security/credentials.h>
 #include <grpcpp/server.h>
 #include <grpcpp/server_builder.h>
 #include <grpcpp/support/channel_arguments.h>
+#include <gtest/gtest.h>
 
-#include "src/core/ext/filters/client_channel/backup_poller.h"
-#include "src/core/ext/filters/client_channel/resolver/fake/fake_resolver.h"
+#include <deque>
+#include <map>
+#include <thread>
+
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_join.h"
+#include "absl/types/optional.h"
+#include "src/core/client_channel/backup_poller.h"
+#include "src/core/config/config_vars.h"
 #include "src/core/lib/address_utils/parse_address.h"
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/config/config_vars.h"
-#include "src/core/lib/gprpp/env.h"
-#include "src/core/lib/gprpp/host_port.h"
-#include "src/core/lib/gprpp/time.h"
 #include "src/core/lib/iomgr/sockaddr.h"
 #include "src/core/lib/security/credentials/fake/fake_credentials.h"
-#include "src/core/lib/service_config/service_config_impl.h"
-#include "src/core/lib/uri/uri_parser.h"
-#include "src/cpp/client/secure_credentials.h"
+#include "src/core/load_balancing/rls/rls.h"
+#include "src/core/resolver/fake/fake_resolver.h"
+#include "src/core/service_config/service_config_impl.h"
+#include "src/core/util/env.h"
+#include "src/core/util/host_port.h"
+#include "src/core/util/time.h"
+#include "src/core/util/uri.h"
 #include "src/cpp/server/secure_server_credentials.h"
 #include "src/proto/grpc/lookup/v1/rls.grpc.pb.h"
 #include "src/proto/grpc/lookup/v1/rls.pb.h"
 #include "src/proto/grpc/testing/echo.grpc.pb.h"
-#include "test/core/util/port.h"
-#include "test/core/util/resolve_localhost_ip46.h"
-#include "test/core/util/test_config.h"
-#include "test/core/util/test_lb_policies.h"
+#include "test/core/event_engine/event_engine_test_utils.h"
+#include "test/core/test_util/fake_stats_plugin.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/resolve_localhost_ip46.h"
+#include "test/core/test_util/test_config.h"
+#include "test/core/test_util/test_lb_policies.h"
 #include "test/cpp/end2end/counted_service.h"
 #include "test/cpp/end2end/rls_server.h"
 #include "test/cpp/end2end/test_service_impl.h"
+#include "test/cpp/util/credentials.h"
 #include "test/cpp/util/test_config.h"
 
 using ::grpc::lookup::v1::RouteLookupRequest;
@@ -73,6 +77,7 @@ namespace {
 
 const char* kServerName = "test.google.fr";
 const char* kRequestMessage = "Live long and prosper.";
+const char* kRlsInstanceUuid = "rls_instance_uuid";
 
 const char* kCallCredsMdKey = "call_cred_name";
 const char* kCallCredsMdValue = "call_cred_value";
@@ -170,15 +175,12 @@ class RlsEnd2endTest : public ::testing::Test {
 
   static void TearDownTestSuite() {
     grpc_shutdown_blocking();
+    WaitForSingleOwner(
+        grpc_event_engine::experimental::GetDefaultEventEngine());
     grpc_core::CoreConfiguration::Reset();
   }
 
   void SetUp() override {
-    bool localhost_resolves_to_ipv4 = false;
-    bool localhost_resolves_to_ipv6 = false;
-    grpc_core::LocalhostResolves(&localhost_resolves_to_ipv4,
-                                 &localhost_resolves_to_ipv6);
-    ipv6_only_ = !localhost_resolves_to_ipv4 && localhost_resolves_to_ipv6;
     rls_server_ = std::make_unique<ServerThread<RlsServiceImpl>>(
         "rls", [](grpc::ServerContext* ctx) {
           EXPECT_THAT(ctx->client_metadata(),
@@ -187,6 +189,7 @@ class RlsEnd2endTest : public ::testing::Test {
           EXPECT_EQ(ctx->ExperimentalGetAuthority(), kServerName);
         });
     rls_server_->Start();
+    rls_server_target_ = absl::StrFormat("localhost:%d", rls_server_->port_);
     // Set up client.
     resolver_response_generator_ =
         std::make_unique<FakeResolverResponseGeneratorWrapper>();
@@ -194,17 +197,17 @@ class RlsEnd2endTest : public ::testing::Test {
     args.SetPointer(GRPC_ARG_FAKE_RESOLVER_RESPONSE_GENERATOR,
                     resolver_response_generator_->Get());
     args.SetString(GRPC_ARG_FAKE_SECURITY_EXPECTED_TARGETS, kServerName);
+    args.SetString(GRPC_ARG_TEST_ONLY_RLS_INSTANCE_ID, kRlsInstanceUuid);
     grpc_channel_credentials* channel_creds =
         grpc_fake_transport_security_credentials_create();
     grpc_call_credentials* call_creds = grpc_md_only_test_credentials_create(
         kCallCredsMdKey, kCallCredsMdValue);
-    auto creds = std::make_shared<SecureChannelCredentials>(
-        grpc_composite_channel_credentials_create(channel_creds, call_creds,
-                                                  nullptr));
+    auto creds = std::make_shared<TestCompositeChannelCredentials>(
+        channel_creds, call_creds);
     call_creds->Unref();
     channel_creds->Unref();
-    channel_ = grpc::CreateCustomChannel(absl::StrCat("fake:///", kServerName),
-                                         std::move(creds), args);
+    target_uri_ = absl::StrCat("fake:///", kServerName);
+    channel_ = grpc::CreateCustomChannel(target_uri_, creds, args);
     stub_ = grpc::testing::EchoTestService::NewStub(channel_);
   }
 
@@ -228,13 +231,8 @@ class RlsEnd2endTest : public ::testing::Test {
     }
   }
 
-  std::string TargetStringForPort(int port) {
-    if (ipv6_only_) return absl::StrCat("ipv6:[::1]:", port);
-    return absl::StrCat("ipv4:127.0.0.1:", port);
-  }
-
   struct RpcOptions {
-    int timeout_ms = 2000;
+    int timeout_ms = 5000;
     bool wait_for_ready = false;
     std::vector<std::pair<std::string, std::string>> metadata;
 
@@ -305,8 +303,8 @@ class RlsEnd2endTest : public ::testing::Test {
 
   class ServiceConfigBuilder {
    public:
-    explicit ServiceConfigBuilder(int rls_server_port)
-        : rls_server_port_(rls_server_port) {}
+    explicit ServiceConfigBuilder(absl::string_view rls_server_target)
+        : rls_server_target_(rls_server_target) {}
 
     ServiceConfigBuilder& set_lookup_service_timeout(
         grpc_core::Duration timeout) {
@@ -343,7 +341,7 @@ class RlsEnd2endTest : public ::testing::Test {
       // First build parts of routeLookupConfig.
       std::vector<std::string> route_lookup_config_parts;
       route_lookup_config_parts.push_back(absl::StrFormat(
-          "        \"lookupService\":\"localhost:%d\"", rls_server_port_));
+          "        \"lookupService\":\"%s\"", rls_server_target_));
       if (lookup_service_timeout_ > grpc_core::Duration::Zero()) {
         route_lookup_config_parts.push_back(
             absl::StrFormat("        \"lookupServiceTimeout\":\"%fs\"",
@@ -392,7 +390,7 @@ class RlsEnd2endTest : public ::testing::Test {
     }
 
    private:
-    int rls_server_port_;
+    absl::string_view rls_server_target_;
     grpc_core::Duration lookup_service_timeout_;
     std::string default_target_;
     grpc_core::Duration max_age_;
@@ -402,7 +400,7 @@ class RlsEnd2endTest : public ::testing::Test {
   };
 
   ServiceConfigBuilder MakeServiceConfigBuilder() {
-    return ServiceConfigBuilder(rls_server_->port_);
+    return ServiceConfigBuilder(rls_server_target_);
   }
 
   void SetNextResolution(absl::string_view service_config_json) {
@@ -418,8 +416,8 @@ class RlsEnd2endTest : public ::testing::Test {
           service_(std::forward<Args>(args)...) {}
 
     void Start() {
-      gpr_log(GPR_INFO, "starting %s server on port %d", type_.c_str(), port_);
-      GPR_ASSERT(!running_);
+      LOG(INFO) << "starting " << type_ << " server on port " << port_;
+      CHECK(!running_);
       running_ = true;
       service_.Start();
       grpc::internal::Mutex mu;
@@ -430,7 +428,7 @@ class RlsEnd2endTest : public ::testing::Test {
       thread_ = std::make_unique<std::thread>(
           std::bind(&ServerThread::Serve, this, &mu, &cond));
       cond.Wait(&mu);
-      gpr_log(GPR_INFO, "%s server startup complete", type_.c_str());
+      LOG(INFO) << type_ << " server startup complete";
     }
 
     void Serve(grpc::internal::Mutex* mu, grpc::internal::CondVar* cond) {
@@ -449,11 +447,11 @@ class RlsEnd2endTest : public ::testing::Test {
 
     void Shutdown() {
       if (!running_) return;
-      gpr_log(GPR_INFO, "%s about to shutdown", type_.c_str());
+      LOG(INFO) << type_ << " about to shutdown";
       service_.Shutdown();
       server_->Shutdown(grpc_timeout_milliseconds_to_deadline(0));
       thread_->join();
-      gpr_log(GPR_INFO, "%s shutdown completed", type_.c_str());
+      LOG(INFO) << type_ << " shutdown completed";
       running_ = false;
     }
 
@@ -465,11 +463,12 @@ class RlsEnd2endTest : public ::testing::Test {
     bool running_ = false;
   };
 
-  bool ipv6_only_;
   std::vector<std::unique_ptr<ServerThread<MyTestServiceImpl>>> backends_;
+  std::string rls_server_target_;
   std::unique_ptr<ServerThread<RlsServiceImpl>> rls_server_;
   std::unique_ptr<FakeResolverResponseGeneratorWrapper>
       resolver_response_generator_;
+  std::string target_uri_;
   std::shared_ptr<grpc::Channel> channel_;
   std::unique_ptr<grpc::testing::EchoTestService::Stub> stub_;
 };
@@ -494,7 +493,7 @@ TEST_F(RlsEnd2endTest, Basic) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -525,7 +524,7 @@ TEST_F(RlsEnd2endTest, DuplicateHeadersAreMerged) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, absl::StrCat(kTestValue, ",", kTestValue2)}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Same header present twice in the request.  Values should be merged.
   CheckRpcSendOk(
       DEBUG_LOCATION,
@@ -555,7 +554,7 @@ TEST_F(RlsEnd2endTest, SecondHeaderUsed) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key2", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -594,7 +593,7 @@ TEST_F(RlsEnd2endTest, MultipleHeaderKeys) {
           {kTestKey, kTestValue},
           {kTestKey2, kTestValue2},
       }),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(
       DEBUG_LOCATION,
       RpcOptions().set_metadata({{"key1", kTestValue}, {"key2", kTestValue2}}));
@@ -625,7 +624,7 @@ TEST_F(RlsEnd2endTest, NoHeaderMatch) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Request does not have header "key1", so kTestKey will not be added.
   CheckRpcSendOk(DEBUG_LOCATION);
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -651,7 +650,7 @@ TEST_F(RlsEnd2endTest, WildcardMethod) {
                         .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -679,7 +678,7 @@ TEST_F(RlsEnd2endTest, NoKeyBuilderForMethod) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION);
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
   EXPECT_EQ(rls_server_->service_.response_count(), 1);
@@ -707,7 +706,7 @@ TEST_F(RlsEnd2endTest, HeaderData) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)},
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)},
                        kHeaderData));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
@@ -754,7 +753,7 @@ TEST_F(RlsEnd2endTest, ExtraKeysAndConstantKeys) {
           {kMethodKey, kMethodValue},
           {kConstantKey, kConstantValue},
       }),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -783,10 +782,10 @@ TEST_F(RlsEnd2endTest, TwoCacheEntriesWithSameTarget) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue2}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -830,10 +829,10 @@ TEST_F(RlsEnd2endTest, FailedRlsRequestWithoutDefaultTarget) {
   const char* kTestValue3 = "test_value_3";
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue2}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue3}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue2}}));
   CheckRpcSendOk(DEBUG_LOCATION,
@@ -855,7 +854,7 @@ TEST_F(RlsEnd2endTest, FailedRlsRequestWithoutDefaultTarget) {
   // Now give the RLS server the right response.
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Sleep long enough for backoff to elapse, then try another RPC.
   gpr_sleep_until(grpc_timeout_seconds_to_deadline(3));
   CheckRpcSendOk(DEBUG_LOCATION,
@@ -882,7 +881,7 @@ TEST_F(RlsEnd2endTest, FailedRlsRequestWithDefaultTarget) {
                                          "  }"
                                          "]",
                                          kServiceValue, kMethodValue, kTestKey))
-          .set_default_target(TargetStringForPort(backends_[0]->port_))
+          .set_default_target(grpc_core::LocalIpUri(backends_[0]->port_))
           .Build());
   // Don't give the RLS server a response, so the RLS request will fail.
   // The data plane RPC should be sent to the default target.
@@ -918,17 +917,18 @@ TEST_F(RlsEnd2endTest, RlsRequestTimeout) {
                                          "  }"
                                          "]",
                                          kServiceValue, kMethodValue, kTestKey))
-          .set_default_target(TargetStringForPort(backends_[1]->port_))
+          .set_default_target(grpc_core::LocalIpUri(backends_[1]->port_))
           .set_lookup_service_timeout(grpc_core::Duration::Seconds(2))
           .Build());
-  // RLS server will send a response, but it's longer than the timeout.
+  // RLS server will send a response, but it takes longer than the
+  // timeout set in the LB policy config.
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}),
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}),
       /*response_delay=*/grpc_core::Duration::Seconds(3));
   // The data plane RPC should be sent to the default target.
-  CheckRpcSendOk(DEBUG_LOCATION, RpcOptions().set_timeout_ms(4000).set_metadata(
-                                     {{"key1", kTestValue}}));
+  CheckRpcSendOk(DEBUG_LOCATION,
+                 RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
   EXPECT_EQ(backends_[0]->service_.request_count(), 0);
   EXPECT_EQ(backends_[1]->service_.request_count(), 1);
@@ -951,7 +951,7 @@ TEST_F(RlsEnd2endTest, UpdateConfig) {
                                          "  }"
                                          "]",
                                          kServiceValue, kMethodValue, kTestKey))
-          .set_default_target(TargetStringForPort(backends_[0]->port_));
+          .set_default_target(grpc_core::LocalIpUri(backends_[0]->port_));
   SetNextResolution(service_config_builder.Build());
   // Don't give the RLS server a response, so the RLS request will fail.
   // The data plane RPC should be sent to the default target.
@@ -971,7 +971,7 @@ TEST_F(RlsEnd2endTest, UpdateConfig) {
   EXPECT_EQ(backends_[1]->service_.request_count(), 0);
   // Now update the config to point to a new default target.
   service_config_builder.set_default_target(
-      TargetStringForPort(backends_[1]->port_));
+      grpc_core::LocalIpUri(backends_[1]->port_));
   SetNextResolution(service_config_builder.Build());
   // Send another RPC, which should go to the new default target.
   // The RLS server will *not* see another request, because the cache
@@ -1004,7 +1004,7 @@ TEST_F(RlsEnd2endTest, CachedResponse) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Send two RPCs.
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
@@ -1038,7 +1038,7 @@ TEST_F(RlsEnd2endTest, StaleCacheEntry) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Send one RPC.  RLS server gets a request, and RPC goes to backend.
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
@@ -1051,7 +1051,7 @@ TEST_F(RlsEnd2endTest, StaleCacheEntry) {
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}},
                       RouteLookupRequest::REASON_STALE),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Wait longer than stale age.
   gpr_sleep_until(grpc_timeout_seconds_to_deadline(2));
   // Send another RPC.  This should use the stale value but should
@@ -1088,7 +1088,7 @@ TEST_F(RlsEnd2endTest, StaleCacheEntryWithHeaderData) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)},
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)},
                        kHeaderData));
   // Send one RPC.  RLS server gets a request, and RPC goes to backend.
   CheckRpcSendOk(DEBUG_LOCATION,
@@ -1102,7 +1102,7 @@ TEST_F(RlsEnd2endTest, StaleCacheEntryWithHeaderData) {
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}},
                       RouteLookupRequest::REASON_STALE, kHeaderData),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)},
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)},
                        kHeaderData));
   // Wait longer than stale age.
   gpr_sleep_until(grpc_timeout_seconds_to_deadline(2));
@@ -1139,7 +1139,7 @@ TEST_F(RlsEnd2endTest, ExpiredCacheEntry) {
           .Build());
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   // Send one RPC.  RLS server gets a request, and RPC goes to backend.
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
@@ -1185,10 +1185,10 @@ TEST_F(RlsEnd2endTest, CacheSizeLimit) {
   // Set RLS responses for both kTestValue and kTestValue2.
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue}}),
-      BuildRlsResponse({TargetStringForPort(backends_[0]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[0]->port_)}));
   rls_server_->service_.SetResponse(
       BuildRlsRequest({{kTestKey, kTestValue2}}),
-      BuildRlsResponse({TargetStringForPort(backends_[1]->port_)}));
+      BuildRlsResponse({grpc_core::LocalIpUri(backends_[1]->port_)}));
   // Send an RPC for kTestValue.
   // RLS server gets a request, and RPC goes to backend.
   CheckRpcSendOk(DEBUG_LOCATION,
@@ -1256,7 +1256,7 @@ TEST_F(RlsEnd2endTest, MultipleTargets) {
       BuildRlsResponse(
           // Second target will report TRANSIENT_FAILURE, but should
           // never be used.
-          {TargetStringForPort(backends_[0]->port_), "invalid_target"}));
+          {grpc_core::LocalIpUri(backends_[0]->port_), "invalid_target"}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -1286,7 +1286,7 @@ TEST_F(RlsEnd2endTest, MultipleTargetsFirstInTransientFailure) {
       BuildRlsRequest({{kTestKey, kTestValue}}),
       BuildRlsResponse(
           // First target will report TRANSIENT_FAILURE.
-          {"invalid_target", TargetStringForPort(backends_[0]->port_)}));
+          {"invalid_target", grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -1317,7 +1317,7 @@ TEST_F(RlsEnd2endTest, ConnectivityStateReady) {
       BuildRlsRequest({{kTestKey, kTestValue}}),
       BuildRlsResponse(
           // One target in TRANSIENT_FAILURE, the other in READY.
-          {"invalid_target", TargetStringForPort(backends_[0]->port_)}));
+          {"invalid_target", grpc_core::LocalIpUri(backends_[0]->port_)}));
   CheckRpcSendOk(DEBUG_LOCATION,
                  RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
@@ -1373,7 +1373,7 @@ TEST_F(RlsEnd2endTest, ConnectivityStateTransientFailure) {
                                     BuildRlsResponse({"invalid_target"}));
   CheckRpcSendFailure(
       DEBUG_LOCATION, StatusCode::UNAVAILABLE,
-      "empty address list: no address in fixed_address_lb policy",
+      "empty address list (no address in fixed_address_lb policy)",
       RpcOptions().set_metadata({{"key1", kTestValue}}));
   EXPECT_EQ(rls_server_->service_.request_count(), 1);
   EXPECT_EQ(rls_server_->service_.response_count(), 1);
@@ -1381,6 +1381,309 @@ TEST_F(RlsEnd2endTest, ConnectivityStateTransientFailure) {
             channel_->GetState(/*try_to_connect=*/false));
 }
 
+class RlsMetricsEnd2endTest : public RlsEnd2endTest {
+ protected:
+  void SetUp() override {
+    // Register stats plugin before initializing client.
+    stats_plugin_ = grpc_core::FakeStatsPluginBuilder()
+                        .UseDisabledByDefaultMetrics(true)
+                        .BuildAndRegister();
+    RlsEnd2endTest::SetUp();
+  }
+
+  std::shared_ptr<grpc_core::FakeStatsPlugin> stats_plugin_;
+};
+
+TEST_F(RlsMetricsEnd2endTest, MetricDefinitionDefaultTargetPicks) {
+  const auto* descriptor =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::FindMetricDescriptorByName(
+          "grpc.lb.rls.default_target_picks");
+  ASSERT_NE(descriptor, nullptr);
+  EXPECT_EQ(descriptor->value_type,
+            grpc_core::GlobalInstrumentsRegistry::ValueType::kUInt64);
+  EXPECT_EQ(descriptor->instrument_type,
+            grpc_core::GlobalInstrumentsRegistry::InstrumentType::kCounter);
+  EXPECT_EQ(descriptor->enable_by_default, false);
+  EXPECT_EQ(descriptor->name, "grpc.lb.rls.default_target_picks");
+  EXPECT_EQ(descriptor->unit, "{pick}");
+  EXPECT_THAT(descriptor->label_keys,
+              ::testing::ElementsAre("grpc.target", "grpc.lb.rls.server_target",
+                                     "grpc.lb.rls.data_plane_target",
+                                     "grpc.lb.pick_result"));
+  EXPECT_THAT(descriptor->optional_label_keys, ::testing::ElementsAre());
+}
+
+TEST_F(RlsMetricsEnd2endTest, MetricDefinitionTargetPicks) {
+  const auto* descriptor =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::FindMetricDescriptorByName(
+          "grpc.lb.rls.target_picks");
+  ASSERT_NE(descriptor, nullptr);
+  EXPECT_EQ(descriptor->value_type,
+            grpc_core::GlobalInstrumentsRegistry::ValueType::kUInt64);
+  EXPECT_EQ(descriptor->instrument_type,
+            grpc_core::GlobalInstrumentsRegistry::InstrumentType::kCounter);
+  EXPECT_EQ(descriptor->enable_by_default, false);
+  EXPECT_EQ(descriptor->name, "grpc.lb.rls.target_picks");
+  EXPECT_EQ(descriptor->unit, "{pick}");
+  EXPECT_THAT(descriptor->label_keys,
+              ::testing::ElementsAre("grpc.target", "grpc.lb.rls.server_target",
+                                     "grpc.lb.rls.data_plane_target",
+                                     "grpc.lb.pick_result"));
+  EXPECT_THAT(descriptor->optional_label_keys, ::testing::ElementsAre());
+}
+
+TEST_F(RlsMetricsEnd2endTest, MetricDefinitionFailedPicks) {
+  const auto* descriptor =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::FindMetricDescriptorByName(
+          "grpc.lb.rls.failed_picks");
+  ASSERT_NE(descriptor, nullptr);
+  EXPECT_EQ(descriptor->value_type,
+            grpc_core::GlobalInstrumentsRegistry::ValueType::kUInt64);
+  EXPECT_EQ(descriptor->instrument_type,
+            grpc_core::GlobalInstrumentsRegistry::InstrumentType::kCounter);
+  EXPECT_EQ(descriptor->enable_by_default, false);
+  EXPECT_EQ(descriptor->name, "grpc.lb.rls.failed_picks");
+  EXPECT_EQ(descriptor->unit, "{pick}");
+  EXPECT_THAT(
+      descriptor->label_keys,
+      ::testing::ElementsAre("grpc.target", "grpc.lb.rls.server_target"));
+  EXPECT_THAT(descriptor->optional_label_keys, ::testing::ElementsAre());
+}
+
+TEST_F(RlsMetricsEnd2endTest, MetricDefinitionCacheEntries) {
+  const auto* descriptor =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::FindMetricDescriptorByName(
+          "grpc.lb.rls.cache_entries");
+  ASSERT_NE(descriptor, nullptr);
+  EXPECT_EQ(descriptor->value_type,
+            grpc_core::GlobalInstrumentsRegistry::ValueType::kInt64);
+  EXPECT_EQ(
+      descriptor->instrument_type,
+      grpc_core::GlobalInstrumentsRegistry::InstrumentType::kCallbackGauge);
+  EXPECT_EQ(descriptor->enable_by_default, false);
+  EXPECT_EQ(descriptor->name, "grpc.lb.rls.cache_entries");
+  EXPECT_EQ(descriptor->unit, "{entry}");
+  EXPECT_THAT(descriptor->label_keys,
+              ::testing::ElementsAre("grpc.target", "grpc.lb.rls.server_target",
+                                     "grpc.lb.rls.instance_uuid"));
+  EXPECT_THAT(descriptor->optional_label_keys, ::testing::ElementsAre());
+}
+
+TEST_F(RlsMetricsEnd2endTest, MetricDefinitionCacheSize) {
+  const auto* descriptor =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::FindMetricDescriptorByName(
+          "grpc.lb.rls.cache_size");
+  ASSERT_NE(descriptor, nullptr);
+  EXPECT_EQ(descriptor->value_type,
+            grpc_core::GlobalInstrumentsRegistry::ValueType::kInt64);
+  EXPECT_EQ(
+      descriptor->instrument_type,
+      grpc_core::GlobalInstrumentsRegistry::InstrumentType::kCallbackGauge);
+  EXPECT_EQ(descriptor->enable_by_default, false);
+  EXPECT_EQ(descriptor->name, "grpc.lb.rls.cache_size");
+  EXPECT_EQ(descriptor->unit, "By");
+  EXPECT_THAT(descriptor->label_keys,
+              ::testing::ElementsAre("grpc.target", "grpc.lb.rls.server_target",
+                                     "grpc.lb.rls.instance_uuid"));
+  EXPECT_THAT(descriptor->optional_label_keys, ::testing::ElementsAre());
+}
+
+TEST_F(RlsMetricsEnd2endTest, MetricValues) {
+  auto kMetricTargetPicks =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::
+          FindUInt64CounterHandleByName("grpc.lb.rls.target_picks")
+              .value();
+  auto kMetricFailedPicks =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::
+          FindUInt64CounterHandleByName("grpc.lb.rls.failed_picks")
+              .value();
+  auto kMetricCacheEntries =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::
+          FindCallbackInt64GaugeHandleByName("grpc.lb.rls.cache_entries")
+              .value();
+  auto kMetricCacheSize =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::
+          FindCallbackInt64GaugeHandleByName("grpc.lb.rls.cache_size")
+              .value();
+  StartBackends(2);
+  SetNextResolution(
+      MakeServiceConfigBuilder()
+          .AddKeyBuilder(absl::StrFormat("\"names\":[{"
+                                         "  \"service\":\"%s\","
+                                         "  \"method\":\"%s\""
+                                         "}],"
+                                         "\"headers\":["
+                                         "  {"
+                                         "    \"key\":\"%s\","
+                                         "    \"names\":["
+                                         "      \"key1\""
+                                         "    ]"
+                                         "  }"
+                                         "]",
+                                         kServiceValue, kMethodValue, kTestKey))
+          .Build());
+  const std::string rls_target0 = grpc_core::LocalIpUri(backends_[0]->port_);
+  const std::string rls_target1 = grpc_core::LocalIpUri(backends_[1]->port_);
+  // Send an RPC to the target for backend 0.
+  rls_server_->service_.SetResponse(BuildRlsRequest({{kTestKey, rls_target0}}),
+                                    BuildRlsResponse({rls_target0}));
+  CheckRpcSendOk(DEBUG_LOCATION,
+                 RpcOptions().set_metadata({{"key1", rls_target0}}));
+  EXPECT_EQ(rls_server_->service_.request_count(), 1);
+  EXPECT_EQ(rls_server_->service_.response_count(), 1);
+  EXPECT_EQ(backends_[0]->service_.request_count(), 1);
+  EXPECT_EQ(backends_[1]->service_.request_count(), 0);
+  // Check exported metrics.
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricTargetPicks,
+          {target_uri_, rls_server_target_, rls_target0, "complete"}, {}),
+      ::testing::Optional(1));
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricTargetPicks,
+          {target_uri_, rls_server_target_, rls_target1, "complete"}, {}),
+      absl::nullopt);
+  EXPECT_EQ(stats_plugin_->GetUInt64CounterValue(
+                kMetricFailedPicks, {target_uri_, rls_server_target_}, {}),
+            absl::nullopt);
+  stats_plugin_->TriggerCallbacks();
+  EXPECT_THAT(stats_plugin_->GetInt64CallbackGaugeValue(
+                  kMetricCacheEntries,
+                  {target_uri_, rls_server_target_, kRlsInstanceUuid}, {}),
+              ::testing::Optional(1));
+  auto cache_size = stats_plugin_->GetInt64CallbackGaugeValue(
+      kMetricCacheSize, {target_uri_, rls_server_target_, kRlsInstanceUuid},
+      {});
+  EXPECT_THAT(cache_size, ::testing::Optional(::testing::Ge(1)));
+  // Send an RPC to the target for backend 1.
+  rls_server_->service_.SetResponse(BuildRlsRequest({{kTestKey, rls_target1}}),
+                                    BuildRlsResponse({rls_target1}));
+  CheckRpcSendOk(DEBUG_LOCATION,
+                 RpcOptions().set_metadata({{"key1", rls_target1}}));
+  EXPECT_EQ(rls_server_->service_.request_count(), 2);
+  EXPECT_EQ(rls_server_->service_.response_count(), 2);
+  EXPECT_EQ(backends_[0]->service_.request_count(), 1);
+  EXPECT_EQ(backends_[1]->service_.request_count(), 1);
+  // Check exported metrics.
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricTargetPicks,
+          {target_uri_, rls_server_target_, rls_target0, "complete"}, {}),
+      ::testing::Optional(1));
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricTargetPicks,
+          {target_uri_, rls_server_target_, rls_target1, "complete"}, {}),
+      ::testing::Optional(1));
+  EXPECT_EQ(stats_plugin_->GetUInt64CounterValue(
+                kMetricFailedPicks, {target_uri_, rls_server_target_}, {}),
+            absl::nullopt);
+  stats_plugin_->TriggerCallbacks();
+  EXPECT_THAT(stats_plugin_->GetInt64CallbackGaugeValue(
+                  kMetricCacheEntries,
+                  {target_uri_, rls_server_target_, kRlsInstanceUuid}, {}),
+              ::testing::Optional(2));
+  auto cache_size2 = stats_plugin_->GetInt64CallbackGaugeValue(
+      kMetricCacheSize, {target_uri_, rls_server_target_, kRlsInstanceUuid},
+      {});
+  EXPECT_THAT(cache_size2, ::testing::Optional(::testing::Ge(2)));
+  if (cache_size.has_value() && cache_size2.has_value()) {
+    EXPECT_GT(*cache_size2, *cache_size);
+  }
+  // Send an RPC for which the RLS server has no response, which means
+  // that the RLS request will fail.  There is no default target, so the
+  // data plane RPC will fail.
+  CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE,
+                      "RLS request failed: INTERNAL: no response entry",
+                      RpcOptions().set_metadata({{"key1", kTestValue}}));
+  EXPECT_THAT(
+      rls_server_->service_.GetUnmatchedRequests(),
+      ::testing::ElementsAre(
+          // TODO(roth): Change this to use ::testing::ProtoEquals()
+          // once that becomes available in OSS.
+          ::testing::Property(
+              &RouteLookupRequest::DebugString,
+              BuildRlsRequest({{kTestKey, kTestValue}}).DebugString())));
+  EXPECT_EQ(rls_server_->service_.request_count(), 3);
+  EXPECT_EQ(rls_server_->service_.response_count(), 2);
+  EXPECT_EQ(backends_[0]->service_.request_count(), 1);
+  EXPECT_EQ(backends_[1]->service_.request_count(), 1);
+  // Check exported metrics.
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricTargetPicks,
+          {target_uri_, rls_server_target_, rls_target0, "complete"}, {}),
+      ::testing::Optional(1));
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricTargetPicks,
+          {target_uri_, rls_server_target_, rls_target1, "complete"}, {}),
+      ::testing::Optional(1));
+  EXPECT_THAT(stats_plugin_->GetUInt64CounterValue(
+                  kMetricFailedPicks, {target_uri_, rls_server_target_}, {}),
+              ::testing::Optional(1));
+  stats_plugin_->TriggerCallbacks();
+  EXPECT_THAT(stats_plugin_->GetInt64CallbackGaugeValue(
+                  kMetricCacheEntries,
+                  {target_uri_, rls_server_target_, kRlsInstanceUuid}, {}),
+              ::testing::Optional(3));
+  auto cache_size3 = stats_plugin_->GetInt64CallbackGaugeValue(
+      kMetricCacheSize, {target_uri_, rls_server_target_, kRlsInstanceUuid},
+      {});
+  EXPECT_THAT(cache_size3, ::testing::Optional(::testing::Ge(3)));
+  if (cache_size.has_value() && cache_size3.has_value()) {
+    EXPECT_GT(*cache_size3, *cache_size);
+  }
+}
+
+TEST_F(RlsMetricsEnd2endTest, MetricValuesDefaultTargetRpcs) {
+  auto kMetricDefaultTargetPicks =
+      grpc_core::GlobalInstrumentsRegistryTestPeer::
+          FindUInt64CounterHandleByName("grpc.lb.rls.default_target_picks")
+              .value();
+  StartBackends(1);
+  const std::string default_target = grpc_core::LocalIpUri(backends_[0]->port_);
+  SetNextResolution(
+      MakeServiceConfigBuilder()
+          .AddKeyBuilder(absl::StrFormat("\"names\":[{"
+                                         "  \"service\":\"%s\","
+                                         "  \"method\":\"%s\""
+                                         "}],"
+                                         "\"headers\":["
+                                         "  {"
+                                         "    \"key\":\"%s\","
+                                         "    \"names\":["
+                                         "      \"key1\""
+                                         "    ]"
+                                         "  }"
+                                         "]",
+                                         kServiceValue, kMethodValue, kTestKey))
+          .set_default_target(default_target)
+          .Build());
+  // Don't give the RLS server a response, so the RLS request will fail.
+  // The data plane RPC should be sent to the default target.
+  CheckRpcSendOk(DEBUG_LOCATION,
+                 RpcOptions().set_metadata({{"key1", kTestValue}}));
+  EXPECT_THAT(
+      rls_server_->service_.GetUnmatchedRequests(),
+      ::testing::ElementsAre(
+          // TODO(roth): Change this to use ::testing::ProtoEquals()
+          // once that becomes available in OSS.
+          ::testing::Property(
+              &RouteLookupRequest::DebugString,
+              BuildRlsRequest({{kTestKey, kTestValue}}).DebugString())));
+  EXPECT_EQ(rls_server_->service_.request_count(), 1);
+  EXPECT_EQ(rls_server_->service_.response_count(), 0);
+  EXPECT_EQ(backends_[0]->service_.request_count(), 1);
+  // Check expected metrics.
+  EXPECT_THAT(
+      stats_plugin_->GetUInt64CounterValue(
+          kMetricDefaultTargetPicks,
+          {target_uri_, rls_server_target_, default_target, "complete"}, {}),
+      ::testing::Optional(1));
+}
+
 }  // namespace
 }  // namespace testing
 }  // namespace grpc
