@@ -18,20 +18,18 @@
 
 #include "test/cpp/end2end/test_service_impl.h"
 
-#include <string>
-#include <thread>
-
-#include <gtest/gtest.h>
-
-#include "absl/log/check.h"
-#include "absl/log/log.h"
-
 #include <grpcpp/alarm.h>
 #include <grpcpp/security/credentials.h>
 #include <grpcpp/server_context.h>
+#include <gtest/gtest.h>
 
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/notification.h"
+#include <string>
+#include <thread>
+
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/notification.h"
 #include "src/proto/grpc/testing/echo.grpc.pb.h"
 #include "test/cpp/util/string_ref_helper.h"
 
@@ -290,7 +288,7 @@ ServerUnaryReactor* CallbackTestServiceImpl::Echo(
         }
       }
       if (req_->has_param() &&
-          (req_->param().expected_client_identity().length() > 0 ||
+          (!req_->param().expected_client_identity().empty() ||
            req_->param().check_auth_context())) {
         internal::CheckServerAuthContext(
             ctx_, req_->param().expected_transport_security_type(),
