@@ -13,26 +13,24 @@
 // limitations under the License.
 //
 
-#include <vector>
-
 #include <gmock/gmock.h>
+#include <grpc/event_engine/endpoint_config.h>
 #include <gtest/gtest.h>
 
+#include <vector>
+
 #include "absl/log/check.h"
 #include "absl/log/log.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
-
-#include <grpc/event_engine/endpoint_config.h>
-
+#include "envoy/extensions/clusters/aggregate/v3/cluster.pb.h"
 #include "src/core/client_channel/backup_poller.h"
+#include "src/core/config/config_vars.h"
 #include "src/core/lib/address_utils/sockaddr_utils.h"
-#include "src/core/lib/config/config_vars.h"
-#include "src/core/lib/gprpp/env.h"
 #include "src/core/load_balancing/xds/xds_channel_args.h"
 #include "src/core/resolver/endpoint_addresses.h"
 #include "src/core/resolver/fake/fake_resolver.h"
-#include "src/proto/grpc/testing/xds/v3/aggregate_cluster.grpc.pb.h"
+#include "src/core/util/env.h"
 #include "test/core/test_util/resolve_localhost_ip46.h"
 #include "test/core/test_util/scoped_env_var.h"
 #include "test/cpp/end2end/connection_attempt_injector.h"
@@ -42,24 +40,31 @@ namespace grpc {
 namespace testing {
 namespace {
 
-using ::envoy::config::cluster::v3::CustomClusterType;
 using ::envoy::extensions::clusters::aggregate::v3::ClusterConfig;
 
 class ClusterTypeTest : public XdsEnd2endTest {
  protected:
-  void SetUp() override {
-    logical_dns_cluster_resolver_response_generator_ =
-        grpc_core::MakeRefCounted<grpc_core::FakeResolverResponseGenerator>();
-    InitClient();
+  ClusterTypeTest()
+      : logical_dns_cluster_resolver_response_generator_(
+            grpc_core::MakeRefCounted<
+                grpc_core::FakeResolverResponseGenerator>()) {}
+
+  // Subclasses must call this to initialize.
+  void LogicalDnsInitClient(
+      absl::optional<XdsBootstrapBuilder> builder = absl::nullopt,
+      std::shared_ptr<ChannelCredentials> credentials = nullptr) {
     ChannelArguments args;
     args.SetPointerWithVtable(
         GRPC_ARG_XDS_LOGICAL_DNS_CLUSTER_FAKE_RESOLVER_RESPONSE_GENERATOR,
         logical_dns_cluster_resolver_response_generator_.get(),
         &grpc_core::FakeResolverResponseGenerator::kChannelArgPointerVtable);
-    ResetStub(/*failover_timeout_ms=*/0, &args);
+    InitClient(builder, /*lb_expected_authority=*/"",
+               /*xds_resource_does_not_exist_timeout_ms=*/0,
+               /*balancer_authority_override=*/"", /*args=*/&args,
+               std::move(credentials));
   }
 
-  grpc_core::EndpointAddressesList CreateAddressListFromPortList(
+  static grpc_core::EndpointAddressesList CreateAddressListFromPortList(
       const std::vector<int>& ports) {
     grpc_core::EndpointAddressesList addresses;
     for (int port : ports) {
@@ -81,12 +86,16 @@ class ClusterTypeTest : public XdsEnd2endTest {
 // LOGICAL_DNS cluster tests
 //
 
-using LogicalDNSClusterTest = ClusterTypeTest;
+class LogicalDNSClusterTest : public ClusterTypeTest {
+ protected:
+  void SetUp() override {}  // Individual tests call LogicalDnsInitClient().
+};
 
 INSTANTIATE_TEST_SUITE_P(XdsTest, LogicalDNSClusterTest,
                          ::testing::Values(XdsTestType()), &XdsTestType::Name);
 
 TEST_P(LogicalDNSClusterTest, Basic) {
+  LogicalDnsInitClient();
   CreateAndStartBackends(1);
   // Create Logical DNS Cluster
   auto cluster = default_cluster_;
@@ -112,6 +121,226 @@ TEST_P(LogicalDNSClusterTest, Basic) {
   CheckRpcSendOk(DEBUG_LOCATION);
 }
 
+TEST_P(LogicalDNSClusterTest, FailedBackendConnectionCausesReresolution) {
+  LogicalDnsInitClient();
+  CreateAndStartBackends(2);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kServerName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Set Logical DNS result to backend 0.
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts(0, 1));
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // RPCs should succeed.
+  CheckRpcSendOk(DEBUG_LOCATION);
+  // Now shut down backend 0.
+  ShutdownBackend(0);
+  // Wait for logical DNS resolver to see a re-resolution request.
+  // Then return a DNS result pointing to backend 1.
+  {
+    grpc_core::ExecCtx exec_ctx;
+    ASSERT_TRUE(logical_dns_cluster_resolver_response_generator_
+                    ->WaitForReresolutionRequest(absl::Seconds(10) *
+                                                 grpc_test_slowdown_factor()));
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts(1, 2));
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Wait for traffic to switch to backend 1.
+  // RPCs may fail until the client sees the resolver result.
+  WaitForBackend(DEBUG_LOCATION, 1, [](const RpcResult& result) {
+    if (!result.status.ok()) {
+      EXPECT_EQ(StatusCode::UNAVAILABLE, result.status.error_code());
+      EXPECT_THAT(result.status.error_message(),
+                  MakeConnectionFailureRegex(
+                      "connections to all backends failing; last error: "));
+    }
+  });
+}
+
+TEST_P(LogicalDNSClusterTest, AutoHostRewrite) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_AUTHORITY_REWRITE");
+  constexpr char kDnsName[] = "dns.example.com";
+  // Set auto_host_rewrite in the RouteConfig.
+  RouteConfiguration new_route_config = default_route_config_;
+  new_route_config.mutable_virtual_hosts(0)
+      ->mutable_routes(0)
+      ->mutable_route()
+      ->mutable_auto_host_rewrite()
+      ->set_value(true);
+  SetRouteConfiguration(balancer_.get(), new_route_config);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  // Note: We use InsecureCreds, since FakeCreds are too picky about
+  // what authority gets sent.
+  LogicalDnsInitClient(MakeBootstrapBuilder().SetTrustedXdsServer(),
+                       InsecureChannelCredentials());
+  CreateAndStartBackends(1, /*xds_enabled=*/false, InsecureServerCredentials());
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), absl::StrCat(kDnsName, ":443"));
+}
+
+TEST_P(LogicalDNSClusterTest, NoAuthorityRewriteWithoutEnvVar) {
+  constexpr char kDnsName[] = "dns.example.com";
+  // Set auto_host_rewrite in the RouteConfig.
+  RouteConfiguration new_route_config = default_route_config_;
+  new_route_config.mutable_virtual_hosts(0)
+      ->mutable_routes(0)
+      ->mutable_route()
+      ->mutable_auto_host_rewrite()
+      ->set_value(true);
+  SetRouteConfiguration(balancer_.get(), new_route_config);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  LogicalDnsInitClient(MakeBootstrapBuilder().SetTrustedXdsServer());
+  CreateAndStartBackends(1);
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), kServerName);
+}
+
+TEST_P(LogicalDNSClusterTest, NoAuthorityRewriteIfServerNotTrustedInBootstrap) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_AUTHORITY_REWRITE");
+  constexpr char kDnsName[] = "dns.example.com";
+  // Set auto_host_rewrite in the RouteConfig.
+  RouteConfiguration new_route_config = default_route_config_;
+  new_route_config.mutable_virtual_hosts(0)
+      ->mutable_routes(0)
+      ->mutable_route()
+      ->mutable_auto_host_rewrite()
+      ->set_value(true);
+  SetRouteConfiguration(balancer_.get(), new_route_config);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  LogicalDnsInitClient();
+  CreateAndStartBackends(1);
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), kServerName);
+}
+
+TEST_P(LogicalDNSClusterTest, NoAuthorityRewriteIfNotEnabledInRoute) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_AUTHORITY_REWRITE");
+  constexpr char kDnsName[] = "dns.example.com";
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  LogicalDnsInitClient(MakeBootstrapBuilder().SetTrustedXdsServer());
+  CreateAndStartBackends(1);
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), kServerName);
+}
+
 //
 // aggregate cluster tests
 //
@@ -119,7 +348,10 @@ TEST_P(LogicalDNSClusterTest, Basic) {
 // TODO(roth): Add tests showing that load reporting is enabled on a
 // per-underlying-cluster basis.
 
-using AggregateClusterTest = ClusterTypeTest;
+class AggregateClusterTest : public ClusterTypeTest {
+ protected:
+  void SetUp() override { LogicalDnsInitClient(); }
+};
 
 INSTANTIATE_TEST_SUITE_P(XdsTest, AggregateClusterTest,
                          ::testing::Values(XdsTestType()), &XdsTestType::Name);
@@ -154,7 +386,7 @@ TEST_P(AggregateClusterTest, Basic) {
   balancer_->ads_service()->SetCdsResource(new_cluster2);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewCluster1Name);
@@ -207,7 +439,7 @@ TEST_P(AggregateClusterTest, LoadBalancingPolicyComesFromUnderlyingCluster) {
   balancer_->ads_service()->SetCdsResource(new_cluster2);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewCluster1Name);
@@ -280,7 +512,7 @@ TEST_P(AggregateClusterTest, LoadBalancingPolicyComesFromAggregateCluster) {
   balancer_->ads_service()->SetCdsResource(new_cluster2);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewCluster1Name);
@@ -342,7 +574,7 @@ TEST_P(AggregateClusterTest, FallBackWithConnectivityChurn) {
   balancer_->ads_service()->SetCdsResource(cluster2);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kClusterName1);
@@ -417,7 +649,7 @@ TEST_P(AggregateClusterTest, EdsToLogicalDns) {
   balancer_->ads_service()->SetCdsResource(logical_dns_cluster);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewCluster1Name);
@@ -478,7 +710,7 @@ TEST_P(AggregateClusterTest, LogicalDnsToEds) {
   balancer_->ads_service()->SetCdsResource(logical_dns_cluster);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kLogicalDNSClusterName);
@@ -522,9 +754,9 @@ TEST_P(AggregateClusterTest, ReconfigEdsWhileLogicalDnsChildFails) {
   // - Priority 0: locality0
   // - Priority 1: locality1, locality2
   EdsResourceArgs args1({
-      {"locality0", {MakeNonExistantEndpoint()}, kDefaultLocalityWeight, 0},
-      {"locality1", {MakeNonExistantEndpoint()}, kDefaultLocalityWeight, 1},
-      {"locality2", {MakeNonExistantEndpoint()}, kDefaultLocalityWeight, 1},
+      {"locality0", {MakeNonExistentEndpoint()}, kDefaultLocalityWeight, 0},
+      {"locality1", {MakeNonExistentEndpoint()}, kDefaultLocalityWeight, 1},
+      {"locality2", {MakeNonExistentEndpoint()}, kDefaultLocalityWeight, 1},
   });
   balancer_->ads_service()->SetEdsResource(
       BuildEdsResource(args1, kNewEdsService1Name));
@@ -549,7 +781,7 @@ TEST_P(AggregateClusterTest, ReconfigEdsWhileLogicalDnsChildFails) {
   balancer_->ads_service()->SetCdsResource(logical_dns_cluster);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewCluster1Name);
@@ -566,8 +798,8 @@ TEST_P(AggregateClusterTest, ReconfigEdsWhileLogicalDnsChildFails) {
   }
   // When an RPC fails, we know the channel has seen the update.
   constexpr char kErrorMessage[] =
-      "empty address list: DNS resolution failed for server.example.com:443: "
-      "UNAVAILABLE: injected error";
+      "empty address list \\(DNS resolution failed for server.example.com:443: "
+      "UNAVAILABLE: injected error\\)";
   CheckRpcSendFailure(DEBUG_LOCATION, StatusCode::UNAVAILABLE, kErrorMessage);
   // Send an EDS update that moves locality1 to priority 0.
   args1 = EdsResourceArgs({
@@ -594,7 +826,7 @@ TEST_P(AggregateClusterTest, MultipleClustersWithSameLocalities) {
   const char* kNewClusterName2 = "new_cluster_2";
   const char* kNewEdsServiceName2 = "new_eds_service_name_2";
   // Populate EDS resource for cluster 1 with unreachable endpoint.
-  EdsResourceArgs args1({{"locality0", {MakeNonExistantEndpoint()}}});
+  EdsResourceArgs args1({{"locality0", {MakeNonExistentEndpoint()}}});
   balancer_->ads_service()->SetEdsResource(
       BuildEdsResource(args1, kNewEdsServiceName1));
   // Populate CDS resource for cluster 1.
@@ -615,7 +847,7 @@ TEST_P(AggregateClusterTest, MultipleClustersWithSameLocalities) {
   balancer_->ads_service()->SetCdsResource(new_cluster2);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewClusterName1);
@@ -661,7 +893,7 @@ TEST_P(AggregateClusterTest, UpdateOfChildCluster) {
   balancer_->ads_service()->SetCdsResource(new_cluster1);
   // Create Aggregate Cluster
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewCluster1Name);
@@ -712,7 +944,7 @@ TEST_P(AggregateClusterTest, DiamondDependency) {
   // Populate top-level aggregate cluster pointing to kNewClusterName1
   // and kNewAggregateClusterName.
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewClusterName1);
@@ -748,7 +980,7 @@ TEST_P(AggregateClusterTest, DependencyLoopWithNoLeafClusters) {
   const char* kNewClusterName1 = "new_cluster_1";
   // Default cluster is an aggregate cluster pointing to kNewClusterName1.
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewClusterName1);
@@ -779,7 +1011,7 @@ TEST_P(AggregateClusterTest, DependencyLoopWithLeafClusters) {
   // Populate top-level aggregate cluster pointing to itself and the new
   // CDS cluster.
   auto cluster = default_cluster_;
-  CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+  auto* custom_cluster = cluster.mutable_cluster_type();
   custom_cluster->set_name("envoy.clusters.aggregate");
   ClusterConfig cluster_config;
   cluster_config.add_clusters(kNewClusterName1);
@@ -803,7 +1035,7 @@ TEST_P(AggregateClusterTest, RecursionDepthJustBelowMax) {
   for (int i = 14; i >= 0; --i) {
     auto cluster = default_cluster_;
     if (i > 0) cluster.set_name(absl::StrCat(kDefaultClusterName, i));
-    CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+    auto* custom_cluster = cluster.mutable_cluster_type();
     custom_cluster->set_name("envoy.clusters.aggregate");
     ClusterConfig cluster_config;
     cluster_config.add_clusters(absl::StrCat(kDefaultClusterName, i + 1));
@@ -827,7 +1059,7 @@ TEST_P(AggregateClusterTest, RecursionMaxDepth) {
   for (int i = 15; i >= 0; --i) {
     auto cluster = default_cluster_;
     if (i > 0) cluster.set_name(absl::StrCat(kDefaultClusterName, i));
-    CustomClusterType* custom_cluster = cluster.mutable_cluster_type();
+    auto* custom_cluster = cluster.mutable_cluster_type();
     custom_cluster->set_name("envoy.clusters.aggregate");
     ClusterConfig cluster_config;
     cluster_config.add_clusters(absl::StrCat(kDefaultClusterName, i + 1));
