@@ -18,6 +18,14 @@
 
 #include "test/cpp/interop/interop_client.h"
 
+#include <grpc/grpc.h>
+#include <grpc/support/alloc.h>
+#include <grpc/support/string_util.h>
+#include <grpc/support/time.h>
+#include <grpcpp/channel.h>
+#include <grpcpp/client_context.h>
+#include <grpcpp/security/credentials.h>
+
 #include <cinttypes>
 #include <fstream>
 #include <memory>
@@ -32,18 +40,9 @@
 #include "absl/strings/str_format.h"
 #include "absl/strings/str_join.h"
 #include "absl/types/optional.h"
-
-#include <grpc/grpc.h>
-#include <grpc/support/alloc.h>
-#include <grpc/support/string_util.h>
-#include <grpc/support/time.h>
-#include <grpcpp/channel.h>
-#include <grpcpp/client_context.h>
-#include <grpcpp/security/credentials.h>
-
-#include "src/core/lib/config/config_vars.h"
-#include "src/core/lib/config/core_configuration.h"
-#include "src/core/lib/gprpp/crash.h"
+#include "src/core/config/config_vars.h"
+#include "src/core/config/core_configuration.h"
+#include "src/core/util/crash.h"
 #include "src/proto/grpc/testing/empty.pb.h"
 #include "src/proto/grpc/testing/messages.pb.h"
 #include "src/proto/grpc/testing/test.grpc.pb.h"
@@ -605,7 +604,7 @@ bool InteropClient::DoServerCompressedStreaming() {
         absl::StrFormat("(compression=%s; size=%d)",
                         compressions[i] ? "true" : "false", sizes[i]);
 
-    VLOG(2) << "Sending request streaming rpc " << log_suffix.c_str();
+    VLOG(2) << "Sending request streaming rpc " << log_suffix;
 
     ResponseParameters* const response_parameter =
         request.add_response_parameters();
