@@ -1,41 +1,40 @@
-/*
- *
- * Copyright 2018 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
-
-#include <grpc/impl/codegen/port_platform.h>
+//
+//
+// Copyright 2018 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
 #include "src/cpp/server/load_reporter/load_reporter.h"
 
+#include <gmock/gmock.h>
+#include <grpc/grpc.h>
+#include <grpc/support/port_platform.h>
+#include <gtest/gtest.h>
+
 #include <set>
 #include <vector>
 
-#include <gmock/gmock.h>
-#include <gtest/gtest.h>
-
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/memory/memory.h"
 #include "opencensus/stats/testing/test_utils.h"
-
-#include <grpc/grpc.h>
-
 #include "src/core/ext/filters/load_reporting/registered_opencensus_objects.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/cpp/server/load_reporter/constants.h"
-#include "test/core/util/port.h"
-#include "test/core/util/test_config.h"
+#include "test/core/test_util/port.h"
+#include "test/core/test_util/test_config.h"
 
 namespace grpc {
 namespace testing {
@@ -60,7 +59,7 @@ class MockCensusViewProvider : public CensusViewProvider {
 
   const ViewDescriptor& FindViewDescriptor(const std::string& view_name) {
     auto it = view_descriptor_map().find(view_name);
-    GPR_ASSERT(it != view_descriptor_map().end());
+    CHECK(it != view_descriptor_map().end());
     return it->second;
   }
 };
@@ -174,9 +173,8 @@ class LbFeedbackTest : public LoadReporterTest {
                 DoubleNear(expected_qps, expected_qps * 0.3));
     ASSERT_THAT(static_cast<double>(lb_feedback.errors_per_second()),
                 DoubleNear(expected_eps, expected_eps * 0.3));
-    gpr_log(GPR_INFO,
-            "Verified LB feedback matches the samples of index [%zu, %zu).",
-            start, start + count);
+    LOG(INFO) << "Verified LB feedback matches the samples of index [" << start
+              << ", " << start + count << ").";
   }
 
   const std::vector<std::pair<double, double>> kQpsEpsSamples = {
@@ -487,11 +485,11 @@ TEST_F(LoadReportTest, BasicReport) {
   // First fetch.
   load_reporter_->FetchAndSample();
   load_reporter_->GenerateLoads(kHostname1, kLbId1);
-  gpr_log(GPR_INFO, "First load generated.");
+  LOG(INFO) << "First load generated.";
   // Second fetch.
   load_reporter_->FetchAndSample();
   load_reporter_->GenerateLoads(kHostname2, kLbId2);
-  gpr_log(GPR_INFO, "Second load generated.");
+  LOG(INFO) << "Second load generated.";
   // TODO(juanlishen): Verify the data.
 }
 
