@@ -18,14 +18,13 @@
 
 #include "test/cpp/util/cli_credentials.h"
 
-#include "absl/flags/flag.h"
-
 #include <grpc/slice.h>
-#include <grpc/support/log.h>
 #include <grpcpp/support/slice.h>
 
-#include "src/core/lib/gprpp/crash.h"
-#include "src/core/lib/gprpp/load_file.h"
+#include "absl/flags/flag.h"
+#include "absl/log/log.h"
+#include "src/core/util/crash.h"
+#include "src/core/util/load_file.h"
 
 ABSL_RETIRED_FLAG(bool, enable_ssl, false,
                   "Replaced by --channel_creds_type=ssl.");
@@ -64,7 +63,7 @@ namespace {
 
 const char ACCESS_TOKEN_PREFIX[] = "access_token=";
 constexpr int ACCESS_TOKEN_PREFIX_LEN =
-    sizeof(ACCESS_TOKEN_PREFIX) / sizeof(*ACCESS_TOKEN_PREFIX) - 1;
+    (sizeof(ACCESS_TOKEN_PREFIX) / sizeof(*ACCESS_TOKEN_PREFIX)) - 1;
 
 bool IsAccessToken(const std::string& auth) {
   return auth.length() > ACCESS_TOKEN_PREFIX_LEN &&
@@ -97,9 +96,9 @@ CliCredentials::GetChannelCredentials() const {
       auto cert = grpc_core::LoadFile(absl::GetFlag(FLAGS_ssl_client_cert),
                                       /*add_null_terminator=*/false);
       if (!cert.ok()) {
-        gpr_log(GPR_ERROR, "error loading file %s: %s",
-                absl::GetFlag(FLAGS_ssl_client_cert).c_str(),
-                cert.status().ToString().c_str());
+        LOG(ERROR) << "error loading file "
+                   << absl::GetFlag(FLAGS_ssl_client_cert) << ": "
+                   << cert.status();
       } else {
         ssl_creds_options.pem_cert_chain = std::string(cert->as_string_view());
       }
@@ -108,9 +107,9 @@ CliCredentials::GetChannelCredentials() const {
       auto key = grpc_core::LoadFile(absl::GetFlag(FLAGS_ssl_client_key),
                                      /*add_null_terminator=*/false);
       if (!key.ok()) {
-        gpr_log(GPR_ERROR, "error loading file %s: %s",
-                absl::GetFlag(FLAGS_ssl_client_key).c_str(),
-                key.status().ToString().c_str());
+        LOG(ERROR) << "error loading file "
+                   << absl::GetFlag(FLAGS_ssl_client_key) << ": "
+                   << key.status();
       } else {
         ssl_creds_options.pem_private_key = std::string(key->as_string_view());
       }
