@@ -26,14 +26,15 @@ source tools/internal_ci/helper_scripts/prepare_build_macos_rc
 
 # TODO(jtattermusch): cleanup this prepare build step (needed for python artifact build)
 # install cython for all python versions
-python3.7 -m pip install -U 'cython<3.0.0rc1' setuptools==65.4.1 wheel --user
-python3.8 -m pip install -U 'cython<3.0.0rc1' setuptools==65.4.1 wheel --user
-python3.9 -m pip install -U 'cython<3.0.0rc1' setuptools==65.4.1 wheel --user
-python3.10 -m pip install -U 'cython<3.0.0rc1' setuptools==65.4.1 wheel --user
-python3.11 -m pip install -U 'cython<3.0.0rc1' setuptools==65.4.1 wheel --user
+python3.8 -m pip install -U 'cython<4.0.0rc1' setuptools==65.4.1 wheel --user
+python3.9 -m pip install -U 'cython<4.0.0rc1' setuptools==65.4.1 wheel --user
+python3.10 -m pip install -U 'cython<4.0.0rc1' setuptools==65.4.1 wheel --user
+python3.11 -m pip install -U 'cython<4.0.0rc1' setuptools==65.4.1 wheel --user
+python3.12 -m pip install -U 'cython<4.0.0rc1' setuptools==65.4.1 wheel --user
+python3.13 -m pip install -U 'cython<4.0.0rc1' setuptools==65.4.1 wheel --user
 
 # Build all python macos artifacts (this step actually builds all the binary wheels and source archives)
-tools/run_tests/task_runner.py -f artifact macos python ${TASK_RUNNER_EXTRA_FILTERS} -j 4 -x build_artifacts/sponge_log.xml || FAILED="true"
+tools/run_tests/task_runner.py -f artifact macos python ${TASK_RUNNER_EXTRA_FILTERS} -j 2 -x build_artifacts/sponge_log.xml || FAILED="true"
 
 # the next step expects to find the artifacts from the previous step in the "input_artifacts" folder.
 rm -rf input_artifacts
@@ -48,6 +49,9 @@ cp -r input_artifacts/python_*/* artifacts/ || true
 # TODO(jtattermusch): Here we would normally run python macos distribtests, but currently no such tests are defined
 # in distribtest_targets.py
 
+# This step checks if any of the artifacts exceeds a per-file size limit.
+tools/internal_ci/helper_scripts/check_python_artifacts_size.sh
+
 tools/internal_ci/helper_scripts/store_artifacts_from_moved_src_tree.sh
 
 if [ "$FAILED" != "" ]
