@@ -134,6 +134,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.62.0", ReleaseInfo()),
             ("v1.63.1", ReleaseInfo()),
             ("v1.64.1", ReleaseInfo()),
+            ("v1.65.0", ReleaseInfo()),
+            ("v1.66.0", ReleaseInfo()),
+            ("v1.67.0", ReleaseInfo()),
         ]
     ),
     "go": OrderedDict(
@@ -300,8 +303,13 @@ LANG_RELEASE_MATRIX = {
             ("v1.60.1", ReleaseInfo(runtimes=["go1.19"])),
             ("v1.61.2", ReleaseInfo(runtimes=["go1.19"])),
             ("v1.62.2", ReleaseInfo(runtimes=["go1.19"])),
-            ("v1.63.2", ReleaseInfo(runtimes=["go1.19"])),
-            ("v1.64.0", ReleaseInfo()),
+            ("v1.63.3", ReleaseInfo()),
+            ("v1.64.1", ReleaseInfo()),
+            ("v1.65.1", ReleaseInfo()),
+            ("v1.66.3", ReleaseInfo()),
+            ("v1.67.3", ReleaseInfo()),
+            ("v1.68.2", ReleaseInfo()),
+            ("v1.69.2", ReleaseInfo()),
         ]
     ),
     "java": OrderedDict(
@@ -443,8 +451,13 @@ LANG_RELEASE_MATRIX = {
             ("v1.59.1", ReleaseInfo()),
             ("v1.60.1", ReleaseInfo()),
             ("v1.61.0", ReleaseInfo()),
-            ("v1.63.1", ReleaseInfo()),
-            ("v1.64.0", ReleaseInfo()),
+            ("v1.63.2", ReleaseInfo()),
+            ("v1.64.2", ReleaseInfo()),
+            ("v1.65.1", ReleaseInfo()),
+            ("v1.66.0", ReleaseInfo()),
+            ("v1.67.1", ReleaseInfo()),
+            ("v1.68.2", ReleaseInfo()),
+            ("v1.69.0", ReleaseInfo()),
         ]
     ),
     "python": OrderedDict(
@@ -797,6 +810,24 @@ LANG_RELEASE_MATRIX = {
                     runtimes=["python"], testcases_file="python__master"
                 ),
             ),
+            (
+                "v1.65.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
+            (
+                "v1.66.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
+            (
+                "v1.67.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
         ]
     ),
     "node": OrderedDict(
@@ -895,6 +926,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.62.0", ReleaseInfo()),
             ("v1.63.0", ReleaseInfo()),
             ("v1.64.0", ReleaseInfo()),
+            ("v1.65.0", ReleaseInfo()),
+            ("v1.66.0", ReleaseInfo()),
+            ("v1.67.0", ReleaseInfo()),
         ]
     ),
     "php": OrderedDict(
@@ -957,6 +991,9 @@ LANG_RELEASE_MATRIX = {
             ("v1.62.0", ReleaseInfo()),
             ("v1.63.0", ReleaseInfo()),
             ("v1.64.0", ReleaseInfo()),
+            ("v1.65.0", ReleaseInfo()),
+            ("v1.66.0", ReleaseInfo()),
+            ("v1.67.0", ReleaseInfo()),
         ]
     ),
     "csharp": OrderedDict(
