@@ -36,13 +36,25 @@ if [ "$SYSTEM" == "Darwin" ]; then
 fi
 bundle exec rake compile
 
+# Log stuff and save a hash of the binary verify later at test runtime, in order
+# to detect corruption.
+if [ "$SYSTEM" == "Darwin" ]; then
+  ls -l src/ruby/lib/grpc/grpc_c.bundle
+  file src/ruby/lib/grpc/grpc_c.bundle
+  shasum -a 256 src/ruby/lib/grpc/grpc_c.bundle | awk '{print $1}' > src/ruby/lib/grpc/grpc_c_sha256
+else
+  ls -l src/ruby/lib/grpc/grpc_c.so
+  file src/ruby/lib/grpc/grpc_c.so
+  sha256sum src/ruby/lib/grpc/grpc_c.so | awk '{print $1}' > src/ruby/lib/grpc/grpc_c_sha256
+fi
+
 # build grpc_ruby_plugin
 mkdir -p cmake/build
 pushd cmake/build
-cmake -DgRPC_BUILD_TESTS=OFF -DCMAKE_BUILD_TYPE=${CMAKE_CONFIG} ../..
+cmake -DgRPC_BUILD_TESTS=OFF -DCMAKE_BUILD_TYPE=${CMAKE_CONFIG} -DCMAKE_CXX_STANDARD=17 ../..
 make protoc grpc_ruby_plugin -j2
 popd
 
 # unbreak subsequent make builds by restoring zconf.h (previously renamed by cmake build)
 # see https://github.com/madler/zlib/issues/133
-(cd third_party/zlib; git checkout zconf.h)
+(cd third_party/zlib; git checkout zconf.h || cp zconf.h.included zconf.h)
