@@ -31,11 +31,14 @@ os.chdir(_ROOT)
 _DEFAULT_RUNTESTS_TIMEOUT = 1 * 60 * 60
 
 # C/C++ tests can take long time
-_CPP_RUNTESTS_TIMEOUT = 4 * 60 * 60
+_CPP_RUNTESTS_TIMEOUT = 6 * 60 * 60
 
 # Set timeout high for ObjC for Cocoapods to install pods
 _OBJC_RUNTESTS_TIMEOUT = 4 * 60 * 60
 
+# Set timeout high for Ruby for MacOS for slow xcodebuild
+_RUBY_RUNTESTS_TIMEOUT = 2 * 60 * 60
+
 # Number of jobs assigned to each run_tests.py instance
 _DEFAULT_INNER_JOBS = 2
 
@@ -302,17 +305,18 @@ def _create_test_jobs(extra_args=[], inner_jobs=_DEFAULT_INNER_JOBS):
     )
 
     test_jobs += _generate_jobs(
-        languages=["ruby", "php7"],
+        languages=["ruby", "php8"],
         configs=["dbg", "opt"],
         platforms=["linux", "macos"],
         labels=["basictests", "multilang"],
         extra_args=extra_args + ["--report_multi_target"],
         inner_jobs=inner_jobs,
+        timeout_seconds=_RUBY_RUNTESTS_TIMEOUT,
     )
 
     # ARM64 Linux Ruby and PHP tests
     test_jobs += _generate_jobs(
-        languages=["ruby", "php7"],
+        languages=["ruby", "php8"],
         configs=["dbg", "opt"],
         platforms=["linux"],
         arch="arm64",
@@ -358,11 +362,11 @@ def _create_portability_test_jobs(
         # TODO(b/283304471): Tests using OpenSSL's engine APIs were broken and removed
         "gcc10.2_openssl102",
         "gcc10.2_openssl111",
-        "gcc12",
         "gcc12_openssl309",
+        "gcc14",
         "gcc_musl",
         "clang7",
-        "clang18",
+        "clang19",
     ]:
         test_jobs += _generate_jobs(
             languages=["c", "c++"],
@@ -383,7 +387,7 @@ def _create_portability_test_jobs(
         configs=["dbg"],
         platforms=["windows"],
         arch="default",
-        compiler="cmake_ninja_vs2019",
+        compiler="cmake_ninja_vs2022",
         labels=["portability", "corelang"],
         extra_args=extra_args,
         inner_jobs=inner_jobs,
@@ -392,17 +396,17 @@ def _create_portability_test_jobs(
 
     # portability C and C++ on Windows with the "Visual Studio 2022" cmake
     # generator, i.e. not using Ninja (to verify that we can still build with msbuild)
-    test_jobs += _generate_jobs(
-        languages=["c", "c++"],
-        configs=["dbg"],
-        platforms=["windows"],
-        arch="x64",
-        compiler="cmake_vs2022",
-        labels=["portability", "corelang"],
-        extra_args=extra_args,
-        inner_jobs=inner_jobs,
-        timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
-    )
+    # test_jobs += _generate_jobs(
+    #     languages=["c", "c++"],
+    #     configs=["dbg"],
+    #     platforms=["windows"],
+    #     arch="x64",
+    #     compiler="cmake_vs2022",
+    #     labels=["portability", "corelang"],
+    #     extra_args=extra_args,
+    #     inner_jobs=inner_jobs,
+    #     timeout_seconds=_CPP_RUNTESTS_TIMEOUT,
+    # )
 
     # C and C++ with no-exceptions on Linux
     test_jobs += _generate_jobs(
